@@ -237,13 +237,10 @@
 
 #-(or (and darwin x86) win32)
 (deftest stat.2
-  (let* ((stat (sb-posix:stat "/"))
-         (mode (sb-posix::stat-mode stat)))
-    ;; it's logically possible for / to be writeable by others... but
-    ;; if it is, either someone is playing with strange security
-    ;; modules or they want to know about it anyway.
-    (logand mode sb-posix::s-iwoth))
-  0)
+  (eql
+   (sb-posix::stat-mode (sb-posix:stat "/"))
+   (sb-posix::stat-mode (sb-posix:stat (make-pathname :directory '(:absolute :up)))))
+  t)
 
 (deftest stat.3
   (let* ((now (get-universal-time))
@@ -256,16 +253,6 @@
     (< (- atime unix-now) 10))
   t)
 
-#-(or (and darwin x86) win32)
-(deftest stat.4
-  (let* ((stat (sb-posix:stat (make-pathname :directory '(:absolute :up))))
-         (mode (sb-posix::stat-mode stat)))
-    ;; it's logically possible for / to be writeable by others... but
-    ;; if it is, either someone is playing with strange security
-    ;; modules or they want to know about it anyway.
-    (logand mode sb-posix::s-iwoth))
-  0)
-
 ;; Test that stat can take a second argument.
 #-win32
 (deftest stat.5
@@ -423,7 +410,6 @@
         (ignore-errors (sb-posix:unlink name))))
   nil)
 
-#-hpux ; fix: cant handle c-vargs
 (deftest open.error.1
   (handler-case (sb-posix:open *test-directory* sb-posix::o-wronly)
     (sb-posix:syscall-error (c)
@@ -442,7 +428,7 @@
                sb-posix::o-nonblock))
   t)
 
-#-(or hpux win32 netbsd) ; fix: cant handle c-vargs
+#-(or win32 netbsd) ; fix: cant handle c-vargs
 (deftest fcntl.flock.1
     (locally (declare (sb-ext:muffle-conditions sb-ext:compiler-note))
       (let ((flock (make-instance 'sb-posix:flock
@@ -564,7 +550,8 @@
                                         (directory (make-pathname
                                                     :name :wild
                                                     :type :wild
-                                                    :defaults *current-directory*))))
+                                                    :defaults *current-directory*)
+                                                   :resolve-symlinks nil)))
                         #'string<))
         (sb-posix:closedir dir)))
   t)
@@ -666,21 +653,24 @@
   t)
 
 #-(or (and darwin x86) win32)
-(deftest utimes.1
-    (let ((file (merge-pathnames #p"utimes.1" *test-directory*))
-          (atime (random (1- (expt 2 31))))
-          (mtime (random (1- (expt 2 31)))))
-      (with-open-file (stream file
-                       :direction :output
-                       :if-exists :supersede
-                       :if-does-not-exist :create)
-        (princ "Hello, utimes" stream))
-      (sb-posix:utime file atime mtime)
-      (let* ((stat (sb-posix:stat file)))
-        (delete-file file)
-        (list (= (sb-posix:stat-atime stat) atime)
-              (= (sb-posix:stat-mtime stat) mtime))))
-  (t t))
+(macrolet ((test (name posix-fun)
+             `(deftest ,name
+                (let ((file (merge-pathnames #p"utimes.1" *test-directory*))
+                      (atime (random (1- (expt 2 31))))
+                      (mtime (random (1- (expt 2 31)))))
+                  (with-open-file (stream file
+                                          :direction :output
+                                          :if-exists :supersede
+                                          :if-does-not-exist :create)
+                                  (princ "Hello, utimes" stream))
+                  (,posix-fun file atime mtime)
+                  (let* ((stat (sb-posix:stat file)))
+                    (delete-file file)
+                    (list (= (sb-posix:stat-atime stat) atime)
+                          (= (sb-posix:stat-mtime stat) mtime))))
+                (t t))))
+  (test utime.1 sb-posix:utime)
+  (test utimes.1 sb-posix:utimes))
 
 ;; readlink tests.
 #-win32
@@ -697,7 +687,6 @@
   ;; Same thing, but with a very long link target (which doesn't have
   ;; to exist).  This tests the array adjustment in the wrapper,
   ;; provided that the target's length is long enough.
-  #-hpux ; arg2 to readlink is 80, and arg0 is larger than that
   (deftest readlink.2
       (let ((target-pathname (make-pathname
                               :name (make-string 255 :initial-element #\a)
@@ -836,9 +825,8 @@
           (delete-file temp))))
   t "mkstemp-1")
 
-;#-(or win32 sunos hpux)
 ;;;; mkdtemp is unimplemented on at least Solaris 10
-#-(or win32 hpux sunos)
+#-(or win32 sunos)
 ;;; But it is implemented on OpenSolaris 2008.11
 (deftest mkdtemp.1
     (let ((pathname
