@@ -130,13 +130,10 @@
   (stack (make-fop-vector 100) :type simple-vector)
   (name-buffer (vector (make-string  1 :element-type 'character)
                        (make-string 31 :element-type 'base-char)))
-  (deprecated-stuff nil :type list)
-  ;; Sometimes we want to skip over any FOPs with side-effects (like
-  ;; function calls) while executing other FOPs. SKIP-UNTIL will
-  ;; either contain the position where the skipping will stop, or
-  ;; NIL if we're executing normally.
-  (skip-until nil :type (or null fixnum))
-  (print nil :type boolean))
+  (print nil :type boolean)
+  ;; We keep track of partial source info for the input in case
+  ;; loading gets interrupted.
+  (partial-source-info nil :type (or null sb-c::debug-source)))
 (declaim (freeze-type fasl-input))
 
 ;;; Output the current number of semicolons after a fresh-line.
@@ -422,8 +419,7 @@
                `((macrolet
                    ((fasl-input () '(truly-the fasl-input .fasl-input.))
                     (fasl-input-stream () '(%fasl-input-stream (fasl-input)))
-                    (operand-stack () '(%fasl-input-stack (fasl-input)))
-                    (skip-until () '(%fasl-input-skip-until (fasl-input))))
+                    (operand-stack () '(%fasl-input-stack (fasl-input))))
                   ,@(if (null stack-args)
                         forms
                         (with-unique-names (stack ptr)
@@ -713,40 +709,14 @@
                    byte
                    n-operands arg1 arg2 arg3)
            (when (functionp function)
-             (format *trace-output* " ~35t~(~a~)" (%fun-name function))))
-         (let ((stack (%fasl-input-stack fasl-input)))
-           (declare (ignorable stack)) ; not used in xc-host
-           (when (and (eq byte #.(get 'fop-funcall-for-effect 'opcode))
-                      (fop-stack-empty-p stack)) ; (presumed) end of TLF
-             (awhen (%fasl-input-deprecated-stuff fasl-input)
-               ;; Delaying this message rather than printing it
-               ;; in fop-fdefn makes it more informative (usually).
-               (setf (%fasl-input-deprecated-stuff fasl-input) nil)
-               (loader-deprecation-warn
-                it
-                (and (eq (svref stack 1) 'sb-impl::%defun) (svref stack 2)))))))))))
-
-;; This is the moral equivalent of a warning from /usr/bin/ld that
-;; "gets() is dangerous." You're informed by both the compiler and linker.
-(defun loader-deprecation-warn (stuff whence)
-  ;; Stuff is a list: ((<state> name . category) ...)
-  ;; For now we only deal with category = :FUNCTION so we ignore it.
-  (let ((warning-class
-         ;; We're only going to warn once (per toplevel form),
-         ;; so pick the most stern warning applicable.
-         (if (every (lambda (x) (eq (car x) :early)) stuff)
-             'simple-style-warning 'simple-warning)))
-    (warn warning-class
-          :format-control "Reference to deprecated function~P ~S~@[ from ~S~]"
-          :format-arguments
-          (list (length stuff) (mapcar #'second stuff) whence))))
+             (format *trace-output* " ~35t~(~a~)" (%fun-name function)))))))))
 
 (defun load-as-fasl (stream verbose print)
   (when (zerop (file-length stream))
     (error "attempt to load an empty FASL file:~%  ~S" (namestring stream)))
   (maybe-announce-load stream verbose)
   (let ((fasl-input (make-fasl-input stream print)))
-    (with-deferred-package-names
+    (with-loader-package-names
       (unwind-protect
            (loop while (load-fasl-group fasl-input))
         ;; Nuke the table and stack to avoid keeping garbage on
@@ -789,7 +759,8 @@
         (n-data-words (- size sb-vm:instance-data-start)))
     (with-fop-stack ((stack (operand-stack)) ptr n-data-words)
       (declare (type index ptr))
-      (let ((bitmap (wrapper-bitmap layout)))
+      ;; FIXME: this is basically DO-LAYOUT-BITMAP, but probably not as efficient.
+      (let ((bitmap (sb-kernel::%layout-bitmap layout)))
         ;; Values on the stack are in the same order as in the structure itself.
         (do ((i sb-vm:instance-data-start (1+ i)))
             ((>= i size))
@@ -865,10 +836,10 @@
                           'read-char-string-as-varints)
                       (%fasl-input-stream fasl-input) string namelen)
              (values string namelen elt-type)))
-         (aux-fop-intern (length+flag package fasl-input)
+         (aux-fop-intern (length+flag package inherited fasl-input)
            (multiple-value-bind (name length elt-type)
                (read-symbol-name length+flag fasl-input)
-             (push-fop-table (%intern name length package elt-type t)
+             (push-fop-table (%intern name length package elt-type t inherited)
                              fasl-input)))
          (ensure-hashed (symbol)
            ;; ENSURE-SYMBOL-HASH when vop-translated is flushable since it is
@@ -882,11 +853,13 @@
            symbol))
 
   (define-fop 77 :not-host (fop-lisp-symbol-save ((:operands length+flag)))
-    (aux-fop-intern length+flag *cl-package* (fasl-input)))
+    (aux-fop-intern length+flag *cl-package* t (fasl-input)))
   (define-fop 78 :not-host (fop-keyword-symbol-save ((:operands length+flag)))
-    (aux-fop-intern length+flag *keyword-package* (fasl-input)))
+    (aux-fop-intern length+flag *keyword-package* t (fasl-input)))
   (define-fop 79 :not-host (fop-symbol-in-package-save ((:operands length+flag pkg-index)))
-    (aux-fop-intern length+flag (ref-fop-table (fasl-input) pkg-index) (fasl-input)))
+    (aux-fop-intern length+flag (ref-fop-table (fasl-input) pkg-index) t (fasl-input)))
+  (define-fop 84 :not-host (fop-symbol-in-package-internal-save ((:operands length+flag pkg-index)))
+    (aux-fop-intern length+flag (ref-fop-table (fasl-input) pkg-index) nil (fasl-input)))
 
   (define-fop 80 :not-host (fop-uninterned-symbol-save ((:operands length+flag)))
     (multiple-value-bind (name len) (read-symbol-name length+flag (fasl-input))
@@ -904,12 +877,8 @@
 (define-fop 83 :not-host (fop-named-package-save ((:operands length)) nil)
   (let ((package-name (make-string length)))
     (read-char-string-as-varints (fasl-input-stream) package-name)
-    (push-fop-table
-     (handler-case (find-undeleted-package-or-lose package-name)
-       (simple-package-error (condition)
-         (declare (ignore condition))
-         (make-deferred-package package-name)))
-     (fasl-input))))
+    (push-fop-table (find-or-maybe-make-deferred-package package-name)
+                    (fasl-input))))
 
 ;;;; fops for loading numbers
 
@@ -1064,24 +1033,21 @@
     (read-n-bytes (fasl-input-stream) vector 0 bytes)
     vector))
 
-(defun fop-funcall* (argc stack skipping)
+(defun fop-funcall* (argc stack)
   (with-fop-stack ((stack) ptr (1+ argc))
-    (unless skipping
-      (do ((i (+ ptr argc))
-           (args))
-          ((= i ptr) (apply (fop-stack-ref i) args))
-        (declare (type index i))
-        (push (fop-stack-ref i) args)
-        (decf i)))))
+    (if (zerop argc)
+        (funcall (fop-stack-ref ptr))
+        (do ((i (+ ptr argc))
+             (args))
+            ((= i ptr) (apply (fop-stack-ref i) args))
+          (declare (type index i))
+          (push (fop-stack-ref i) args)
+          (decf i)))))
 
 (define-fop 55 (fop-funcall ((:operands n)))
-  (fop-funcall* n (operand-stack) (skip-until)))
+  (fop-funcall* n (operand-stack)))
 (define-fop 56 (fop-funcall-for-effect ((:operands n)) nil)
-  (fop-funcall* n (operand-stack) (skip-until)))
-
-;;; For LOAD-TIME-VALUE which is used for MAKE-LOAD-FORM
-(define-fop 57 (fop-funcall-no-skip ((:operands n)))
-  (fop-funcall* n (operand-stack) nil))
+  (fop-funcall* n (operand-stack)))
 
 ;;;; fops for fixing up circularities
 
@@ -1153,6 +1119,8 @@
           ;; Assign debug-info last. A code object that has no debug-info will never
           ;; have its fun table accessed in conservative_root_p() or pin_object().
           (setf (%code-debug-info code) (svref stack (+ ptr n-constants)))
+          (setf (sb-c::debug-info-source (%code-debug-info code))
+                (%fasl-input-partial-source-info (fasl-input)))
           ;; Boxed constants can be assigned only after figuring out where the range
           ;; of implicitly tagged words is, which requires knowing how many functions
           ;; are in the code component, which requires reading the code trailer.
@@ -1187,9 +1155,11 @@
 ;; this gets you an #<fdefn> object, not the result of (FDEFINITION x)
 ;; cold-loader uses COLD-FDEFINITION-OBJECT instead.
 (define-fop 18 :not-host (fop-fdefn (name))
-  (awhen (deprecated-thing-p 'function name) ; returns the stage of deprecation
-    (pushnew (list* it name :function)
-             (%fasl-input-deprecated-stuff (fasl-input)) :test 'equal))
+  (when (deprecated-thing-p 'function name)
+    ;; This is the moral equivalent of a warning from /usr/bin/ld
+    ;; that "gets() is dangerous." You're informed by both the
+    ;; compiler and linker.
+    (check-deprecated-thing 'function name))
   (find-or-create-fdefn name))
 
 (define-fop 19 :not-host (fop-known-fun (name))
@@ -1245,42 +1215,13 @@
 (define-fop 22 (fop-assembler-code)
   (error "cannot load assembler code except at cold load"))
 
-;;; FOPs needed for implementing an IF operator in a FASL
+;;;; fops for debug info
 
-;;; Skip until a FOP-MAYBE-STOP-SKIPPING with the same POSITION is
-;;; executed. While skipping, we execute most FOPs normally, except
-;;; for ones that a) funcall/eval b) start skipping. This needs to
-;;; be done to ensure that the fop table gets populated correctly
-;;; regardless of the execution path.
-(define-fop 6 (fop-skip ((:operands position)) nil)
-  (unless (skip-until)
-    (setf (skip-until) position))
-  (values))
-
-;;; As before, but only start skipping if the top of the FOP stack is NIL.
-(define-fop 7 (fop-skip-if-false ((:operands position) condition) nil)
-  (unless (or condition (skip-until))
-    (setf (skip-until) position))
-  (values))
-
-;;; If skipping, pop the top of the stack and discard it. Needed for
-;;; ensuring that the stack stays balanced when skipping.
-(define-fop 8 (fop-drop-if-skipping () nil)
-  (when (skip-until)
-    (fop-stack-pop-n (operand-stack) 1))
-  (values))
-
-;;; If skipping, push a dummy value on the stack. Needed for
-;;; ensuring that the stack stays balanced when skipping.
-(define-fop 9 (fop-push-nil-if-skipping () nil)
-  (when (skip-until)
-    (push-fop-stack nil (fasl-input)))
-  (values))
-
-;;; Stop skipping if the top of the stack matches SKIP-UNTIL
-(define-fop 10 (fop-maybe-stop-skipping ((:operands label)) nil)
-  (when (eql (skip-until) label)
-    (setf (skip-until) nil))
+(define-fop 124 (fop-note-partial-source-info (namestring created plist) nil)
+  (setf (%fasl-input-partial-source-info (fasl-input))
+        (sb-c::make-debug-source :namestring namestring
+                                 :created created
+                                 :plist plist))
   (values))
 
 ;;;; fops for code coverage
