@@ -45,8 +45,6 @@
                           complex-array-widetag))
         (setf (svref infos code) info)))
 
-    (setf (svref infos filler-widetag) (make-room-info 'filler))
-
     (dotimes (i (length *specialized-array-element-type-properties*))
       (let ((saetp (aref *specialized-array-element-type-properties* i)))
         (setf (svref infos (saetp-typecode saetp)) saetp)))
@@ -189,21 +187,31 @@
         (end (descriptor-sap end)))
     (loop
      (if (sap>= start end) (return))
-     (binding* ((widetag (widetag@baseptr start))
-                (obj (lispobj@baseptr start widetag))
-                ((typecode size)
+     (let ((word (sap-ref-word start 0)))
+       (cond
+         ((= (logand word widetag-mask) filler-widetag) ; pseudo-object
+          (let ((size (+ (* (ash word -8) n-word-bytes) n-word-bytes)))
+            (setq start (sap+ start size))))
+         ((= word most-positive-word)
+          ;; has to be a pseudo-cons resulting from removing an insignificant
+          ;; sign word of a bignum. Don't call FUN
+          (setq start (sap+ start (* 2 n-word-bytes))))
+         (t
+          (binding*
+              ((widetag (widetag@baseptr start))
+               (obj (lispobj@baseptr start widetag))
+               ((typecode size)
                  ;; PRIMITIVE-OBJECT-SIZE works on conses, but they're exceptions already
                  ;; because of absence of a widetag, so may as well not call the sizer.
                  (if (listp obj)
                      (values list-pointer-lowtag (* 2 n-word-bytes))
                      (values widetag (primitive-object-size obj)))))
-       ;; SIZE is surely a fixnum. Non-fixnum would imply at least
-       ;; a 512MB object if 32-bit words, and is inconceivable if 64-bit.
-       ;; But check to be sure.
-       (aver (not (logtest (the fixnum size) lowtag-mask)))
-       (unless (= typecode filler-widetag)
-         (funcall fun obj typecode size))
-       (setq start (sap+ start size))))
+              ;; SIZE is surely a fixnum. Non-fixnum would imply at least
+              ;; a 512MB object if 32-bit words, and is inconceivable if 64-bit.
+              ;; But check to be sure.
+              (aver (not (logtest (the fixnum size) lowtag-mask)))
+              (funcall fun obj typecode size)
+              (setq start (sap+ start size)))))))
     (when strict-bound
      ;; If START is not eq to END, then we have blown past our endpoint.
       #+sb-devel
@@ -217,27 +225,8 @@
 
 ;;; Access to the GENCGC page table for better precision in
 ;;; MAP-ALLOCATED-OBJECTS
-#+gencgc
+#+immobile-space
 (progn
-  (define-alien-type (struct page)
-      (struct page
-              ;; To cut down the size of the page table, the scan_start_offset
-              ;; - a/k/a "start" - is measured in 4-byte integers regardless
-              ;; of word size. This is fine for 32-bit address space,
-              ;; but if 64-bit then we have to scale the value. Additionally
-              ;; there is a fallback for when even the scaled value is too big.
-              (start #+64-bit (unsigned 32) #-64-bit signed)
-              ;; On platforms with small enough GC pages, this field
-              ;; will be a short. On platforms with larger ones, it'll
-              ;; be an int. It should probably never be an int.
-              (words-used (unsigned
-                           #.(if (typep gencgc-page-words '(unsigned-byte 16))
-                                 16
-                                 32)))
-              (flags (unsigned 8)) ; in C this is {type, need_zerofill, pinned}
-              (gen (signed 8))))
-  #+immobile-space
-  (progn
     (define-alien-type (struct immobile-page)
         ;; ... and yet another place for Lisp to become out-of-sync with C.
         (struct immobile-page
@@ -249,11 +238,7 @@
                 (page-link (unsigned 16))
                 (prior-free-index (unsigned 16))))
     (define-alien-variable "fixedobj_pages" (* (struct immobile-page))))
-  (declaim (inline find-page-index))
-  (define-alien-routine ("ext_find_page_index" find-page-index)
-    long (index unsigned))
-  (define-alien-variable "next_free_page" sb-kernel::page-index-t)
-  (define-alien-variable "page_table" (* (struct page))))
+(define-alien-variable "next_free_page" sb-kernel::page-index-t)
 
 #+immobile-space
 (progn
@@ -431,6 +416,8 @@ We could try a few things to mitigate this:
           (when (and (logbitp (logand (slot (deref page-table start-page) 'gen) 7)
                               generation-mask)
                      (= (logand flags page-type-mask) page-type-constraint))
+            ;; FIXME: should exclude (0 . 0) conses on PAGE_TYPE_{BOXED,UNBOXED}
+            ;; resulting from zeroing the tail of a bignum or vector etc.
             (map-objects-in-range fun
                                   (%make-lisp-obj (sap-int start))
                                   (%make-lisp-obj (sap-int end))
@@ -913,9 +900,13 @@ We could try a few things to mitigate this:
          ;; These two are in fact generally the most frequently occurring type.
          ,.(make-case 'cons `(car ,obj) `(cdr ,obj))
          ,.(make-case* 'instance
-            `(progn
+            ;; %INSTANCE-LAYOUT is defknown'ed to return a LAYOUT,
+            ;; but heap walking might encounter an instance with no layout,
+            ;; hence the need to access the slot opaquely.
+            `(unless (eql 0 #+compact-instance-header (%primitive %instance-layout ,obj)
+                            #-compact-instance-header (%instance-ref ,obj 0))
                (,functoid (%instance-layout ,obj) ,@more)
-               (do-instance-tagged-slot (.i. ,obj nil)
+               (do-instance-tagged-slot (.i. ,obj)
                  (,functoid (%instance-ref ,obj .i.) ,@more))))
          (function
           (typecase ,obj
@@ -1036,7 +1027,6 @@ We could try a few things to mitigate this:
           (case (widetag-of this)
             (#.value-cell-widetag
              (test (value-cell-ref this)))
-            (#.filler-widetag)
             (t
              (bug "Unknown object type #x~x addr=~x"
                   (widetag-of this)
@@ -1195,21 +1185,6 @@ We could try a few things to mitigate this:
         (%make-lisp-obj varyobj-space-start)
         (%make-lisp-obj (sap-int *varyobj-space-free-pointer*))))))
 
-#+gencgc
-(defun generation-of (object)
-  (with-pinned-objects (object)
-    (let* ((addr (get-lisp-obj-address object))
-           (page (find-page-index addr)))
-      (cond ((>= page 0) (slot (deref page-table page) 'gen))
-            #+immobile-space
-            ((immobile-space-addr-p addr)
-             ;; SIMPLE-FUNs don't contain a generation byte
-             (when (simple-fun-p object)
-               (setq addr (get-lisp-obj-address (fun-code-header object))))
-             (let ((sap (int-sap (logandc2 addr lowtag-mask))))
-               (logand (if (fdefn-p object) (sap-ref-8 sap 1) (sap-ref-8 sap 3))
-                       #xF)))))))
-
 ;;; Show objects in a much simpler way than print-allocated-objects.
 ;;; Probably don't use this for generation 0 of dynamic space. Other spaces are ok.
 ;;; (And this is removed from the image; it's meant for developers)
