@@ -53,7 +53,7 @@
       (set-result (+ result (ash result 3)))
       (set-result (logxor result (ash result -11)))
       (set-result (logxor result (ash result 15)))
-      (logand result sb-xc:most-positive-fixnum))))
+      (logand result most-positive-fixnum))))
 ;;; test:
 ;;;   (let ((ht (make-hash-table :test 'equal)))
 ;;;     (do-all-symbols (symbol)
@@ -122,10 +122,10 @@
   ;; search starting from 2^60*pi.  The multiplication should be
   ;; efficient no matter what the platform thanks to modular
   ;; arithmetic.
-  (let* ((mul (logand 3622009729038463111 sb-xc:most-positive-fixnum))
-         (xor (logand 608948948376289905 sb-xc:most-positive-fixnum))
-         (xy (logand (+ (* x mul) y) sb-xc:most-positive-fixnum)))
-    (logand (logxor xor xy (ash xy -5)) sb-xc:most-positive-fixnum)))
+  (let* ((mul (logand 3622009729038463111 most-positive-fixnum))
+         (xor (logand 608948948376289905 most-positive-fixnum))
+         (xy (logand (+ (* x mul) y) most-positive-fixnum)))
+    (logand (logxor xor xy (ash xy -5)) most-positive-fixnum)))
 
 ;;; Same as above, but don't mask computations to n-positive-fixnum-bits.
 (declaim (inline word-mix))
@@ -146,9 +146,9 @@
 (defun murmur3-fmix32 (h)
   (declare (type (unsigned-byte 32) h))
   (setq h (logxor h (ash h -16)))
-  (setq h (logand (* h #x85ebca6b) most-positive-word))
+  (setq h (logand (* h #x85ebca6b) #.most-positive-word))
   (setq h (logxor h  (ash h -13)))
-  (setq h (logand (* h #xc2b2ae35) most-positive-word))
+  (setq h (logand (* h #xc2b2ae35) #.most-positive-word))
   (logxor h (ash h -16))))
 
 #+64-bit
@@ -185,15 +185,14 @@
 ;;;   an extra right-shift to remove bits that lack any randomness. CACHE-MASK can be
 ;;;   reworked to examine bits other than at the low end.
 (defun hash-layout-name (name)
-  (let ((limit (1+ (ash sb-xc:most-positive-fixnum -1)))
-        (random-state (load-time-value (make-random-state))))
+  (let ((limit (1+ (ash most-positive-fixnum -1))))
     (declare (notinline random))
     (logior (if (typep name '(and symbol (not null)))
                 (flet ((improve-hash (x)
                          (#+64-bit murmur3-fmix64 #-64-bit murmur3-fmix32 x)))
                   (mix (logand
                         (improve-hash (sb-impl::%sxhash-simple-string (symbol-name name)))
-                        sb-xc:most-positive-fixnum)
+                        most-positive-fixnum)
                        (let ((package (sb-xc:symbol-package name)))
                          (sb-impl::%sxhash-simple-string
                           ;; Must specifically look for CL package when cross-compiling
@@ -203,5 +202,16 @@
                           (cond #+sb-xc ((eq package *cl-package*) "COMMON-LISP")
                                 ((not package) "uninterned")
                                 (t (package-name package)))))))
-                (random (ash limit -1) random-state))
+                ;; This L-T-V form has to remain out of the common path,
+                ;; or else cheneygc will crash in cold-init.
+                ;; Cold-init calls HASH-LAYOUT-NAME many times *before* the L-T-V
+                ;; is actually executed and stuffed in as a constant.
+                ;; That's harmless - the loader put an unbound marker there, and we don't care.
+                ;; *HOWEVER* there is an interesting issue that arises if the value-cell
+                ;; indirection is present (see the IR1 translator for L-T-V and the
+                ;; conditional code for cheneygc): when can the compiler dereference the
+                ;; value-cell? If the binding were earlier (before the IF), then the dereference
+                ;; would happen earlier, and crash, because unbound-marker isn't a pointer.
+                (let ((random-state (load-time-value (make-random-state))))
+                  (random (ash limit -1) random-state)))
             limit)))
