@@ -27,13 +27,13 @@
   (let ((word
          ;; threads imply gencgc. use the per-thread alloc region pointer
          #+sb-thread
-         (sap-int (sb-vm::current-thread-offset-sap sb-vm::thread-boxed-tlab-slot))
+         (sap-int (sb-vm::current-thread-offset-sap sb-vm::thread-mixed-tlab-slot))
          #+(and (not sb-thread) cheneygc)
          (sap-int (dynamic-space-free-pointer))
          ;; dynamic-space-free-pointer increments only when a page is full.
-         ;; Using boxed_region directly is finer-grained.
+         ;; Using mixed_region directly is finer-grained.
          #+(and (not sb-thread) gencgc)
-         (sb-sys:sap-ref-word (sb-sys:int-sap sb-vm::boxed-region) 0)))
+         (sb-sys:sap-ref-word (sb-sys:int-sap sb-vm::mixed-region) 0)))
     ;; counter should increase by 1 for each cons cell allocated
     (ash word (- (1+ sb-vm:word-shift)))))
 
@@ -380,7 +380,7 @@
   (declare (optimize speed))
   (labels
       ((data-vector-hash (data start end depthoid)
-         (declare (optimize (sb-c::insert-array-bounds-checks 0)))
+         (declare (optimize (sb-c:insert-array-bounds-checks 0)))
          (let ((result 572539))
            (declare (type hash-code result))
            (when (plusp depthoid)
@@ -590,6 +590,18 @@
                              (recurse (cdr x) (1- depthoid)))))))
       (traverse 0 name 10))))
 
+;;; These "good" hashers act on sb-vm:word, returning a fixnum, do not cons,
+;;; and have better avalanche behavior then SXHASH - changing any one input bit
+;;; should affect each bit of output with equal chance.
+#+64-bit
+(defun good-hash-word->fixnum (x)
+  (declare (inline murmur3-fmix64))
+  (logand (murmur3-fmix64 (truly-the sb-vm:word x)) most-positive-fixnum))
+#-64-bit
+(defun good-hash-word->fixnum (x)
+  (declare (inline murmur3-fmix32))
+  (logand (murmur3-fmix32 (truly-the sb-vm:word x)) most-positive-fixnum))
+
 ;;; Not needed post-build
 (clear-info :function :inlining-data '%sxhash-simple-substring)
 
