@@ -147,14 +147,11 @@
   ;; a string designator for the package name
   (name (error "missing PACKAGE-DATA-NAME datum"))
   ;; a doc string
-  (doc (error "missing PACKAGE-DOC datum"))
+  (documentation (error "missing PACKAGE-DOCUMENATION datum"))
   ;; a list of string designators for shadowing symbols
   shadow
-  ;; a tree containing names for exported symbols which'll be set up at package
-  ;; creation time, and NILs, which are ignored. (This is a tree in order to
-  ;; allow constructs like '("ENOSPC" #+LINUX ("EDQUOT" "EISNAM" "ENAVAIL"
-  ;; "EREMOTEIO")) to be used in initialization. NIL entries in the tree are
-  ;; ignored for the same reason of notational convenience.)
+  ;; a list of string designators for exported symbols which'll be set
+  ;; up at package creation time.
   export
   ;; a list of string designators for exported symbols which don't necessarily
   ;; originate in this package (so their EXPORT operations should be handled
@@ -165,7 +162,7 @@
   ;; package to import from, and the remaining elements are the names of
   ;; symbols to import.
   import-from
-  ;; a tree of string designators for package names of other packages
+  ;; a list of string designators for package names of other packages
   ;; which this package uses
   use)
 
@@ -201,7 +198,101 @@
     ;;
     "BYTE" "BYTE-POSITION" "BYTE-SIZE"
     "DPB" "LDB" "LDB-TEST"
-    "DEPOSIT-FIELD" "MASK-FIELD"))
+    "DEPOSIT-FIELD" "MASK-FIELD"
+    ;;
+    ;; the constants (except for T and NIL which have
+    ;; a specially hacked correspondence between
+    ;; cross-compilation host Lisp and target Lisp)
+
+    ;; We include these here since there is no reason to reference
+    ;; host constants.
+    "ARRAY-DIMENSION-LIMIT"
+    "ARRAY-RANK-LIMIT"
+    "ARRAY-TOTAL-SIZE-LIMIT"
+    "BOOLE-1"
+    "BOOLE-2"
+    "BOOLE-AND"
+    "BOOLE-ANDC1"
+    "BOOLE-ANDC2"
+    "BOOLE-C1"
+    "BOOLE-C2"
+    "BOOLE-CLR"
+    "BOOLE-EQV"
+    "BOOLE-IOR"
+    "BOOLE-NAND"
+    "BOOLE-NOR"
+    "BOOLE-ORC1"
+    "BOOLE-ORC2"
+    "BOOLE-SET"
+    "BOOLE-XOR"
+    "CALL-ARGUMENTS-LIMIT"
+    "CHAR-CODE-LIMIT"
+    "DOUBLE-FLOAT-EPSILON"
+    "DOUBLE-FLOAT-NEGATIVE-EPSILON"
+    "INTERNAL-TIME-UNITS-PER-SECOND"
+    "LAMBDA-LIST-KEYWORDS"
+    "LAMBDA-PARAMETERS-LIMIT"
+    "LEAST-NEGATIVE-DOUBLE-FLOAT"
+    "LEAST-NEGATIVE-LONG-FLOAT"
+    "LEAST-NEGATIVE-NORMALIZED-DOUBLE-FLOAT"
+    "LEAST-NEGATIVE-NORMALIZED-LONG-FLOAT"
+    "LEAST-NEGATIVE-NORMALIZED-SHORT-FLOAT"
+    "LEAST-NEGATIVE-NORMALIZED-SINGLE-FLOAT"
+    "LEAST-NEGATIVE-SHORT-FLOAT"
+    "LEAST-NEGATIVE-SINGLE-FLOAT"
+    "LEAST-POSITIVE-DOUBLE-FLOAT"
+    "LEAST-POSITIVE-LONG-FLOAT"
+    "LEAST-POSITIVE-NORMALIZED-DOUBLE-FLOAT"
+    "LEAST-POSITIVE-NORMALIZED-LONG-FLOAT"
+    "LEAST-POSITIVE-NORMALIZED-SHORT-FLOAT"
+    "LEAST-POSITIVE-NORMALIZED-SINGLE-FLOAT"
+    "LEAST-POSITIVE-SHORT-FLOAT"
+    "LEAST-POSITIVE-SINGLE-FLOAT"
+    "LONG-FLOAT-EPSILON"
+    "LONG-FLOAT-NEGATIVE-EPSILON"
+    "MOST-NEGATIVE-DOUBLE-FLOAT"
+    "MOST-NEGATIVE-FIXNUM"
+    "MOST-NEGATIVE-LONG-FLOAT"
+    "MOST-NEGATIVE-SHORT-FLOAT"
+    "MOST-NEGATIVE-SINGLE-FLOAT"
+    "MOST-POSITIVE-DOUBLE-FLOAT"
+    "MOST-POSITIVE-FIXNUM"
+    "MOST-POSITIVE-LONG-FLOAT"
+    "MOST-POSITIVE-SHORT-FLOAT"
+    "MOST-POSITIVE-SINGLE-FLOAT"
+    "MULTIPLE-VALUES-LIMIT"
+    "PI"
+    "SHORT-FLOAT-EPSILON"
+    "SHORT-FLOAT-NEGATIVE-EPSILON"
+    "SINGLE-FLOAT-EPSILON"
+    "SINGLE-FLOAT-NEGATIVE-EPSILON"
+
+    ;; The cross-compiler itself shouldn't really need to use the host
+    ;; versions of these in target code except in exceptional cases.
+    "CHAR-CODE"
+    "CODE-CHAR"
+    "COMPILE-FILE"
+    "COMPILE-FILE-PATHNAME"
+    "*COMPILE-FILE-PATHNAME*"
+    "*COMPILE-FILE-TRUENAME*"
+    "*COMPILE-PRINT*"
+    "*COMPILE-VERBOSE*"
+    "COMPILER-MACRO-FUNCTION"
+    "CONSTANTP"
+    "GET-SETF-EXPANSION"
+    "*GENSYM-COUNTER*"
+    "LISP-IMPLEMENTATION-TYPE" "LISP-IMPLEMENTATION-VERSION"
+    "MACRO-FUNCTION"
+    "MACROEXPAND" "MACROEXPAND-1" "*MACROEXPAND-HOOK*"
+    "MAKE-LOAD-FORM"
+    "MAKE-LOAD-FORM-SAVING-SLOTS"
+    "PROCLAIM"
+    "SPECIAL-OPERATOR-P"
+    "SUBTYPEP"
+    "UPGRADED-ARRAY-ELEMENT-TYPE"
+    "UPGRADED-COMPLEX-PART-TYPE"
+    "WITH-COMPILATION-UNIT"
+    ))
 
 ;;; A symbol in the "dual personality" list refers to the symbol in CL unless
 ;;; package-prefixed with SB-XC:.  The main reason for not putting these
@@ -251,105 +342,17 @@
 ;; compilation of the target.
 ;;
 (let ((package-name "SB-XC"))
-  (dolist (name (append (append *undefineds* *dual-personality-math-symbols*)))
+  (dolist (name (append *undefineds* *dual-personality-math-symbols*))
     (export (intern name package-name) package-name))
-  (dolist (name '(;; the constants (except for T and NIL which have
-                  ;; a specially hacked correspondence between
-                  ;; cross-compilation host Lisp and target Lisp)
-                  "ARRAY-DIMENSION-LIMIT"
-                  "ARRAY-RANK-LIMIT"
-                  "ARRAY-TOTAL-SIZE-LIMIT"
-                  "BOOLE-1"
-                  "BOOLE-2"
-                  "BOOLE-AND"
-                  "BOOLE-ANDC1"
-                  "BOOLE-ANDC2"
-                  "BOOLE-C1"
-                  "BOOLE-C2"
-                  "BOOLE-CLR"
-                  "BOOLE-EQV"
-                  "BOOLE-IOR"
-                  "BOOLE-NAND"
-                  "BOOLE-NOR"
-                  "BOOLE-ORC1"
-                  "BOOLE-ORC2"
-                  "BOOLE-SET"
-                  "BOOLE-XOR"
-                  "CALL-ARGUMENTS-LIMIT"
-                  "CHAR-CODE-LIMIT"
-                  "DOUBLE-FLOAT-EPSILON"
-                  "DOUBLE-FLOAT-NEGATIVE-EPSILON"
-                  "INTERNAL-TIME-UNITS-PER-SECOND"
-                  "LAMBDA-LIST-KEYWORDS"
-                  "LAMBDA-PARAMETERS-LIMIT"
-                  "LEAST-NEGATIVE-DOUBLE-FLOAT"
-                  "LEAST-NEGATIVE-LONG-FLOAT"
-                  "LEAST-NEGATIVE-NORMALIZED-DOUBLE-FLOAT"
-                  "LEAST-NEGATIVE-NORMALIZED-LONG-FLOAT"
-                  "LEAST-NEGATIVE-NORMALIZED-SHORT-FLOAT"
-                  "LEAST-NEGATIVE-NORMALIZED-SINGLE-FLOAT"
-                  "LEAST-NEGATIVE-SHORT-FLOAT"
-                  "LEAST-NEGATIVE-SINGLE-FLOAT"
-                  "LEAST-POSITIVE-DOUBLE-FLOAT"
-                  "LEAST-POSITIVE-LONG-FLOAT"
-                  "LEAST-POSITIVE-NORMALIZED-DOUBLE-FLOAT"
-                  "LEAST-POSITIVE-NORMALIZED-LONG-FLOAT"
-                  "LEAST-POSITIVE-NORMALIZED-SHORT-FLOAT"
-                  "LEAST-POSITIVE-NORMALIZED-SINGLE-FLOAT"
-                  "LEAST-POSITIVE-SHORT-FLOAT"
-                  "LEAST-POSITIVE-SINGLE-FLOAT"
-                  "LONG-FLOAT-EPSILON"
-                  "LONG-FLOAT-NEGATIVE-EPSILON"
-                  "MOST-NEGATIVE-DOUBLE-FLOAT"
-                  "MOST-NEGATIVE-FIXNUM"
-                  "MOST-NEGATIVE-LONG-FLOAT"
-                  "MOST-NEGATIVE-SHORT-FLOAT"
-                  "MOST-NEGATIVE-SINGLE-FLOAT"
-                  "MOST-POSITIVE-DOUBLE-FLOAT"
-                  "MOST-POSITIVE-FIXNUM"
-                  "MOST-POSITIVE-LONG-FLOAT"
-                  "MOST-POSITIVE-SHORT-FLOAT"
-                  "MOST-POSITIVE-SINGLE-FLOAT"
-                  "MULTIPLE-VALUES-LIMIT"
-                  "PI"
-                  "SHORT-FLOAT-EPSILON"
-                  "SHORT-FLOAT-NEGATIVE-EPSILON"
-                  "SINGLE-FLOAT-EPSILON"
-                  "SINGLE-FLOAT-NEGATIVE-EPSILON"
-                  "*READ-DEFAULT-FLOAT-FORMAT*"
-
+  (dolist (name '("*READ-DEFAULT-FLOAT-FORMAT*"
                   "ARRAY-ELEMENT-TYPE"
-                  "CHAR-CODE"
-                  "CODE-CHAR"
-                  "COMPILE-FILE"
-                  "COMPILE-FILE-PATHNAME"
-                  "*COMPILE-FILE-PATHNAME*"
-                  "*COMPILE-FILE-TRUENAME*"
-                  "*COMPILE-PRINT*"
-                  "*COMPILE-VERBOSE*"
-                  "COMPILER-MACRO-FUNCTION"
-                  "CONSTANTP"
-                  "DEFCONSTANT"
-                  "DEFINE-MODIFY-MACRO"
-                  "DEFINE-SETF-EXPANDER"
-                  "DEFMACRO" "DEFSETF" "DEFSTRUCT" "DEFTYPE"
-                  "GENSYM" "*GENSYM-COUNTER*"
-                  "GET-SETF-EXPANSION"
-                  "LISP-IMPLEMENTATION-TYPE" "LISP-IMPLEMENTATION-VERSION"
-                  "MACRO-FUNCTION"
-                  "MACROEXPAND" "MACROEXPAND-1" "*MACROEXPAND-HOOK*"
+                  "DEFMACRO" "DEFSTRUCT" "DEFTYPE"
+                  "GENSYM"
                   "MAKE-ARRAY"
-                  "MAKE-LOAD-FORM"
-                  "MAKE-LOAD-FORM-SAVING-SLOTS"
-                  "PROCLAIM"
                   "SIMPLE-VECTOR"
-                  "SPECIAL-OPERATOR-P"
-                  "SUBTYPEP"
-                  "TYPE-OF" "TYPEP"
-                  "UPGRADED-ARRAY-ELEMENT-TYPE"
-                  "UPGRADED-COMPLEX-PART-TYPE"
-                  "WITH-COMPILATION-UNIT"))
-      (export (intern name package-name) package-name)))
+                  "TYPEP"
+                  ))
+    (export (intern name package-name) package-name)))
 
 (defun count-symbols (pkg)
   (let ((n 0))
@@ -359,7 +362,7 @@
 
 ;;; Build a new package that exports a not-necessarily-strict subset of
 ;;; what the host CL exports. This deals with hosts that have too many
-;;; symbols exported froM CL.
+;;; symbols exported from CL.
 (let ((cl-model-package (make-package "XC-STRICT-CL" :use nil)))
   (flet ((new-external (x package &aux (s (intern x package)))
            (export s package)
@@ -368,7 +371,7 @@
            (import x cl-model-package)
            (export x cl-model-package)))
     (reexport (list nil))
-    (dolist (string (read-from-file "common-lisp-exports.lisp-expr"))
+    (dolist (string (read-from-file "^common-lisp-exports.lisp-expr"))
       (unless (string= string "NIL") ; already done
         (cond ((member string *undefineds* :test #'string=)
                (new-external string cl-model-package))
@@ -391,94 +394,103 @@
   (assert (equal *package-symbol-counts* (compute-cl-package-symbol-counts))))
 
 (defun create-target-packages (package-data-list)
-  (labels ((flatten (tree)
-             (let ((result (mapcan (lambda (x) (if (listp x) (flatten x) (list x)))
-                                   tree)))
-               (when (< (length (remove-duplicates result :test 'equal))
-                        (length result))
-                 (error "Duplicates in package-data-list: ~a~%"
-                        (mapcon (lambda (x)
-                                  (when (member (car x) (cdr x) :test 'equal)
-                                    (list (car x))))
-                                result)))
-               result)))
+  (hide-host-packages)
 
-    (hide-host-packages)
+  ;; Build all packages that we need, and initialize them as far as we
+  ;; can without referring to any other packages.
+  (dolist (package-data package-data-list)
+    (let ((package (make-package (package-data-name package-data) :use nil)))
+      (dolist (string (package-data-shadow package-data))
+        (shadow string package))
+      (dolist (string (package-data-export package-data))
+        (export (intern string package) package))))
+  ;; Now that all packages exist, we can set up package-package
+  ;; references.
+  (dolist (package-data package-data-list)
+    (use-package (substitute "XC-STRICT-CL" "CL"
+                             (package-data-use package-data)
+                             :test 'string=)
+                 (package-data-name package-data))
+    (dolist (sublist (package-data-import-from package-data))
+      (let ((from-package (first sublist)))
+        (import (mapcar (lambda (name) (intern name from-package))
+                        (rest sublist))
+                (package-data-name package-data)))))
 
-    ;; Build all packages that we need, and initialize them as far as we
-    ;; can without referring to any other packages.
-    (dolist (package-data package-data-list)
-      (let* ((name (package-data-name package-data))
-             (package (make-package name :use nil)))
-        ;; Walk the tree of shadowing names
-        (dolist (string (flatten (package-data-shadow package-data)))
-          (shadow string package))
-        ;; Walk the tree of exported names, exporting each name.
-        (dolist (string (flatten (package-data-export package-data)))
-          (export (intern string package) package))))
+  (unhide-host-format-funs)
 
-    ;; Now that all packages exist, we can set up package-package
-    ;; references.
-    (dolist (package-data package-data-list)
-      (use-package (substitute "XC-STRICT-CL" "CL"
-                               (package-data-use package-data)
-                               :test 'string=)
-                   (package-data-name package-data))
-      (dolist (sublist (package-data-import-from package-data))
-        (let ((from-package (first sublist)))
-          (import (mapcar (lambda (name) (intern name from-package))
-                          (rest sublist))
-                  (package-data-name package-data)))))
+  ;; Now that all package-package references exist, we can handle
+  ;; REEXPORT operations. (We have to wait until now because they
+  ;; interact with USE operations.)  This code handles dependencies
+  ;; properly, but is somewhat ugly.
+  (let (done)
+    (labels
+        ((reexport (package-data)
+           (let ((package (find-package (package-data-name package-data))))
+             (cond
+               ((member package done))
+               ((null (package-data-reexport package-data))
+                (push package done))
+               (t
+                (mapcar #'reexport
+                        (remove-if-not
+                         (lambda (x)
+                           (member x (package-data-use package-data)
+                                   :test #'string=))
+                         package-data-list
+                         :key #'package-data-name))
+                (dolist (symbol-name (package-data-reexport package-data))
+                  (multiple-value-bind (symbol status)
+                      (find-symbol symbol-name package)
+                    (unless status
+                      (error "No symbol named ~S is accessible in ~S."
+                             symbol-name package))
+                    (when (eq (symbol-package symbol) package)
+                      (error
+                       "~S is not inherited/imported, but native to ~S."
+                       symbol-name package))
+                    (export symbol package)))
+                (push package done))))))
+      (dolist (x package-data-list)
+        (reexport x))
+      (assert (= (length done) (length package-data-list))))))
 
-    (unhide-host-format-funs)
+(export '*undefined-fun-allowlist*)
+(defvar *undefined-fun-allowlist* (make-hash-table :test 'equal))
 
-    ;; Now that all package-package references exist, we can handle
-    ;; REEXPORT operations. (We have to wait until now because they
-    ;; interact with USE operations.)  This code handles dependencies
-    ;; properly, but is somewhat ugly.
-    (let (done)
-      (labels
-          ((reexport (package-data)
-             (let ((package (find-package (package-data-name package-data))))
-               (cond
-                 ((member package done))
-                 ((null (package-data-reexport package-data))
-                  (push package done))
-                 (t
-                  (mapcar #'reexport
-                          (remove-if-not
-                           (lambda (x)
-                             (member x (package-data-use package-data)
-                                     :test #'string=))
-                           package-data-list
-                           :key #'package-data-name))
-                  (dolist (symbol-name
-                           (flatten (package-data-reexport package-data)))
-                    (multiple-value-bind (symbol status)
-                        (find-symbol symbol-name package)
-                      (unless status
-                        (error "No symbol named ~S is accessible in ~S."
-                               symbol-name package))
-                      (when (eq (symbol-package symbol) package)
-                        (error
-                         "~S is not inherited/imported, but native to ~S."
-                         symbol-name package))
-                      (export symbol package)))
-                  (push package done))))))
-        (dolist (x package-data-list)
-          (reexport x))
-        (assert (= (length done) (length package-data-list)))))))
+(defparameter *package-data-list* '())
 
-(export '*undefined-fun-whitelist*)
-(defvar *undefined-fun-whitelist* (make-hash-table :test 'equal))
-(let ((list
-       (with-open-file (data (prepend-genfile-path "package-data-list.lisp-expr"))
-         ;; There's no need to use the precautionary READ-FROM-FILE function
-         ;; with package-data-list because it is not a customization file.
-         (create-target-packages (let ((*readtable* *xc-readtable*)) (read data)))
-         (let ((*readtable* *xc-readtable*)) (read data)))))
-  (dolist (name (apply #'append list))
-    (setf (gethash name *undefined-fun-whitelist*) t)))
+;;; Like DEFPACKAGE, but can handle :REEXPORT.
+(defmacro defpackage* (name &rest options)
+  (let ((flattened-options '()))
+    (dolist (option options)
+      (destructuring-bind (kind . args) option
+        (case kind
+          ((:use :export :reexport :shadow)
+           (let ((existing-option (assoc kind flattened-options)))
+             (if existing-option
+                 (setf (second (second existing-option))
+                       (append (second (second existing-option)) args))
+                 (push (list kind (list 'quote args)) flattened-options))))
+          ((:import-from)
+           (let ((existing-option (assoc kind flattened-options)))
+             (if existing-option
+                 (push args (second (second existing-option)))
+                 (push (list kind (list 'quote (list args))) flattened-options))))
+          ((:documentation)
+           (push option flattened-options)))))
+    `(push (make-package-data :name ',name ,@(mapcan #'identity flattened-options))
+           *package-data-list*)))
+
+(let ((*readtable* *xc-readtable*))
+  (load (find-bootstrap-file "^exports.lisp"))
+  (create-target-packages *package-data-list*))
+
+(defun read-undefined-fun-allowlist ()
+  (with-open-file (data (find-bootstrap-file "^undefined-fun-allowlist.lisp-expr"))
+    (let ((*readtable* *xc-readtable*))
+      (dolist (name (apply #'append (read data)))
+        (setf (gethash name *undefined-fun-allowlist*) t)))))
 
 (defvar *asm-package-use-list*
   '("SB-ASSEM" "SB-DISASSEM"
@@ -504,18 +516,17 @@
 (make-assembler-package (backend-asm-package-name))
 
 (defun package-list-for-genesis ()
-  (append (let ((*readtable* *xc-readtable*))
-            (read-from-file "package-data-list.lisp-expr" nil))
+  (append *package-data-list*
           (let ((asm-package (backend-asm-package-name)))
             (list (make-package-data :name asm-package
                                      :use (list* "CL" *asm-package-use-list*)
-                                     :doc nil)))))
+                                     :documentation nil)))))
 
 ;;; Not all things shown by this are actually unused. Some get removed
 ;;; by the tree-shaker as intended.
 #+nil
 (defun show-unused-exports (&aux nonexistent uninteresting)
-  (dolist (entry (with-open-file (f "package-data-list.lisp-expr") (read f)))
+  (dolist (entry *package-data-list*)
     (let ((pkg (find-package (package-data-name entry))))
       (dolist (string (mapcan (lambda (x) (if (stringp x) (list x) x))
                               (package-data-export entry)))
@@ -524,7 +535,7 @@
             (cond ((not s)
                    (push (cons pkg string) nonexistent))
                   ((and (not (boundp s))
-                        (not (sb-kernel:symbol-info s))
+                        (not (sb-kernel:symbol-%info s))
                         (not (gethash s sb-c::*backend-parsed-vops*)))
                    (push s uninteresting))))))))
   (format t "~&Nonexistent:~%")
