@@ -72,22 +72,29 @@
 #+sb-thread
 (progn
   (define-vop (set)
-    (:args (object :scs (descriptor-reg))
+    (:args (object :scs (descriptor-reg)
+                   :load-if (not (and (sc-is object constant)
+                                      (symbol-always-has-tls-value-p (tn-value object)))))
            (value :scs (descriptor-reg any-reg zero)))
     (:temporary (:sc any-reg) tls-index)
     (:generator 4
-      (inst ldr (32-bit-reg tls-index) (tls-index-of object))
-      (inst ldr tmp-tn (@ thread-tn tls-index))
-      (inst cmp tmp-tn no-tls-value-marker-widetag)
-      (inst b :ne LOCAL)
-      (storew value object symbol-value-slot other-pointer-lowtag)
-      (inst b DONE)
-      LOCAL
-      (inst str value (@ thread-tn tls-index))
-      DONE))
+      (sc-case object
+        (constant
+         (inst str value (@ thread-tn (make-fixup (tn-value object) :symbol-tls-index))))
+        (t
+         (assemble ()
+           (inst ldr (32-bit-reg tls-index) (tls-index-of object))
+           (inst ldr tmp-tn (@ thread-tn tls-index))
+           (inst cmp tmp-tn no-tls-value-marker-widetag)
+           (inst b :ne LOCAL)
+           (storew value object symbol-value-slot other-pointer-lowtag)
+           (inst b DONE)
+           LOCAL
+           (inst str value (@ thread-tn tls-index))
+           DONE)))))
 
   (define-vop (symbol-value checked-cell-ref)
-    (:translate symeval)
+    (:translate symbol-value)
     (:args (symbol :scs (descriptor-reg) :to :save
                    :load-if (not (and (sc-is symbol constant)
                                       (or (symbol-always-has-tls-value-p (tn-value symbol))
@@ -140,7 +147,7 @@
   (define-vop (set cell-set)
     (:variant symbol-value-slot other-pointer-lowtag))
   (define-vop (symbol-value checked-cell-ref)
-    (:translate symeval)
+    (:translate symbol-value)
     (:generator 9
                 (loadw value object symbol-value-slot other-pointer-lowtag)
                 (let ((err-lab (generate-error-code vop 'unbound-symbol-error object)))
@@ -149,7 +156,7 @@
   (define-vop (fast-symbol-value cell-ref)
     (:variant symbol-value-slot other-pointer-lowtag)
     (:policy :fast)
-    (:translate symeval)))
+    (:translate symbol-value)))
 
 (define-vop (boundp)
   (:args (object :scs (descriptor-reg)))
@@ -180,11 +187,11 @@
 (define-vop (fast-symbol-global-value cell-ref)
   (:variant symbol-value-slot other-pointer-lowtag)
   (:policy :fast)
-  (:translate sym-global-val))
+  (:translate symbol-global-value))
 
 (define-vop (symbol-global-value)
   (:policy :fast-safe)
-  (:translate sym-global-val)
+  (:translate symbol-global-value)
   (:args (object :scs (descriptor-reg) :to (:result 1)))
   (:results (value :scs (descriptor-reg any-reg)))
   (:vop-var vop)
@@ -562,6 +569,16 @@
   (:result-types unsigned-num)
   (:translate %raw-instance-cas/word))
 
+(define-vop (%raw-instance-cas/signed-word %instance-cas)
+  (:args (object)
+         (index)
+         (old-value :scs (signed-reg))
+         (new-value :scs (signed-reg)))
+  (:arg-types * tagged-num signed-num signed-num)
+  (:results (result :scs (signed-reg) :from :load))
+  (:result-types signed-num)
+  (:translate %raw-instance-cas/signed-word))
+
 
 ;;;; Code object frobbing.
 
