@@ -36,9 +36,11 @@
   ;; can get them from the table rather than dumping them again. The
   ;; SIMILAR-TABLE is used for lists and strings, and the EQ-TABLE is
   ;; used for everything else. We use a separate EQ table to avoid
-  ;; performance pathologies with objects for which EQUAL degenerates
-  ;; to EQL. Everything entered in the SIMILAR table is also entered in
-  ;; the EQ table.
+  ;; performance pathologies with objects for which SIMILAR
+  ;; degenerates to EQL. Everything entered in the SIMILAR table is
+  ;; also entered in the EQ table. The NAMED-CONSTANT is used for
+  ;; named constants whose references are dumped as load time values
+  ;; of SYMBOL-GLOBAL-VALUE.
   (similar-table (make-similarity-table) :type hash-table :read-only t)
   (eq-table (make-hash-table :test 'eq) :type hash-table :read-only t)
   ;; the INSTANCE table maps dumpable instances to unique IDs for calculating
@@ -74,6 +76,10 @@
   ;; is the offset in the table of the code object needing to be
   ;; patched, and <offset> is the offset that must be patched.
   (patch-table (make-hash-table :test 'eq) :type hash-table)
+  ;; a list of the table handles for all of the DEBUG-INFO structures
+  ;; dumped in this file. These structures must be back-patched with
+  ;; source location information when the compilation is complete.
+  (debug-info () :type list)
   ;; This is used to keep track of objects that we are in the process
   ;; of dumping so that circularities can be preserved. The key is the
   ;; object that we have previously seen, and the value is the object
@@ -86,8 +92,8 @@
   ;; a hash table of structures that are allowed to be dumped. If we
   ;; try to dump a structure that isn't in this hash table, we lose.
   (valid-structures (make-hash-table :test 'eq) :type hash-table)
-  ;; DEBUG-SOURCE written at the very beginning
-  (source-info nil :type (or null sb-c::debug-source)))
+  ;; a hash table of slots to be saved when dumping an instance.
+  (saved-slot-names (make-hash-table :test 'eq) :type hash-table))
 (declaim (freeze-type fasl-output))
 
 ;;; Similarity hash table logic.
@@ -158,6 +164,7 @@
 ;; Unlike EQUAL-HASH, we never call EQ-HASH, because there is generally no reason
 ;; to try to look up an object that lacks a content-based hash value.
 (defun similar-hash (x)
+  (declare (special sb-c::*compile-object*))
   (named-let recurse ((x x))
     ;; There is no depth cutoff - X must not be circular,
     ;; which was already determined as a precondition to calling this,
@@ -209,11 +216,13 @@
 ;;; This structure holds information about a circularity.
 (defstruct (circularity (:copier nil))
   ;; the kind of modification to make to create circularity
-  (type (missing-arg) :type (member :rplaca :rplacd :svset :struct-set))
+  (type (missing-arg) :type (member :rplaca :rplacd :svset :struct-set :slot-set))
   ;; the object containing circularity
   object
   ;; index in object for circularity
   (index (missing-arg) :type index)
+  ;; slot name in object for circularity
+  (slot-name nil :type symbol)
   ;; the object to be stored at INDEX in OBJECT. This is that the key
   ;; that we were using when we discovered the circularity.
   value
@@ -434,7 +443,7 @@
 ;;; dumping them. If the object is in the EQ-TABLE, then we push it,
 ;;; otherwise, we do a type dispatch to a type specific dumping
 ;;; function. The type specific branches do any appropriate
-;;; EQUAL-TABLE check and table entry.
+;;; SIMILAR-TABLE check and table entry.
 ;;;
 ;;; When we go to dump the object, we enter it in the CIRCULARITY-TABLE.
 (defun dump-non-immediate-object (x file)
@@ -466,15 +475,15 @@
                               ((eq x sb-c::*debug-name-ellipsis*) 2)
                               (t (bug "Bogus debug name marker")))))
              (instance
-              (let ((c (gethash x (sb-c::eql-constants sb-c::*ir1-namespace*))))
-                (cond ((and c (sb-c::leaf-has-source-name-p c))
-                       (dump-load-time-symbol-global-value c file))
-                      (t
-                       (dump-structure x file)
-                       (eq-save-object x file)))))
+              (multiple-value-bind (slot-names slot-names-p)
+                  (gethash x (fasl-output-saved-slot-names file))
+                (if slot-names-p
+                    (dump-instance-saving-slots x slot-names file)
+                    (dump-structure x file)))
+              (eq-save-object x file))
              (array
               ;; DUMP-ARRAY (and its callees) are responsible for
-              ;; updating the EQ and EQUAL hash tables.
+              ;; updating the EQ and SIMILAR hash tables.
               (dump-array x file))
              (number
               (unless (similar-check-table x file)
@@ -507,7 +516,7 @@
 #+(and (not sb-xc-host) sb-simd-pack-256)
 (defun dump-simd-pack-256 (x file)
   (dump-fop 'fop-simd-pack file)
-  (dump-integer-as-n-bytes (logior (%simd-pack-256-tag x) 4) 8 file)
+  (dump-integer-as-n-bytes (logior (%simd-pack-256-tag x) (ash 1 6)) 8 file)
   (dump-integer-as-n-bytes (%simd-pack-256-0 x) 8 file)
   (dump-integer-as-n-bytes (%simd-pack-256-1 x) 8 file)
   (dump-integer-as-n-bytes (%simd-pack-256-2 x) 8 file)
@@ -565,7 +574,10 @@
                    (:rplaca     #.(get 'fop-rplaca 'opcode))
                    (:rplacd     #.(get 'fop-rplacd 'opcode))
                    (:svset      #.(get 'fop-svset 'opcode))
-                   (:struct-set #.(get 'fop-structset 'opcode)))
+                   (:struct-set #.(get 'fop-structset 'opcode))
+                   (:slot-set
+                    (dump-object (circularity-slot-name info) file)
+                    #.(get 'fop-slotset 'opcode)))
                  file)
       (dump-varint (gethash (circularity-object info) table) file)
       (dump-varint (circularity-index info) file))))
@@ -593,16 +605,13 @@
 
 ;;; Emit a funcall of the function and return the handle for the
 ;;; result.
-(defun fasl-dump-load-time-value-lambda (fun file no-skip)
+(defun fasl-dump-load-time-value-lambda (fun file)
   (declare (type sb-c::clambda fun) (type fasl-output file))
   (let ((handle (gethash (sb-c::leaf-info fun)
                          (fasl-output-entry-table file))))
     (aver handle)
     (dump-push handle file)
-    ;; Can't skip MAKE-LOAD-FORM due to later references
-    (if no-skip
-        (dump-fop 'fop-funcall-no-skip file 0)
-        (dump-fop 'fop-funcall file 0)))
+    (dump-fop 'fop-funcall file 0))
   (dump-pop file))
 
 ;;; Return T iff CONSTANT has already been dumped. It's been dumped if
@@ -626,9 +635,15 @@
 
 ;;; Note that the specified structure can just be dumped by
 ;;; enumerating the slots.
-(defun fasl-note-dumpable-instance (structure file)
+(defun fasl-validate-structure (structure file)
   (setf (gethash structure (fasl-output-valid-structures file)) t)
   (values))
+
+;;; Note that the specified standard object can just be dumped by
+;;; saving its slot values.
+(defun fasl-note-instance-saves-slots (instance slot-names file)
+  (setf (gethash instance (fasl-output-saved-slot-names file)) slot-names)
+  (values))
 
 ;;;; number dumping
 
@@ -850,7 +865,7 @@
       #+sb-xc-host (bug "Can't dump multi-dim array")))
 
 ;;; Dump the vector object. If it's not simple, then actually dump a
-;;; simple realization of it. But we enter the original in the EQ or EQUAL
+;;; simple realization of it. But we enter the original in the EQ or SIMILAR
 ;;; tables.
 (defun dump-vector (x file)
   (let ((simple-version (if (array-header-p x)
@@ -879,8 +894,9 @@
        (dump-simple-vector simple-version file)
        (eq-save-object x file))
       (t
-       (dump-specialized-vector simple-version file)
-       (eq-save-object x file)))))
+       (unless (similar-check-table x file)
+         (dump-specialized-vector simple-version file)
+         (similar-save-object x file))))))
 
 ;;; Dump a SIMPLE-VECTOR, handling any circularities.
 (defun dump-simple-vector (v file)
@@ -984,8 +1000,9 @@
            (dump-fop 'fop-keyword-symbol-save file length+flag))
           (t
            (let ((pkg-index (dump-package pkg file)))
-             (dump-fop 'fop-symbol-in-package-save file
-                       length+flag pkg-index))))
+             (if (eq (find-symbol pname pkg) :inherited)
+                 (dump-fop 'fop-symbol-in-package-save file length+flag pkg-index)
+                 (dump-fop 'fop-symbol-in-package-internal-save file length+flag pkg-index)))))
 
     (unless dumped-as-copy
       (dump-chars pname file base-string-p)
@@ -1017,7 +1034,7 @@
 
 (defconstant-eqx +fixup-flavors+
   #(:assembly-routine :assembly-routine* :asm-routine-nil-offset
-    :symbol-tls-index
+    :gc-barrier :symbol-tls-index
     :foreign :foreign-dataref :code-object
     :layout :immobile-symbol :named-call :static-call
     :symbol-value
@@ -1060,7 +1077,7 @@
                                    flavor))
            (operand
             (ecase flavor
-              (:code-object (the null name))
+              ((:code-object :gc-barrier) (the null name))
               (:layout
                (if (symbolp name)
                    name
@@ -1085,11 +1102,6 @@
       (dump-integer info fasl-output))
     (incf n)))
 
-(defun dump-load-time-symbol-global-value (constant fasl-output)
-  (dump-object 'symbol-global-value fasl-output)
-  (dump-object (sb-c::leaf-source-name constant) fasl-output)
-  (dump-fop 'fop-funcall fasl-output 1))
-
 ;;; Dump out the constant pool and code-vector for component, push the
 ;;; result in the table, and return the offset.
 ;;;
@@ -1102,7 +1114,7 @@
 ;;; constants.
 ;;;
 ;;; We dump trap objects in any unused slots or forward referenced slots.
-(defun dump-code-object (component code-segment code-length fixups fasl-output)
+(defun dump-code-object (component code-segment code-length fixups alloc-points fasl-output)
   (declare (type component component)
            (type index code-length)
            (type fasl-output fasl-output))
@@ -1111,22 +1123,20 @@
          (constants (sb-c:ir2-component-constants 2comp))
          (header-length (length constants))
          (n-named-calls 0))
-    (collect ((patches))
+    (dump-object alloc-points fasl-output)
+    (collect ((patches)
+              (named-constants))
       ;; Dump the constants, noting any :ENTRY constants that have to
       ;; be patched.
       (loop for i from sb-vm:code-constants-offset below header-length do
         (let ((entry (aref constants i)))
           (etypecase entry
             (constant
-             (if (and (sb-c::leaf-has-source-name-p entry)
-                      ;; We can't really reference constants defined
-                      ;; by name at load time in the same block
-                      ;; compilation unit, so dump it anonymously when
-                      ;; such a situation arises.
-                      (not (member (sb-c::leaf-source-name entry)
-                                   sb-c::*hairy-defconstants*)))
-                 (dump-load-time-symbol-global-value entry fasl-output)
-                 (dump-object (sb-c::constant-value entry) fasl-output)))
+             (cond ((sb-c::leaf-has-source-name-p entry)
+                    (named-constants (cons (sb-c::leaf-source-name entry) i))
+                    (dump-fop 'fop-misc-trap fasl-output))
+                   (t
+                    (dump-object (sb-c::constant-value entry) fasl-output))))
             (cons
              (ecase (car entry)
                (:constant ; anything that has not been wrapped in a #<CONSTANT>
@@ -1151,15 +1161,22 @@
                 (dump-fop 'fop-fdefn fasl-output))
                (:known-fun
                 (dump-object (cadr entry) fasl-output)
-                (dump-fop 'fop-known-fun fasl-output))))
+                (dump-fop 'fop-known-fun fasl-output))
+               (:coverage-marks
+                ;; Avoid the coalescence done by DUMP-VECTOR
+                (dump-specialized-vector (make-array (cdr entry)
+                                                     :element-type '(unsigned-byte 8)
+                                                     :initial-element #xFF)
+                                         fasl-output))))
             (null
              (dump-fop 'fop-misc-trap fasl-output)))))
 
       ;; Dump the debug info.
       (let ((info (sb-c::debug-info-for-component component)))
-        (setf (sb-c::debug-info-source info)
-              (fasl-output-source-info fasl-output))
-        (dump-object info fasl-output))
+        (fasl-validate-structure info fasl-output)
+        (dump-object info fasl-output)
+        (push (dump-to-table fasl-output)
+              (fasl-output-debug-info fasl-output)))
 
       (dump-fop 'fop-load-code fasl-output
                 (logior (ash header-length 1)
@@ -1167,6 +1184,9 @@
                 code-length n-fixups)
       ;; Fasl dumper/loader convention allows at most 3 integer args.
       ;; Others have to be written with explicit calls.
+      (dump-integer-as-n-bytes (length (sb-c::ir2-component-entries 2comp))
+                               4 ; output 4 bytes
+                               fasl-output)
       (dump-integer-as-n-bytes (the (unsigned-byte 22) n-named-calls)
                                4 ; output 4 bytes
                                fasl-output)
@@ -1177,12 +1197,17 @@
           (push (cons handle (cdr patch))
                 (gethash (car patch)
                          (fasl-output-patch-table fasl-output))))
+        (dolist (named-constant (named-constants))
+          (dump-object (car named-constant) fasl-output)
+          (dump-push handle fasl-output)
+          (dump-fop 'fop-named-constant-set fasl-output (cdr named-constant)))
         handle))))
 
 ;;; This is only called from assemfile, which doesn't exist in the target.
 #+sb-xc-host
-(defun dump-assembler-routines (code-segment octets fixups routines file)
+(defun dump-assembler-routines (code-segment octets fixups alloc-points routines file)
   (let ((n-fixups (dump-fixups fixups file)))
+    (dump-object alloc-points file)
     ;; The name -> address table has to be created before applying fixups
     ;; because a fixup may refer to an entry point in the same code component.
     ;; So these go on the stack last, i.e. nearest the top.
@@ -1211,11 +1236,7 @@
 
 ;;; Dump the code, constants, etc. for component. We pass in the
 ;;; assembler fixups, code vector and node info.
-(defun fasl-dump-component (component
-                            code-segment
-                            code-length
-                            fixups
-                            file)
+(defun fasl-dump-component (component code-segment code-length fixups alloc-points file)
   (declare (type component component))
   (declare (type fasl-output file))
 
@@ -1224,30 +1245,31 @@
   #+sb-dyncount
   (let ((info (sb-c::ir2-component-dyncount-info (component-info component))))
     (when info
-      (fasl-note-dumpable-instance info file)))
+      (fasl-validate-structure info file)))
 
   (let* ((2comp (component-info component))
          (entries (sb-c::ir2-component-entries 2comp))
          (nfuns (length entries))
          (code-handle
-          ;; fill in the placeholder elements of constants
-          ;; with the NAME, ARGLIST, TYPE, INFO slots of each simple-fun.
-          (let ((constants (sb-c:ir2-component-constants 2comp))
-                (wordindex (+ sb-vm:code-constants-offset
-                              (* sb-vm:code-slots-per-simple-fun nfuns))))
-            (dolist (entry entries)
-              ;; Process in reverse order of ENTRIES.
-              ;; See also MAKE-CORE-COMPONENT which does the same thing.
-              (decf wordindex sb-vm:code-slots-per-simple-fun)
-              (setf (aref constants (+ wordindex sb-vm:simple-fun-name-slot))
-                    `(:constant ,(sb-c::entry-info-name entry))
-                    (aref constants (+ wordindex sb-vm:simple-fun-arglist-slot))
-                    `(:constant ,(sb-c::entry-info-arguments entry))
-                    (aref constants (+ wordindex sb-vm:simple-fun-source-slot))
-                    `(:constant ,(sb-c::entry-info-form/doc entry))
-                    (aref constants (+ wordindex sb-vm:simple-fun-info-slot))
-                    `(:constant ,(sb-c::entry-info-type/xref entry))))
-            (dump-code-object component code-segment code-length fixups file)))
+           ;; fill in the placeholder elements of constants
+           ;; with the NAME, ARGLIST, TYPE, INFO slots of each simple-fun.
+           (let ((constants (sb-c:ir2-component-constants 2comp))
+                 (wordindex (+ sb-vm:code-constants-offset
+                               (* sb-vm:code-slots-per-simple-fun nfuns))))
+             (dolist (entry entries)
+               ;; Process in reverse order of ENTRIES.
+               ;; See also MAKE-CORE-COMPONENT which does the same thing.
+               (decf wordindex sb-vm:code-slots-per-simple-fun)
+               (setf (aref constants (+ wordindex sb-vm:simple-fun-name-slot))
+                     `(:constant ,(sb-c::entry-info-name entry))
+                     (aref constants (+ wordindex sb-vm:simple-fun-arglist-slot))
+                     `(:constant ,(sb-c::entry-info-arguments entry))
+                     (aref constants (+ wordindex sb-vm:simple-fun-source-slot))
+                     `(:constant ,(sb-c::entry-info-form/doc entry))
+                     (aref constants (+ wordindex sb-vm:simple-fun-info-slot))
+                     `(:constant ,(sb-c::entry-info-type/xref entry))))
+             (dump-code-object component code-segment code-length fixups
+                               alloc-points file)))
          (fun-index nfuns))
 
     (dolist (entry entries)
@@ -1257,13 +1279,25 @@
         ;; When cross compiling, if the entry is a DEFUN, then we also
         ;; dump a FOP-FSET so that the cold loader can instantiate the
         ;; definition at cold-load time, allowing forward references
-        ;; to functions in top-level forms.
+        ;; to functions in top-level forms. If the entry is a
+        ;; DEFMETHOD, we dump a FOP-MSET so that the cold loader
+        ;; recognizes the method definition.
         #+sb-xc-host
         (let ((name (sb-c::entry-info-name entry)))
-          (when (sb-c::legal-fun-name-p name)
-            (dump-object name file)
-            (dump-push entry-handle file)
-            (dump-fop 'fop-fset file)))
+          (cond ((sb-c::legal-fun-name-p name)
+                 (dump-object name file)
+                 (dump-push entry-handle file)
+                 (dump-fop 'fop-fset file))
+                ((and (listp name)
+                      (eq (first name) 'sb-pcl::fast-method))
+                 (let ((method (second name))
+                       (qualifiers (butlast (cddr name)))
+                       (specializers (first (last name))))
+                   (dump-object method file)
+                   (dump-object qualifiers file)
+                   (dump-object specializers file)
+                   (dump-push entry-handle file)
+                   (dump-fop 'fop-mset file)))))
         (setf (gethash entry (fasl-output-entry-table file)) entry-handle)
         (let ((old (gethash entry (fasl-output-patch-table file))))
           (when old
@@ -1290,46 +1324,45 @@
   (dump-push-previously-dumped-fun fun fasl-output)
   (dump-fop 'fop-funcall-for-effect fasl-output 0)
   (values))
+
+;;; Dump some information to allow partial reconstruction of the
+;;; DEBUG-SOURCE structure.
+(defun fasl-dump-partial-source-info (info file)
+  (declare (type sb-c::source-info info) (type fasl-output file))
+  (let ((partial (sb-c::debug-source-for-info info)))
+    (dump-object (sb-c::debug-source-namestring partial) file)
+    (dump-object (sb-c::debug-source-created partial) file)
+    (dump-object (sb-c::debug-source-plist partial) file)
+    (dump-fop 'fop-note-partial-source-info file)))
+
+;;; Compute the correct list of DEBUG-SOURCE structures and backpatch
+;;; all of the dumped DEBUG-INFO structures. We clear the
+;;; FASL-OUTPUT-DEBUG-INFO, so that subsequent components with
+;;; different source info may be dumped.
+(defun fasl-dump-source-info (info file)
+  (declare (type sb-c::source-info info) (type fasl-output file))
+  (let ((res (sb-c::debug-source-for-info info)))
+    (fasl-validate-structure res file)
+    (dump-object res file)
+    (let ((res-handle (dump-pop file)))
+      (dolist (info-handle (fasl-output-debug-info file))
+        (dump-push res-handle file)
+        (symbol-macrolet
+            ((debug-info-source-index
+               (let ((dd (find-defstruct-description 'sb-c::debug-info)))
+                 (dsd-index (find 'source (dd-slots dd)
+                                  :key #'dsd-name :test 'string=)))))
+          (dump-fop 'fop-structset file info-handle debug-info-source-index)))))
+
+  (setf (fasl-output-debug-info file) nil)
+  (values))
 
 ;;;; dumping structures
 
-;;; Even as late as calling DUMP-STRUCTURE we might have to deduce that a
-;;; user's "custom" MAKE-LOAD-FORM amounts to MAKE-LOAD-FORM-SAVING-SLOTS
-;;; with the default of all slots. Why: suppose you have some structure
-;;;   (DEFSTRUCT MYSTRUCT A)
-;;; and a macro that returns literal instances of the structure:
-;;;   (DEFMACRO FUNNYMAC (N) (MAKE-MYSTRUCT :A N))
-;;; and a DEFVAR that uses the structure:
-;;;   (DEFVAR *A* (FUNNYMAC 1))
-;;;
-;;; Now, because the fopcompiler expands macros more than once - at least once
-;;; in FOPCOMPILABLE-P and then again in FOPCOMPILE - we see _different_
-;;; instances of MYSTRUCT each of those times. We don't memoize the expansion.
-;;; The two structures are similar but not EQ, and only the instance produced
-;;; during FOPCOMPILABLE-P was entered in the FASL-OUTPUT-VALID-STRUCTURES table.
-;;; The other structure instance isn't there, but we need it to be legal to dump.
-;;;
-;;; This problem is not just theoretical.  We ourselves do just that, e.g.:
-;;;   (defvar *cpus* (... (sb-alien:alien-funcall ...)))
-;;; and the expansion of alien-funcall involves an ALIEN-TYPE literal
-;;; which gets multiply expanded exactly as described above.
-
-(defun load-form-is-default-mlfss-p (struct)
-  ;; FIXME? this is called while writing a fasl and so might need
-  ;; to invoke MAKE-LOAD-FORM, long after IR1 conversion has happened.
-  ;; Surely this is not the best design.
-  (eq (nth-value 1 (sb-c::%make-load-form struct)) 'fop-struct))
-
-;; Having done nothing more than load all files in obj/from-host, the
-;; cross-compiler running under any host Lisp begins life able to access
-;; SBCL-format metadata for any structure that is a subtype of STRUCTURE!OBJECT.
-;; But if it learns a layout by cross-compiling a DEFSTRUCT, that's ok too.
 (defun dump-structure (struct file)
   (unless (or (gethash struct (fasl-output-valid-structures file))
-              (typep struct
-                     '(or sb-c::debug-info sb-c::debug-fun sb-c::debug-source
-                          sb-c:definition-source-location sb-c::debug-name-marker))
-              (load-form-is-default-mlfss-p struct))
+              ;; Assume that DEBUG-FUNs are always valid to dump.
+              (typep struct 'sb-c::debug-fun))
     (error "attempt to dump invalid structure:~%  ~S~%How did this happen?"
            struct))
   (note-potential-circularity struct file)
@@ -1381,3 +1414,37 @@
             (1+ (wrapper-depthoid obj)) ; non-stack args can't be negative
             (logand flags sb-kernel::layout-flags-mask)
             (wrapper-length obj)))
+
+;;;; dumping instances which just save their slots
+
+(defun dump-instance-saving-slots (object slot-names file)
+  (note-potential-circularity object file)
+  (let ((circ (fasl-output-circularity-table file)))
+    (dolist (slot-name slot-names)
+      (if (slot-boundp object slot-name)
+          (let* ((value (slot-value object slot-name))
+                 (ref (gethash value circ)))
+            (cond (ref
+                   (push (make-circularity :type :slot-set
+                                           :object object
+                                           :index 0
+                                           :slot-name slot-name
+                                           :value value
+                                           :enclosing-object ref)
+                         *circularities-detected*)
+                   (sub-dump-object nil file))
+                  (t
+                   (sub-dump-object value file))))
+          (dump-fop 'fop-misc-trap file))
+      (sub-dump-object slot-name file)))
+  (sub-dump-object (class-name (class-of object)) file)
+  (dump-fop 'fop-instance file (length slot-names)))
+
+;;;; code coverage
+
+(defun dump-code-coverage-records (namestring cc file)
+  (declare (type string namestring)
+           (type list cc))
+  (dump-object namestring file)
+  (dump-object cc file)
+  (dump-fop 'fop-record-code-coverage file))
