@@ -111,8 +111,8 @@
 
 ;;; This is not FLUSHABLE, since it's required to signal an error if
 ;;; unbound.
-(defknown (symbol-value symeval) (symbol) t ()
-  :derive-type #'symeval-derive-type)
+(defknown symbol-value (symbol) t ()
+  :derive-type #'symbol-value-derive-type)
 (defknown about-to-modify-symbol-value (symbol t &optional t t) null
   ())
 ;;; From CLHS, "If the symbol is globally defined as a macro or a
@@ -1715,17 +1715,6 @@
    (:entry-points list)
    (:emit-cfasl t))
   (values (or pathname null) boolean boolean))
-(defknown sb-c::compile-files
-    (cons &key (:output-file pathname-designator)
-               (:verbose t)
-               (:print t)
-               (:external-format external-format-designator)
-               (:progress t)
-               (:trace-file t)
-               (:block-compile t)
-               (:entry-points list)
-               (:emit-cfasl t))
-  (values (or pathname null) boolean boolean))
 
 (defknown (compile-file-pathname)
   (pathname-designator &key (:output-file (or pathname-designator
@@ -1802,8 +1791,8 @@
 
 ;;;; miscellaneous extensions
 
-(defknown (symbol-global-value sym-global-val) (symbol) t ()
-  :derive-type #'symeval-derive-type)
+(defknown symbol-global-value (symbol) t ()
+  :derive-type #'symbol-value-derive-type)
 (defknown set-symbol-global-value (symbol t) t ()
   :derive-type #'result-type-last-arg)
 
@@ -1899,9 +1888,9 @@
   (always-translatable))
 ;;; The lowest-level vector SET operators should not return a value.
 ;;; Functions built upon them may return the input value.
-(defknown data-vector-set (array index t) (values) (always-translatable))
+(defknown data-vector-set (array index t) (values) (dx-safe always-translatable))
 (defknown data-vector-set-with-offset (array fixnum fixnum t) (values)
-  (always-translatable))
+  (dx-safe always-translatable))
 (defknown hairy-data-vector-ref (array index) t (foldable))
 (defknown hairy-data-vector-set (array index t) t ())
 (defknown hairy-data-vector-ref/check-bounds (array index) t (foldable))
