@@ -183,6 +183,7 @@ static inline lispobj compute_lispobj(lispobj* base_addr) {
                 (+ #x80 (case widetag
                           (#.instance-widetag instance-pointer-lowtag)
                           (#.+function-widetags+ fun-pointer-lowtag)
+                          (#.filler-widetag 0)
                           (t other-pointer-lowtag)))))))
     (format stream "unsigned char widetag_lowtag[256] = {")
     (dotimes (line 16)
@@ -242,10 +243,12 @@ static inline lispobj compute_lispobj(lispobj* base_addr) {
  = {~{~%  (void(*)(lispobj*,lispobj))~A~^,~}~%};~%" (length ptrtab) ptrtab)
       (write-table "static lispobj (*transother[64])(lispobj object)"
                    "trans_" transtab)
-      (format stream "#define size_pointer size_immediate~%")
+      (format stream "#define size_pointer (sizerfn)0~%")
+      (format stream "#define size_immediate (sizerfn)0~%")
       (format stream "#define size_unboxed size_boxed~%")
       (write-table "sword_t (*sizetab[256])(lispobj *where)"
                    "size_" sizetab)
+      (format stream "#undef size_immediate~%")
       (format stream "#undef size_pointer~%")
       (format stream "#undef size_unboxed~%")))
   (format stream "#endif~%"))
