@@ -122,6 +122,7 @@
 ;;; accepting any type object.
 (define-type-predicate %standard-char-p standard-char)
 (define-type-predicate non-null-symbol-p (and symbol (not null)))
+
 
 (defglobal *backend-type-predicates-grouped*
     (let (plist)
@@ -163,3 +164,32 @@
                  (sb-kernel::ctype-eq-comparable type))
             (vector-getf (truly-the simple-vector group) type #'eq 1)
             (vector-getf (truly-the simple-vector group) type #'type= 1))))))
+
+(defglobal *backend-union-type-predicates*
+    (let ((unions (sort
+                   (loop for (type . pred) in *backend-type-predicates*
+                         when (union-type-p type)
+                         collect (cons type pred))
+                   #'>
+                   :key (lambda (x)
+                          (length (union-type-types (car x)))))))
+      (coerce (loop for (key . value) in unions
+                    collect key
+                    collect value)
+              'vector)))
+(declaim (simple-vector *backend-union-type-predicates*))
+
+(defun split-union-type-tests (type)
+  (let ((predicates *backend-union-type-predicates*)
+        (types (union-type-types type)))
+    (loop for x below (length predicates) by 2
+          for union-types = (union-type-types (aref predicates x))
+          when (subsetp union-types types :test #'type=)
+          return (values (aref predicates (1+ x))
+                         (set-difference types union-types)))))
+
+(unless-vop-existsp (:translate keywordp)
+(define-source-transform keywordp (x)
+  `(let ((object ,x))
+     (and (non-null-symbol-p object)
+          (= (sb-impl::symbol-package-id object) ,sb-impl::+package-id-keyword+)))))
