@@ -15,8 +15,6 @@
 
 ;;;; LAMBDA hackery
 
-;;;; FIXME: where is that file?
-;;;; Answer: As usual, with CMUCL.
 ;;;; Note: Take a look at the compiler-overview.tex section on "Hairy
 ;;;; function representation" before you seriously mess with this
 ;;;; stuff.
@@ -873,6 +871,8 @@
 
     res))
 
+(defvar *lambda-conversions*)
+
 ;;; Convert a LAMBDA form into a LAMBDA leaf or an OPTIONAL-DISPATCH leaf.
 (defun ir1-convert-lambda (form &key (source-name '.anonymous.)
                            debug-name maybe-add-debug-catch
@@ -981,14 +981,6 @@
              (progn
                ,@forms))))))))
 
-;; FIXME: really should be an aspect of the lexical environment,
-;; but LEXENVs don't know whether they are toplevel or not.
-(defun has-toplevelness-decl (lambda-expr)
-  (dolist (expr (cddr lambda-expr)) ; Skip over (LAMBDA (ARGS))
-    (cond ((equal expr '(declare (top-level-form))) (return t))
-          ((typep expr '(or (cons (eql declare)) string))) ; DECL | DOCSTRING
-          (t (return nil)))))
-
 ;;; helper for LAMBDA-like things, to massage them into a form
 ;;; suitable for IR1-CONVERT-LAMBDA.
 (defun ir1-convert-lambdalike (thing
@@ -1006,7 +998,16 @@
     ((named-lambda)
      (let* ((name (cadr thing))
             (lambda-expression `(lambda ,@(cddr thing)))
-            (*inline-expansions* (list name 1 *inline-expansions*)))
+            (*inline-expansions* (list name 1 *inline-expansions*))
+            (simple-lexenv-p (simple-lexical-environment-p *lexenv*)))
+       ;; Discard any forward references to this function unless we
+       ;; are block compiling and the lexical environment doesn't
+       ;; contain any hair. If the lexical environment is too hairy, then we only
+       ;; install the definition during the processing of this NAMED-LAMBDA,
+       ;; ensuring that the function cannot be called outside of the correct
+       ;; environment.
+       (unless (and (block-compile *compilation*) simple-lexenv-p)
+         (remhash name (free-funs *ir1-namespace*)))
        (if (and name (legal-fun-name-p name))
            (let ((defined-fun-res (get-defined-fun name (second lambda-expression)))
                  (res (ir1-convert-lambda lambda-expression
@@ -1014,13 +1015,15 @@
                                           :source-name name))
                  (info (info :function :info name)))
              (setf (functional-inlinep res) (info :function :inlinep name)
-                   (defined-fun-named-lambda-p defined-fun-res) t)
-             (when (has-toplevelness-decl lambda-expression)
-               (setf (functional-top-level-defun-p res) t))
+                   (defined-fun-same-block-p defined-fun-res) t)
              ;; FIXME: Should non-entry block compiled defuns have
              ;; this propagate?
              (assert-global-function-definition-type name res)
-             (push res (defined-fun-functionals defined-fun-res))
+             ;; If in a simple environment, then we can allow
+             ;; backward references to this function from following
+             ;; top-level forms.
+             (when simple-lexenv-p
+               (push res (defined-fun-functionals defined-fun-res)))
              (unless (or
                       (eq (defined-fun-inlinep defined-fun-res) 'notinline)
                       ;; Don't treat recursive stubs like CAR as self-calls
@@ -1075,8 +1078,6 @@
             (allow (when (ll-kwds-allowp llks) '(&allow-other-keys))))
         (careful-specifier-type `(function (,@reqs ,@opts ,@rest ,@keys ,@allow) *))))))
 
-(declaim (start-block maybe-inline-syntactic-closure))
-
 ;;; Return a lambda form that has been "closed" with respect to
 ;;; LEXENV, returning a LAMBDA-WITH-LEXENV if there are interesting
 ;;; declarations. To handle local macros, rather than closing over
@@ -1084,7 +1085,7 @@
 ;;; LAMBDA, so that nothing in the syntactic environment is needed in
 ;;; the expansion. If there is something too complex in the lexical
 ;;; environment (like a lexical variable), then we return NIL.
-(defun maybe-inline-syntactic-closure (lambda lexenv)
+(defun inline-syntactic-closure-lambda (lambda lexenv)
   (declare (type list lambda) (type lexenv-designator lexenv))
   (aver (eql (first lambda) 'lambda))
   (typecase lexenv
@@ -1137,8 +1138,6 @@
    #+sb-fasteval
    (null lambda))) ; trivial case. Never occurs in the compiler.
 
-(declaim (end-block))
-
 (declaim (start-block ir1-convert-inline-lambda))
 
 ;;; Convert the forms produced by RECONSTRUCT-LEXENV to LEXENV
@@ -1288,6 +1287,18 @@
                 "previous declaration"
                 "previous definition"))))
 
+;;; The lexical environment is hairy if it has stuff like lexical
+;;; variables, blocks, or tags that are not load-time constants. Local
+;;; functions and macros are OK, since we are worried about outside
+;;; calls. Macros get expanded, and local functions are load-time
+;;; constants.
+(defun simple-lexical-environment-p (lexenv)
+  (and (null (lexenv-blocks lexenv))
+       (null (lexenv-tags lexenv))
+       (every (lambda (entry)
+                (consp (cdr entry)))
+              (lexenv-vars lexenv))))
+
 ;;; Used for global inline expansion. Earlier something like this was
 ;;; used by %DEFUN too. FIXME: And now it's probably worth rethinking
 ;;; whether this function is a good idea at all.
@@ -1334,9 +1345,10 @@
                          (fun-info-templates info)
                          (fun-info-ir2-convert info))))
       (substitute-leaf fun var)
-      ;; If in a simple environment, then we can allow backward references
-      ;; to this function from following top-level forms.
-      (when expansion
+      ;; If in a simple environment, then we can allow backward
+      ;; references to this function from following top-level
+      ;; forms.
+      (when (simple-lexical-environment-p *lexenv*)
         (push fun (defined-fun-functionals var))))
     fun))
 
