@@ -112,12 +112,12 @@
                                  *current-path*))))
     (funcall thunk)))
 
-(defvar *derive-function-types* nil
-  "Should the compiler assume that function types will never change,
-  so that it can use type information inferred from current definitions
-  to optimize code which uses those definitions? Setting this true
-  gives non-ANSI, early-CMU-CL behavior. It can be useful for improving
-  the efficiency of stable code.")
+(defvar *derive-function-types* :same-file
+  "If true, argument and result type information derived from
+  compilation of DEFUNs is used when compiling calls to that
+  function. If :SAME-FILE (the default, as allowed by ANSI 3.2.2.3),
+  the information is derived only from DEFUNs in the same file. If
+  false, only information from FTYPE proclamations will be used.")
 
 ;;;; namespace management utilities
 
@@ -214,62 +214,17 @@
          :type (if (or (eq where :declared)
                        (and (not latep)
                             (not notinline)
-                            *derive-function-types*))
+                            (eq *derive-function-types* t)))
                    ftype
                    (specifier-type 'function))
-         :defined-type (if (and (not latep) (not notinline))
+         :defined-type (if (and (not latep) (not notinline)
+                                *derive-function-types*)
                            ftype
                            (specifier-type 'function))
          :where-from (if notinline
                          where
                          (maybe-defined-here name where)))))))
 
-;;; Have some DEFINED-FUN-FUNCTIONALS of a FREE-FUNS entry become invalid?
-;;; Drop 'em.
-;;;
-;;; This was added to fix bug 138 in SBCL. It is possible for a FREE-FUNS
-;;; entry to contain a DEFINED-FUN whose DEFINED-FUN-FUNCTIONAL object
-;;; contained IR1 stuff (NODEs, BLOCKs...) referring to an already compiled
-;;; (aka "dead") component. When this IR1 stuff was reused in a new component,
-;;; under further obscure circumstances it could be used by
-;;; WITH-IR1-ENVIRONMENT-FROM-NODE to generate a binding for
-;;; *CURRENT-COMPONENT*. At that point things got all confused, since IR1
-;;; conversion was sending code to a component which had already been compiled
-;;; and would never be compiled again.
-;;;
-;;; Note: as of 1.0.24.41 this seems to happen only in XC, and the original
-;;; BUGS entry also makes it seem like this might not be an issue at all on
-;;; target.
-(defun clear-invalid-functionals (free-fun)
-  ;; There might be other reasons that FREE-FUN entries could
-  ;; become invalid, but the only one we've been bitten by so far
-  ;; (sbcl-0.pre7.118) is this one:
-  (when (defined-fun-p free-fun)
-    (setf (defined-fun-functionals free-fun)
-          (delete-if (lambda (functional)
-                       (or (eq (functional-kind functional) :deleted)
-                           (when (lambda-p functional)
-                             (or
-                              ;; (The main reason for this first test is to bail
-                              ;; out early in cases where the LAMBDA-COMPONENT
-                              ;; call in the second test would fail because links
-                              ;; it needs are uninitialized or invalid.)
-                              ;;
-                              ;; If the BIND node for this LAMBDA is null, then
-                              ;; according to the slot comments, the LAMBDA has
-                              ;; been deleted or its call has been deleted. In
-                              ;; that case, it seems rather questionable to reuse
-                              ;; it, and certainly it shouldn't be necessary to
-                              ;; reuse it, so we cheerfully declare it invalid.
-                              (not (lambda-bind functional))
-                              ;; If this IR1 stuff belongs to a dead component,
-                              ;; then we can't reuse it without getting into
-                              ;; bizarre confusion.
-                              (eq (component-info (lambda-component functional))
-                                  :dead)))))
-                     (defined-fun-functionals free-fun)))
-    nil))
-
 ;;; If NAME already has a valid entry in (FREE-FUNS *IR1-NAMESPACE*), then return
 ;;; the value. Otherwise, make a new GLOBAL-VAR using information from
 ;;; the global environment and enter it in FREE-FUNS. If NAME
@@ -278,20 +233,7 @@
 ;;; demanded a function.
 (declaim (ftype (sfunction (t string) global-var) find-free-fun))
 (defun find-free-fun (name context &aux (free-funs (free-funs *ir1-namespace*)))
-  (or (let ((old-free-fun (gethash name free-funs)))
-        (when old-free-fun
-          (clear-invalid-functionals old-free-fun)
-          (when (or (not (defined-fun-p old-free-fun))
-                    (not (block-compile *compilation*))
-                    ;; When block-compiling, it is the case that we
-                    ;; could proclaim an inline, define some things,
-                    ;; then proclaim a notinline, and in the case that
-                    ;; the function's :inlinep info changes, the
-                    ;; old-free-fun is stale and we should make a new
-                    ;; one.
-                    (eq (info :function :inlinep name)
-                        (defined-fun-inlinep old-free-fun)))
-            old-free-fun)))
+  (or (gethash name free-funs)
       (let ((kind (info :function :kind name)))
         (ecase kind
           ((:macro :special-form)
@@ -702,18 +644,15 @@
 
 ;;; Generate a REF node for LEAF, frobbing the LEAF structure as
 ;;; needed. If LEAF represents a defined function which has already
-;;; been converted, and is not NOTINLINE, then reference the
-;;; functional instead.
+;;; been converted in the same compilation block, and is not
+;;; NOTINLINE, then reference the functional instead.
 (defun reference-leaf (start next result leaf &optional (name '.anonymous.))
   (declare (type ctran start next) (type (or lvar null) result) (type leaf leaf))
   (assure-leaf-live-p leaf)
   (let* ((type (lexenv-find leaf type-restrictions))
          (leaf (or (and (defined-fun-p leaf)
                         (neq (defined-fun-inlinep leaf) 'notinline)
-                        ;; Only reference defined functionals from named-lambda,
-                        ;; everything else should do this through recognize-known-call,
-                        ;; avoiding copying references to global functions
-                        (defined-fun-named-lambda-p leaf)
+                        (defined-fun-same-block-p leaf)
                         (let ((functional (defined-fun-functional leaf)))
                           (when (and functional (not (functional-kind functional)))
                             (maybe-reanalyze-functional functional))))
@@ -754,50 +693,39 @@
 (defun ir1-convert-var (start next result name)
   (declare (type ctran start next) (type (or lvar null) result) (symbol name))
   (let ((var (or (lexenv-find name vars) (find-free-var name))))
-    (if (and (global-var-p var) (not (always-boundp name)))
-        ;; KLUDGE: If the variable may be unbound, convert using SYMEVAL
-        ;; which is not flushable, so that unbound dead variables signal an
-        ;; error (bug 412, lp#722734): checking for null RESULT is not enough,
-        ;; since variables can become dead due to later optimizations.
-        (ir1-convert start next result
-                     (case (global-var-kind var)
-                       (:global `(sym-global-val ',name))
-                       (:unknown
-                        (when (not (deprecated-thing-p 'variable name))
-                          (note-undefined-reference name :variable))
-                        `(symeval ',name))
-                       (t
-                        `(symeval ',name))))
-        (etypecase var
-          (leaf
-           (cond
-             ((lambda-var-p var)
-              (let ((home (ctran-home-lambda-or-null start)))
-                (when home
-                  (sset-adjoin var (lambda-calls-or-closes home))))
-              (when (lambda-var-ignorep var)
-                ;; (ANSI's specification for the IGNORE declaration requires
-                ;; that this be a STYLE-WARNING, not a full WARNING.)
-                #-sb-xc-host
-                (compiler-style-warn "reading an ignored variable: ~S" name)
-                ;; there's no need for us to accept ANSI's lameness when
-                ;; processing our own code, though.
-                #+sb-xc-host
-                (warn "reading an ignored variable: ~S" name))))
-           (reference-leaf start next result var name))
-          ((cons (eql macro)) ; symbol-macro
-           ;; FIXME: the following comment is probably wrong now.
-           ;; If we warn here on :early and :late deprecation
-           ;; then we get an extra warning somehow.
-           ;; This case signals {EARLY,LATE,FINAL}-DEPRECATION-WARNING
-           ;; for symbol-macros. Includes variables, constants,
-           ;; etc. in :FINAL deprecation.
-           (when (eq (deprecated-thing-p 'variable name) :final)
-             (check-deprecated-thing 'variable name))
-           ;; FIXME: [Free] type declarations. -- APD, 2002-01-26
-           (ir1-convert start next result (cdr var)))
-          (heap-alien-info
-           (ir1-convert start next result `(%heap-alien ',var))))))
+    (etypecase var
+      (leaf
+       (when (lambda-var-p var)
+         (let ((home (ctran-home-lambda-or-null start)))
+           (when (and home (neq (lambda-var-home var) home))
+             (sset-adjoin var (lambda-calls-or-closes home))))
+         (when (lambda-var-ignorep var)
+           ;; (ANSI's specification for the IGNORE declaration requires
+           ;; that this be a STYLE-WARNING, not a full WARNING.)
+           #-sb-xc-host
+           (compiler-style-warn "reading an ignored variable: ~S" name)
+           ;; there's no need for us to accept ANSI's lameness when
+           ;; processing our own code, though.
+           #+sb-xc-host
+           (warn "reading an ignored variable: ~S" name)))
+       (when (and (global-var-p var)
+                  (eq (global-var-kind var) :unknown)
+                  (not (deprecated-thing-p 'variable name)))
+         (note-undefined-reference name :variable))
+       (reference-leaf start next result var name))
+      ((cons (eql macro)) ; symbol-macro
+       ;; FIXME: the following comment is probably wrong now.
+       ;; If we warn here on :early and :late deprecation
+       ;; then we get an extra warning somehow.
+       ;; This case signals {EARLY,LATE,FINAL}-DEPRECATION-WARNING
+       ;; for symbol-macros. Includes variables, constants,
+       ;; etc. in :FINAL deprecation.
+       (when (eq (deprecated-thing-p 'variable name) :final)
+         (check-deprecated-thing 'variable name))
+       ;; FIXME: [Free] type declarations. -- APD, 2002-01-26
+       (ir1-convert start next result (cdr var)))
+      (heap-alien-info
+       (ir1-convert start next result `(%heap-alien ',var)))))
   (values))
 
 ;;; Find a compiler-macro for a form, taking FUNCALL into account.
@@ -1005,19 +933,13 @@
 ;;; instrumentation for?
 (defun step-form-p (form)
   (flet ((step-symbol-p (symbol)
-           ;; Consistent treatment of *FOO* vs (SYMBOL-VALUE '*FOO*):
-           ;; we insert calls to SYMEVAL for most non-lexical
-           ;; variable references in order to avoid them being elided
-           ;; if the value is unused.
-           (if (member symbol '(symeval sym-global-val))
-               (not (constantp (second form)))
-               (not (member (sb-xc:symbol-package symbol)
-                            (load-time-value
-                              ;; KLUDGE: packages we're not interested in
-                              ;; stepping.
-                              (mapcar #'find-package '(sb-c sb-int sb-impl
-                                                       sb-kernel sb-pcl))
-                             t))))))
+           (not (member (sb-xc:symbol-package symbol)
+                        (load-time-value
+                         ;; KLUDGE: packages we're not interested in
+                         ;; stepping.
+                         (mapcar #'find-package '(sb-c sb-int sb-impl
+                                                  sb-kernel sb-pcl))
+                         t)))))
     (and *allow-instrumenting*
          (policy *lexenv* (= insert-step-conditions 3))
          (listp form)
@@ -1420,7 +1342,9 @@
       (setf (defined-fun-inline-expansion res)
             (defined-fun-inline-expansion var))
       (setf (defined-fun-functionals res)
-            (defined-fun-functionals var)))
+            (defined-fun-functionals var))
+      (setf (defined-fun-same-block-p res)
+            (defined-fun-same-block-p var)))
     ;; FIXME: Is this really right? Needs we not set the FUNCTIONAL
     ;; to the original global-var?
     res))
@@ -1529,6 +1453,13 @@
          (setf (lambda-var-ignorep var) t)))))
   (values))
 
+(defvar *stack-allocate-dynamic-extent* t
+  "If true (the default), the compiler respects DYNAMIC-EXTENT declarations
+and stack allocates otherwise inaccessible parts of the object whenever
+possible. Potentially long (over one page in size) vectors are, however, not
+stack allocated except in zero SAFETY code, as such a vector could overflow
+the stack without triggering overflow protection.")
+
 (defun process-extent-decl (names vars fvars kind)
   (let ((extent
           (ecase kind
@@ -1755,7 +1686,6 @@
                                       it))
                     (setq explicit-check (or (cdr spec) t)
                           allow-explicit-check nil)) ; at most one of this decl
-                   ((equal spec '(top-level-form))) ; ignore
                    ((typep spec '(cons (eql source-form)))
                     (setf source-form (cadr spec)))
                    ;; Used only for the current function.
@@ -1834,3 +1764,136 @@
                           :where-from :declared))))
 
 (declaim (end-block))
+
+
+;;;; Proclamations:
+;;;
+;;; PROCLAIM changes the global environment, so we must handle its
+;;; compile time side effects if we are to keep the information in the
+;;; (FREE-xxx *IR1-NAMESPACE*) tables up to date. When there is a var
+;;; structure we disown it by replacing it with an updated copy. Uses
+;;; of the variable which were translated before the PROCLAIM will get
+;;; the old version, while subsequent references will get the updated
+;;; information.
+
+;;; If a special block compilation delimiter, then start or end the
+;;; block as appropriate. If :BLOCK-COMPILE is T or NIL, then we
+;;; ignore any start/end block declarations.
+(defun process-block-compile-proclamation (kind entry-points)
+  (if (eq *block-compile-argument* :specified)
+      (ecase kind
+        (start-block
+         (finish-block-compilation)
+         (let ((compilation *compilation*))
+           (setf (block-compile compilation) t)
+           (setf (entry-points compilation) entry-points)))
+        (end-block
+         (finish-block-compilation)))
+      (compiler-notify "ignoring ~S declaration since ~
+                        :BLOCK-COMPILE is not :SPECIFIED"
+                       kind)))
+
+;;; Update function type info cached in (FREE-FUNS *IR1-NAMESPACE*).
+;;; If:
+;;; -- there is a GLOBAL-VAR, then just update the type and remove the
+;;;    name from the list of undefined functions. Someday we should
+;;;    check for incompatible redeclaration.
+;;; -- there is a FUNCTIONAL, then apply the type assertion to that
+;;;    function.  This will only happen during block compilation.
+(defun process-1-ftype-proclamation (name type)
+  (proclaim-as-fun-name name)
+  (let* ((free-funs (free-funs *ir1-namespace*))
+         (var (gethash name free-funs)))
+    (etypecase var
+      (null)
+      (global-var
+       (setf (gethash name free-funs)
+             (let ((kind (global-var-kind var)))
+               (if (defined-fun-p var)
+                   (make-defined-fun
+                    :%source-name name :type type :where-from :declared :kind kind
+                    :inlinep (defined-fun-inlinep var)
+                    :inline-expansion (defined-fun-inline-expansion var)
+                    :same-block-p (defined-fun-same-block-p var)
+                    :functionals (defined-fun-functionals var))
+                   (make-global-var :%source-name name :type type
+                                    :where-from :declared :kind kind))))
+       (when (defined-fun-p var)
+         (let ((fun (defined-fun-functional var)))
+           (when fun
+             (assert-definition-type fun type
+                                     :unwinnage-fun #'compiler-notify
+                                     :where "this declaration"))))))))
+
+(defun process-ftype-proclamation (spec names)
+  (declare (list names))
+  (let ((type (specifier-type spec)))
+    (unless (csubtypep type (specifier-type 'function))
+      (error "Not a function type: ~/sb-impl:print-type/" spec))
+    (dolist (name names)
+      (process-1-ftype-proclamation name type))))
+
+;;; Replace each old var entry with one having the new type.
+(defun process-type-proclamation (spec names)
+  (declare (list names))
+  (let ((free-vars (free-vars *ir1-namespace*))
+        (type (specifier-type spec)))
+    (dolist (name names)
+      (let ((var (gethash name free-vars)))
+        (etypecase var
+          (null)
+          (global-var
+           (setf (gethash name free-vars)
+                 (make-global-var :%source-name name
+                                  :type type :where-from :declared
+                                  :kind (global-var-kind var)))))))))
+
+;;; Similar in effect to FTYPE, but change the :INLINEP. Copying the
+;;; global-var ensures that when we substitute a functional for a
+;;; global var (i.e. for DEFUN) that we won't clobber any uses
+;;; declared :NOTINLINE.
+(defun process-inline-proclamation (kind funs)
+  (declare (type (and inlinep (not null)) kind))
+  (dolist (name funs)
+    (proclaim-as-fun-name name)
+    (let* ((free-funs (free-funs *ir1-namespace*))
+           (var (gethash name free-funs)))
+      (etypecase var
+        (null)
+        (global-var
+         (setf (gethash name free-funs)
+               ;; Use the universal type as the local type restriction,
+               ;; since we are processing the proclamation at the top
+               ;; level and hence in a null lexical environment.
+               (make-new-inlinep var kind *universal-type*)))))))
+
+;;; Handle the compile-time side effects of PROCLAIM that don't happen
+;;; at load time. These mostly side-effect the global state of the
+;;; compiler, rather than the global environment.
+(defun %compiler-proclaim (kind args)
+  (case kind
+    ((special global)
+     (dolist (name args)
+       (let* ((free-vars (free-vars *ir1-namespace*))
+              (old (gethash name free-vars)))
+         (when old
+           (ecase (global-var-kind old)
+             ((:special :global))
+             (:unknown
+              (setf (gethash name free-vars)
+                    (make-global-var :%source-name name :type (leaf-type old)
+                                     :where-from (leaf-where-from old)
+                                     :kind (ecase kind
+                                             (special :special)
+                                             (global :global))))))))))
+    ((start-block end-block)
+     #-(and sb-devel sb-xc-host)
+     (process-block-compile-proclamation kind args))
+    (ftype
+     (destructuring-bind (spec &rest args) args
+       (process-ftype-proclamation spec args)))
+    (type
+     (destructuring-bind (spec &rest args) args
+       (process-type-proclamation spec args)))
+    ((inline notinline maybe-inline)
+     (process-inline-proclamation kind args))))
