@@ -91,24 +91,46 @@
 (defun cmovp (label a b)
   (declare (type label label)
            (type cblock a b))
-  (cond ((eq label (ir2-block-%label (block-info a))))
-        ((eq label (ir2-block-%label (block-info b)))
-         (rotatef a b))
-        (t (return-from cmovp)))
-  (let ((succ-a (block-succ a))
-        (succ-b (block-succ b)))
-    (when (and (singleton-p succ-a)
-               (singleton-p succ-b)
-               (eq (car succ-a) (car succ-b))
-               (singleton-p (block-pred a))
-               (singleton-p (block-pred b)))
-      (multiple-value-bind (value-a target)
-          (move-value-target (block-info a))
-        (multiple-value-bind (value-b targetp)
-            (move-value-target (block-info b))
-          (and value-a value-b (eq target targetp)
-               (values (block-label (car succ-a))
-                       target value-a value-b)))))))
+  (let ((a2 (block-info a))
+        (b2 (block-info b)))
+   (cond ((eq label (ir2-block-%label a2)))
+         ((eq label (ir2-block-%label b2))
+          (rotatef a b)
+          (rotatef a2 b2))
+         (t (return-from cmovp)))
+   (let ((succ-a (block-succ a))
+         (succ-b (block-succ b)))
+     (cond ((and (singleton-p succ-a)
+                 (singleton-p succ-b)
+                 (eq (car succ-a) (car succ-b))
+                 (singleton-p (block-pred a))
+                 (singleton-p (block-pred b)))
+            (multiple-value-bind (value-a target)
+                (move-value-target a2)
+              (multiple-value-bind (value-b targetp)
+                  (move-value-target b2)
+                (and value-a value-b (eq target targetp)
+                     (values (block-label (car succ-a))
+                             target value-a value-b)))))
+           ;; A branch jumping over a move.
+           ((and (singleton-p succ-a)
+                 (singleton-p (block-pred a))
+                 (equal (car succ-a)
+                        b))
+            (multiple-value-bind (value target)
+                (move-value-target a2)
+              (when value
+                (values (block-label b)
+                        target value target))))
+           ((and (singleton-p succ-b)
+                 (singleton-p (block-pred b))
+                 (equal (car succ-b)
+                        a))
+            (multiple-value-bind (value target)
+                (move-value-target b2)
+              (when value
+                (values (block-label a)
+                        target target value))))))))
 
 #-x86-64
 (defun sb-vm::computable-from-flags-p (res x y flags)
@@ -130,6 +152,10 @@
                          vop node 2block
                          &aux (prev (vop-prev vop)))
   (delete-vop vop)
+  (let ((last (ir2-block-last-vop 2block)))
+    (when (and last
+               (eq (vop-name last) 'branch))
+      (delete-vop last)))
   (cond
     ((and (constant-tn-p arg-if)
           (constant-tn-p arg-else)
@@ -586,20 +612,20 @@
     (let* ((successors (ir2block-successors (vop-block next)))
            (info (vop-codegen-info next))
            (label-block (gethash (car info) *2block-info*))
-           (symeval-vop
+           (symbol-value-vop
             (ir2-block-start-vop
              (cond ((eq (second info) nil) label-block) ; not negated test.
                    ;; When negated, the BOUNDP case goes to the "other" successor
                    ;; block, i.e. whichever is not started by the #<label>.
                    ((eq label-block (first successors)) (second successors))
                    (t (first successors))))))
-      (when (and symeval-vop
-                 (or (eq (vop-name symeval-vop) 'symbol-value)
+      (when (and symbol-value-vop
+                 (or (eq (vop-name symbol-value-vop) 'symbol-value)
                      ;; Expect SYMBOL-GLOBAL-VALUE only for variables of kind :GLOBAL.
-                     (and (eq (vop-name symeval-vop) 'symbol-global-value)
+                     (and (eq (vop-name symbol-value-vop) 'symbol-global-value)
                           (constant-tn-p sym)
                           (eq (info :variable :kind (tn-value sym)) :global)))
-                 (eq sym (tn-ref-tn (vop-args symeval-vop)))
+                 (eq sym (tn-ref-tn (vop-args symbol-value-vop)))
                  ;; If the symbol is either a compile-time known symbol,
                  ;; or can only be the same thing for both vops,
                  ;; then we're fine to combine.
@@ -610,11 +636,11 @@
                  ;; Technically we could split the IR2 block and peel off the SYMBOL-VALUE,
                  ;; and if coming from the BRANCH-IF, jump to the block that contained
                  ;; everything else except the SYMBOL-VALUE vop.
-                 (not (cdr (ir2block-predecessors (vop-block symeval-vop)))))
-        (let ((replacement (if (eq (vop-name symeval-vop) 'symbol-global-value)
+                 (not (cdr (ir2block-predecessors (vop-block symbol-value-vop)))))
+        (let ((replacement (if (eq (vop-name symbol-value-vop) 'symbol-global-value)
                                'fast-symbol-global-value
                                'fast-symbol-value))
-              (result-tn (tn-ref-tn (vop-results symeval-vop))))
+              (result-tn (tn-ref-tn (vop-results symbol-value-vop))))
           (emit-and-insert-vop (vop-node vop) (vop-block vop)
                                (template-or-lose replacement)
                                (reference-tn sym nil) (reference-tn result-tn t) next)
@@ -627,7 +653,7 @@
           ;; BRANCH-IF test is automagically correct after substitution.
           ;; Of course, this is machine-dependent.
           (delete-vop vop)
-          (delete-vop symeval-vop)
+          (delete-vop symbol-value-vop)
           next)))))
 
 ;;; Optimize (if x ... nil) to reuse the NIL coming from X.
