@@ -274,10 +274,10 @@
 
 ;;;; generic type inference methods
 
-(defun symeval-derive-type (node &aux (args (basic-combination-args node))
+(defun symbol-value-derive-type (node &aux (args (basic-combination-args node))
                                       (lvar (pop args)))
   (unless (and lvar (endp args))
-    (return-from symeval-derive-type))
+    (return-from symbol-value-derive-type))
   (if (constant-lvar-p lvar)
       (let* ((sym (lvar-value lvar))
              (var (maybe-find-free-var sym))
@@ -305,15 +305,6 @@
     (let ((lvar (nth n (combination-args call))))
       (when lvar (lvar-type lvar)))))
 
-;;; Derive the result type according to the float contagion rules, but
-;;; always return a float. This is used for irrational functions that
-;;; preserve realness of their arguments.
-(defun result-type-float-contagion (call)
-  (declare (type combination call))
-  (reduce #'numeric-contagion (combination-args call)
-          :key #'lvar-type
-          :initial-value (specifier-type 'single-float)))
-
 (defun simplify-list-type (type &key preserve-dimensions)
   ;; Preserve all the list types without dragging
   ;; (cons (eql 10)) stuff in.
@@ -468,16 +459,6 @@
             (max-dim (lvar-type lvar))
             (values max min))))))
 
-(defun position-derive-type (call)
-  (let ((dim (sequence-lvar-dimensions (second (combination-args call)))))
-    (when (integerp dim)
-      (specifier-type `(or (integer 0 (,dim)) null)))))
-
-(defun count-derive-type (call)
-  (let ((dim (sequence-lvar-dimensions (second (combination-args call)))))
-    (when (integerp dim)
-      (specifier-type `(integer 0 ,dim)))))
-
 ;;; This used to be done in DEFOPTIMIZER DERIVE-TYPE, but
 ;;; ASSERT-CALL-TYPE already asserts the ARRAY type, so it gets an extra
 ;;; assertion that may not get eliminated and requires extra work.
