@@ -67,12 +67,13 @@
   ;; The block may end up being deleted due to cast optimization
   ;; caused by USE-GOOD-FOR-DX-P
   (unless (node-to-be-deleted-p call)
-    (let ((dx-lvars
-            (loop for arg in (basic-combination-args call)
-                  for var in (lambda-vars fun)
-                  for dx = (leaf-dynamic-extent var)
-                  when (and dx arg (not (lvar-dynamic-extent arg)))
-                    append (handle-nested-dynamic-extent-lvars dx arg))))
+    (let* ((*dx-lexenv* (node-lexenv call))
+           (dx-lvars
+             (loop for arg in (basic-combination-args call)
+                   for var in (lambda-vars fun)
+                   for dx = (leaf-dynamic-extent var)
+                   when (and dx arg (not (lvar-dynamic-extent arg)))
+                   append (handle-nested-dynamic-extent-lvars dx arg))))
       (when dx-lvars
         (let* ((entry (with-ir1-environment-from-node call
                         (make-entry)))
@@ -1199,11 +1200,10 @@
              (not (functional-inline-expanded clambda)))
     ;; ANSI requires that explicit NOTINLINE be respected.
     (or (eq (lambda-inlinep clambda) 'notinline)
-        ;; If (= LET-CONVERSION 0) we can guess that inlining
-        ;; generally won't be appreciated, but if the user
-        ;; specifically requests inlining, that takes precedence over
-        ;; our general guess.
-        (and (policy clambda (= let-conversion 0))
+        ;; If (= LET-CONVERSION 0) or (= DEBUG 3) we can guess that inlining
+        ;; generally won't be appreciated, but if the user specifically requests
+        ;; inlining, that takes precedence over our general guess.
+        (and (or (policy clambda (or (= let-conversion 0) (= debug 3))))
              (not (eq (lambda-inlinep clambda) 'inline))))))
 
 ;;; We also don't convert calls to named functions which appear in the
