@@ -17,8 +17,6 @@
 ;;; attempt
 (defvar *pack-assign-costs* t)
 (defvar *pack-optimize-saves* t)
-;;; FIXME: Perhaps SB-FLUID should be renamed to SB-TWEAK and these
-;;; should be made conditional on SB-TWEAK.
 
 (declaim (ftype (function (component) index) ir2-block-count))
 
@@ -418,7 +416,6 @@
 
 ;;;; register saving
 
-#-sb-devel
 (declaim (start-block optimized-emit-saves emit-saves assign-tn-costs
                       pack-save-tn))
 
@@ -488,11 +485,14 @@
     (when (eq (tn-kind save) :specified-save)
       (setf (tn-kind save) :save))
     (aver (eq (tn-kind save) :save))
-    (emit-operand-load node block tn save
-                       (if (eq (vop-info-move-args (vop-info vop))
-                               :local-call)
-                           (reverse-find-vop 'allocate-frame vop)
-                           vop))
+    (multiple-value-bind (before block)
+        (if (eq (vop-info-move-args (vop-info vop)) :local-call)
+            (let ((before (reverse-find-vop 'allocate-frame vop)))
+              ;; Because of SPLIT-IR2-BLOCKS the ALLOCATE-FRAME VOP
+              ;; may be in a different block.
+              (values before (vop-block before)))
+            (values vop block))
+      (emit-operand-load node block tn save before))
     (emit-operand-load node block save tn next)))
 
 ;;; Return a VOP after which is an OK place to save the value of TN.
@@ -841,7 +841,6 @@
 
 ;;;; load TN packing
 
-#-sb-devel
 (declaim (start-block pack-load-tns load-tn-conflicts-in-sc))
 
 ;;; These variables indicate the last location at which we computed
@@ -1211,7 +1210,8 @@
     (let ((target (tn-ref-target op))
           (tn (tn-ref-tn op)))
       (when (and target
-                 (not (eq (tn-kind tn) :unused)))
+                 (not (eq (tn-kind tn) :unused))
+                 (tn-primitive-type tn))
         (let* ((load-tn (tn-ref-load-tn op))
                (load-scs (svref (car scs)
                                 (sc-number
@@ -1228,7 +1228,8 @@
     (let ((target (tn-ref-target op))
           (tn (tn-ref-tn op)))
       (unless (or target
-                  (eq (tn-kind tn) :unused))
+                  (eq (tn-kind tn) :unused)
+                  (not (tn-primitive-type tn)))
         (let* ((load-tn (tn-ref-load-tn op))
                (load-scs (svref (car scs)
                                 (sc-number
@@ -1259,7 +1260,6 @@
 
 ;;;; targeting
 
-#-sb-devel
 (declaim (start-block pack pack-tn target-if-desirable
                       ;; needed for pack-iterative
                       pack-wired-tn))
@@ -1479,43 +1479,29 @@
     ;; For non-x86 ports the presence of a save-tn associated with a
     ;; tn is used to identify the old-fp and return-pc tns. It depends
     ;; on the old-fp and return-pc being passed in registers.
-    #-fp-and-pc-standard-save
-    (when (and (not (eq (tn-kind tn) :specified-save))
+    (when (and #-fp-and-pc-standard-save
+               (not (eq (tn-kind tn) :specified-save))
                (conflicts-in-sc original sc offset))
-      (error "~S is wired to a location that it conflicts with." tn))
+      (error "~S is wired to location ~D in SC ~A of kind ~S that it conflicts with."
+             tn offset sc (tn-kind tn)))
 
     ;; Use the above check, but only print a verbose warning. This can
     ;; be helpful for debugging the x86 port.
     #+nil
     (when (and (not (eq (tn-kind tn) :specified-save))
                (conflicts-in-sc original sc offset))
-          (format t "~&* Pack-wired-tn possible conflict:~%  ~
+      (format t "~&* Pack-wired-tn possible conflict:~%  ~
                      tn: ~S; tn-kind: ~S~%  ~
                      sc: ~S~%  ~
                      sb: ~S; sb-name: ~S; sb-kind: ~S~%  ~
                      offset: ~S; end: ~S~%  ~
                      original ~S~%  ~
                      tn-save-tn: ~S; tn-kind of tn-save-tn: ~S~%"
-                  tn (tn-kind tn) sc
-                  sb (sb-name sb) (sb-kind sb)
-                  offset end
-                  original
-                  (tn-save-tn tn) (tn-kind (tn-save-tn tn))))
-
-    ;; On the x86 ports the old-fp and return-pc are often passed on
-    ;; the stack so the above hack for the other ports does not always
-    ;; work. Here the old-fp and return-pc tns are identified by being
-    ;; on the stack in their standard save locations.
-    #+fp-and-pc-standard-save
-    (when (and (not (and
-                     (= (sc-number sc) #.(sc+offset-scn old-fp-passing-offset))
-                     (= offset #.(sc+offset-offset old-fp-passing-offset))))
-               (not (and
-                     (= (sc-number sc) #.(sc+offset-scn return-pc-passing-offset))
-                     (= offset #.(sc+offset-offset return-pc-passing-offset))))
-               (conflicts-in-sc original sc offset))
-      (error "~S is wired to location ~D in SC ~A of kind ~S that it conflicts with."
-             tn offset sc (tn-kind tn)))
+              tn (tn-kind tn) sc
+              sb (sb-name sb) (sb-kind sb)
+              offset end
+              original
+              (tn-save-tn tn) (tn-kind (tn-save-tn tn))))
 
     (unless (eq (sb-kind sb) :unbounded)
       (setf (ldb (byte 1 (truly-the sb-vm:finite-sc-offset offset))
