@@ -13,7 +13,32 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!X86-64-ASM")
+(in-package "SB-X86-64-ASM")
+
+(defun sb-disassem::pre-decode (chunk dstate)
+  (let ((byte (ldb (byte 8 0) chunk)))
+    (case byte
+      ((#x64  ; FS:
+        #x65  ; GS:
+        #x66  ; operand size modifier
+        #x67  ; address size modifier
+        #xf0  ; LOCK
+        #xf2  ; REPNE or SSE inst
+        #xf3) ; REP or SSE inst
+       ;; If the next byte is a REX prefix, then strip it out, recording the 'wrxb'
+       ;; bits in the dstate, and return the chunk as if the REX byte were absent.
+       (let ((next (ldb (byte 8 8) chunk)))
+         (when (= (logand next #xf0) #x40)
+           (dstate-setprop dstate (logior +rex+ (logand next #b1111)))
+           (let ((new (logior byte (ash (ldb (byte 48 16) chunk) 8))))
+             (return-from sb-disassem::pre-decode (values new 1))))))))
+  (values chunk 0))
+
+(defmethod print-object ((reg reg) stream)
+  (if *print-readably*
+      ;; cross-compiled DEFMETHOD can't use call-next-method
+      (default-structure-print reg stream *current-level-in-print*)
+      (write-string (reg-name reg) stream)))
 
 ;;; Return the operand size depending on the prefixes and width bit as
 ;;; stored in DSTATE.
@@ -56,19 +81,26 @@
 ;; because it has no field specification and no prefilter.
 ;; (It's specified directly in the MOV instruction definition)
 (defun reg-imm-data (dchunk dstate) dchunk
-  (aref (sb!disassem::dstate-filtered-values dstate) 4))
+  (aref (sb-disassem::dstate-filtered-values dstate) 4))
 
-(defstruct (machine-ea (:include sb!disassem::filtered-arg)
-                       (:copier nil)
+(defstruct (machine-ea (:copier nil)
                        (:constructor %make-machine-ea))
-  base disp index scale)
+  ;; possible TODO: base,index,scale could be packed thusly in 13 bits:
+  ;;  2 bits for scale
+  ;;  1 bit for base register non-NULL
+  ;;  4 bits for base register number
+  ;;  1 bit for base-register-is-RIP
+  ;;  1 bit for index register non-NULL
+  ;;  4 bits for index register number
+  disp base index scale)
+(declaim (freeze-type machine-ea))
 
 (defun reg-num (reg) (reg-id-num (reg-id reg)))
 
 ;;; Print to STREAM the name of the general-purpose register encoded by
 ;;; VALUE and of size WIDTH.
 (defun print-reg-with-width (value width stream dstate)
-  (declare (type stream stream)
+  (declare (type (or null stream) stream)
            (type disassem-state dstate))
   (let* ((num (etypecase value
                ((unsigned-byte 4) value)
@@ -81,7 +113,9 @@
                                 (<= 4 num 7))
                            (+ 16 -4 num) ; legacy high-byte register
                            num))))
-    (princ (reg-name reg) stream))
+    (if stream
+        (princ (reg-name reg) stream)
+        (operand reg dstate)))
   ;; XXX plus should do some source-var notes
   )
 
@@ -156,36 +190,101 @@
   (princ16 value stream))
 
 (defun print-xmmreg (value stream dstate)
-  (declare (type stream stream) (ignore dstate))
-  (write-string (reg-name (get-fpr
+  (let* ((reg (get-fpr :xmm
                            ;; FIXME: why are we seeing a value from the GPR
                            ;; prefilter instead of XMM prefilter here sometimes?
                            (etypecase value
                              ((unsigned-byte 4) value)
                              (reg (reg-num value)))))
-                stream))
+         (name (reg-name reg)))
+    (if stream
+        (write-string name stream)
+        (operand name dstate))))
 
 (defun print-xmmreg/mem (value stream dstate)
   (if (machine-ea-p value)
       (print-mem-ref :ref value nil stream dstate)
       (print-xmmreg value stream dstate)))
 
-;;; Find the Lisp object, if any, called by a "CALL rel32offs"
-;;; instruction format and add it as an end-of-line comment.
-;;; Alternatively, the value might be an immediate operand to MOV,
-;;; which in general decodes as a signed integer. So ignore it
-;;; unless it looks like an address.
-#!+immobile-space
-(defun maybe-note-lisp-callee (value dstate)
-  (awhen (and (typep value 'word) (sb!vm::find-called-object value))
+;;; Guess whether VALUE is an immobile-space symbol or code blob by looking
+;;; at all code header constants. If it matches any constant, assume that it
+;;; is a use of the constant.  This has false positives of course,
+;;; as does MAYBE-NOTE-STATIC-SYMBOL and friends. Any random immediate value
+;;; used in an unboxed context, such as an ADD instruction,
+;;; can be wrongly construed as an address.
+;;; Note that for symbols we can match either the tagged pointer to it
+;;; OR the untagged address of the SYMBOL-VALUE slot.
+;;;
+;;; "static" in this usage implies "at a fixed address" - it could be
+;;; in static space or immobile space.
+;;;
+;;; TODO: probably should take an &OPTIONAL for ALLOW-INTERIOR-PTR to
+;;; reject false positives from instructions that don't access an object
+;;; except through a tagged pointer.
+(defun maybe-note-static-lispobj (value dstate &optional quote)
+  (when (maybe-note-static-symbol value dstate)
+    ;; Returning T prints VALUE using base 16
+    ;; (see the SIGNED-IMM-DATA printer, PRINT-IMM/ASM-ROUTINE)
+    ;; This should probably pass through the QUOTE option but it's not critical.
+    (return-from maybe-note-static-lispobj t))
+  (let ((code (seg-code (dstate-segment dstate)))
+        (adjusted-val (logior (- value (ash sb-vm:symbol-value-slot sb-vm:word-shift))
+                              sb-vm:other-pointer-lowtag))
+        (found-const)
+        (slot))
+    (when code
+      (loop for i downfrom (1- (code-header-words code)) to sb-vm:code-constants-offset
+            for const = (code-header-ref code i)
+            do (when (symbolp const)
+                 (let ((addr (get-lisp-obj-address const)))
+                   (cond ((eql addr value)
+                          (return (setq found-const const)))
+                         ((eql addr adjusted-val)
+                          (return (setq found-const const
+                                        slot sb-vm:symbol-value-slot)))))))
+      (unless found-const ; try static symbol's value slots
+        (dovector (symbol sb-vm:+static-symbols+)
+          (when (= (get-lisp-obj-address symbol) adjusted-val)
+            (return (setq found-const symbol
+                          slot sb-vm:symbol-value-slot)))))
+      (when found-const
+        (note (cond (slot
+                     (lambda (s) (format s "(SYMBOL-VALUE '~S)" found-const)))
+                    ((and (symbolp found-const) quote)
+                     (lambda (s) (write-char #\' s) (prin1 found-const s)))
+                    (t
+                     (lambda (s) (prin1 found-const s))))
+              dstate)
+        ;; Returning T prints in base 16 (see PRINT-IMM/ASM-ROUTINE)
+        (return-from maybe-note-static-lispobj t))))
+    #| This mysterious code seems to have no regression tests.
+       Comenting it out until I can figure out why it was in target-disassem
+    ;; Kludge: layout of STREAM, FILE-STREAM, and STRING-STREAM can be used
+    ;; as immediate operands without a corresponding boxed header constant.
+    ;; I think we always elide the boxed constant for builtin layouts,
+    ;; but these three have some slightly unusual codegen that causes a PUSH
+    ;; instruction to need some help to show its operand as a lisp object.
+    (dolist (thing (load-time-value (list (find-layout 'stream)
+                                          (find-layout 'file-stream)
+                                          (find-layout 'string-stream))
+                                    t))
+      (when (eql (get-lisp-obj-address thing) address)
+        (return-from found thing))))) |#
+  (awhen (and (typep value 'word)
+              (sb-disassem::find-code-constant-from-interior-pointer value dstate))
     (note (lambda (stream) (princ it stream)) dstate)))
 
+;;; Immobile symbols are heuristically detected with MAYBE-NOTE-STATIC-LISPOBJ
+;;; so that moving the tagged pointer to a symbol is shown thusly:
+;;; ";  BACF003C50       MOV EDX, #x503C00CF             ; '*MYVAR*"
+;;;
 (defun print-imm/asm-routine (value stream dstate)
-  (if (or #!+immobile-space (maybe-note-lisp-callee value dstate)
-          (maybe-note-assembler-routine value nil dstate)
-          (maybe-note-static-symbol value dstate))
-      (princ16 value stream)
-      (princ value stream)))
+  (cond ((not stream) (operand value dstate))
+        ((or (maybe-note-assembler-routine value nil dstate)
+             (maybe-note-static-lispobj value dstate t))
+         (princ16 value stream))
+        (t
+         (princ value stream))))
 
 ;;; Return an instance of REG or MACHINE-EA.
 ;;; MOD and R/M are the extracted bits from the instruction's ModRM byte.
@@ -197,7 +296,7 @@
            (type (unsigned-byte 3) r/m))
   (flet ((make-machine-ea (base &optional disp index scale)
            (let ((ea (the machine-ea
-                       (sb!disassem::new-filtered-arg dstate #'%make-machine-ea))))
+                       (sb-disassem::new-filtered-arg dstate #'%make-machine-ea))))
              (setf (machine-ea-base ea) base
                    (machine-ea-disp ea) disp
                    (machine-ea-index ea) index
@@ -214,7 +313,7 @@
       (cond ((= mod #b11) ; register direct mode
              (case regclass
               (gpr (get-gpr :qword full-reg)) ; size is not really known here
-              (fpr (get-fpr full-reg))))
+              (fpr (get-fpr :xmm full-reg))))
             ((= r/m #b100) ; SIB byte - rex.b is "don't care"
              (let* ((sib (the (unsigned-byte 8) (read-suffix 8 dstate)))
                     (index-reg (extend +rex-x+ (ldb (byte 3 3) sib)))
@@ -236,24 +335,18 @@
 (defun prefilter-xmmreg/mem (dstate mod r/m)
   (decode-mod-r/m dstate mod r/m 'fpr))
 
-#!+sb-thread
-(defun static-symbol-from-tls-index (index)
-  (dovector (sym +static-symbols+)
-    (when (= (symbol-tls-index sym) index)
-      (return sym))))
-
 ;;; Return contents of memory if either it refers to an unboxed code constant
 ;;; or is RIP-relative with a displacement of 0.
 (defun unboxed-constant-ref (dstate addr disp)
   (when (and (minusp disp)
              (awhen (seg-code (dstate-segment dstate))
-               (sb!disassem::points-to-code-constant-p addr it)))
+               (sb-disassem::points-to-code-constant-p addr it)))
     (sap-ref-word (int-sap addr) 0)))
 
 (define-load-time-global thread-slot-names
-    (let* ((slots (primitive-object-slots
-                   (find 'sb!vm::thread *primitive-objects*
-                         :key #'primitive-object-name)))
+    (let* ((slots (coerce (primitive-object-slots
+                           (sb-vm::primitive-object 'sb-vm::thread))
+                          'list))
            (a (make-array (1+ (slot-offset (car (last slots))))
                           :initial-element nil)))
       (dolist (slot slots a)
@@ -271,7 +364,7 @@
 ;;; but in other cases would only add clutter, since a source/destination
 ;;; register implies a size.
 ;;;
-(defun print-mem-ref (mode value width stream dstate)
+(defun print-mem-ref (mode value width stream dstate &key (index-reg-printer #'print-addr-reg))
   ;; :COMPUTE is used for the LEA instruction - it informs this function
   ;; that we're not loading from the address and that the contents should not
   ;; be printed. It'll usually be a reference to code within the disasembly
@@ -279,8 +372,14 @@
   (declare (type (member :ref :sized-ref :compute) mode)
            (type machine-ea value)
            (type (member nil :byte :word :dword :qword) width)
-           (type stream stream)
+           (type (or null stream) stream)
            (type disassem-state dstate))
+  ;; If disassembling into the dstate, print nothing; just stash the operand.
+  (when (null stream)
+    (return-from print-mem-ref
+      (operand (cons value width) dstate)))
+
+  ;; Unpack and print the pieces of the machine EA.
   (let ((base-reg (machine-ea-base value))
         (disp (machine-ea-disp value))
         (index-reg (machine-ea-index value))
@@ -290,6 +389,8 @@
       (princ '| PTR | stream))
     (when (dstate-getprop dstate +fs-segment+)
       (princ "FS:" stream))
+    (when (dstate-getprop dstate +gs-segment+)
+      (princ "GS:" stream))
     (write-char #\[ stream)
     (when base-reg
       (if (eql :rip base-reg)
@@ -298,7 +399,7 @@
       (setq firstp nil))
     (when index-reg
       (unless firstp (write-char #\+ stream))
-      (print-addr-reg index-reg stream dstate)
+      (funcall index-reg-printer index-reg stream dstate)
       (let ((scale (machine-ea-scale value)))
         (unless (= scale 1)
           (write-char #\* stream)
@@ -310,39 +411,44 @@
       (cond ((eq (machine-ea-base value) :rip)
              (princ disp stream))
             (firstp
-               (princ16 disp stream)
-               (or (minusp disp)
-                   (nth-value 1 (note-code-constant-absolute disp dstate))
-                   (maybe-note-assembler-routine disp nil dstate)
-                   ;; Static symbols coming from CELL-REF
-                   (maybe-note-static-symbol (+ disp (- other-pointer-lowtag
-                                                        n-word-bytes))
-                                             dstate)))
+             (princ16 disp stream)
+             ;; Avoid the MAYBE-NOTE call if we can.  A negative offset is never an
+             ;; absolute address as would be used for asm routines and static symbols.
+             ;; FIRSTP implies lack of a base and index register.
+             (unless (minusp disp)
+               (maybe-note-assembler-routine disp nil dstate)))
             (t
              (princ disp stream))))
     (write-char #\] stream)
+
+    ;; Always try to add an end-of-line comment about the EA.
+    ;; Assembler routines were already handled above (not really sure why)
+    ;; so now we have to figure out everything else.
+
     (when (and (eq (machine-ea-base value) :rip) (neq mode :compute))
-      ;; Always try to print the EA as a note
-      (let ((addr (+ disp (dstate-next-addr dstate))))
-        ;; The origin is zero when disassembling into a trace-file.
-        ;; Don't crash on account of it.
-        (when (plusp addr)
-          (or (nth-value
-               1 (note-code-constant-absolute addr dstate width))
-              ;; Don't try to look up C symbols in immobile space.
-              ;; In an elfinated core, the range that is reserved for
-              ;; compilation to memory says it is all associated with
-              ;; the symbol "lisp_jit_code" which is not useful.
-              (unless (sb!kernel:immobile-space-addr-p addr)
-                (maybe-note-assembler-routine addr nil dstate))
-              ;; Show the absolute address and maybe the contents.
-              (note (format nil "[#x~x]~@[ = #x~x~]"
-                            addr
-                            (case width
-                              (:qword
-                               (unboxed-constant-ref dstate addr disp))))
-                    dstate)))))
-    #!+sb-thread
+      (block nil
+       (binding* ((seg (dstate-segment dstate))
+                  (code (seg-code seg) :exit-if-null)
+                  (offs (sb-disassem::segment-offs-to-code-offs
+                         (+ (dstate-next-offs dstate) disp) seg)))
+           (when (note-code-constant offs dstate) (return)))
+       (let ((addr (+ disp (dstate-next-addr dstate))))
+         ;; The origin is zero when disassembling into a trace-file.
+         ;; Don't crash on account of it.
+         ;; Also, don't try to look up C symbols in immobile space.
+         ;; In an elfinated core, the range that is reserved for
+         ;; compilation to memory says it is all associated with
+         ;; the symbol "lisp_jit_code" which is not useful.
+         (when (plusp addr)
+           (or (unless (sb-kernel:immobile-space-addr-p addr)
+                 (maybe-note-assembler-routine addr nil dstate))
+               ;; Show the absolute address and maybe the contents.
+               (note (format nil "[#x~x]~@[ = #x~x~]"
+                             addr
+                             (case width
+                              (:qword (unboxed-constant-ref dstate addr disp))))
+                     dstate))))))
+
     (flet ((guess-symbol (predicate)
              (binding* ((code-header (seg-code (dstate-segment dstate)) :exit-if-null)
                         (header-n-words (code-header-words code-header)))
@@ -350,18 +456,30 @@
                      for obj = (code-header-ref code-header word-num)
                      when (and (symbolp obj) (funcall predicate obj))
                      do (return obj)))))
+      (when (and (not base-reg) (not index-reg) disp)
+        (let ((addr (+ disp ; guess that DISP points to a symbol-value slot
+                       (- (ash sb-vm:symbol-value-slot sb-vm:word-shift))
+                       sb-vm:other-pointer-lowtag)))
+          (awhen (guess-symbol (lambda (s) (= (get-lisp-obj-address s) addr)))
+            (note (lambda (stream) (prin1 it stream)) dstate)
+            (return-from print-mem-ref))))
       ;; Try to reverse-engineer which thread-local binding this is
+      #+sb-thread
       (cond ((and disp ; Test whether disp looks aligned to an object header
-                  (not (logtest (- disp 4) sb!vm:lowtag-mask))
+                  (not (logtest (- disp 4) sb-vm:lowtag-mask))
                   (not base-reg) (not index-reg))
-             (let* ((addr (+ disp (- 4) sb!vm:other-pointer-lowtag))
+             (let* ((addr (+ disp (- 4) sb-vm:other-pointer-lowtag))
                     (symbol
                      (guess-symbol (lambda (s) (= (get-lisp-obj-address s) addr)))))
                (when symbol
+                 ;; "tls_index:" is access to the half-sized slot within the
+                 ;; symbol header that provides an offset into TLS.
                  (note (lambda (stream) (format stream "tls_index: ~S" symbol))
                        dstate))))
-            ((and (eql base-reg #.(tn-offset sb!vm::thread-base-tn))
-                  (not (dstate-getprop dstate +fs-segment+)) ; not system TLS
+            ;; thread slots
+            ((and (eql base-reg sb-vm::thread-reg)
+                  #+gs-seg (dstate-getprop dstate +gs-segment+)
+                  #-gs-seg (not (dstate-getprop dstate +fs-segment+)) ; not system TLS
                   (not index-reg) ; no index
                   (typep disp '(integer 0 *)) ; positive displacement
                   (zerop (logand disp 7))) ; lispword-aligned
@@ -371,15 +489,17 @@
                    (return-from print-mem-ref
                      (note (lambda (stream) (format stream "thread.~(~A~)" it))
                            dstate)))))
-             (let* ((tls-index (ash disp (- n-fixnum-tag-bits)))
-                    (symbol (or (guess-symbol
-                                 (lambda (s) (= (symbol-tls-index s) tls-index)))
-                                ;; static symbols aren't in the code header
-                                (static-symbol-from-tls-index tls-index))))
+             (let ((symbol (or (guess-symbol
+                                (lambda (s) (= (symbol-tls-index s) disp)))
+                               ;; static symbols aren't in the code header
+                               (find disp +static-symbols+
+                                     :key #'symbol-tls-index))))
                (when symbol
                  (return-from print-mem-ref
+                   ;; "tls:" refers to the current value of the symbol in TLS
                    (note (lambda (stream) (format stream "tls: ~S" symbol))
-                         dstate)))))))))
+                         dstate)))))
+            ))))
 
 (defun lea-compute-label (value dstate)
   ;; If VALUE should be regarded as a label, return the address.
@@ -391,44 +511,53 @@
 ;; Figure out whether LEA should print its EA with just the stuff in brackets,
 ;; or additionally show the EA as either a label or a hex literal.
 (defun lea-print-ea (value stream dstate)
-  (let ((width (inst-operand-size dstate))
-        (addr nil)
-        (fmt "= #x~x"))
-    (etypecase value
-      (machine-ea
-       ;; Indicate to PRINT-MEM-REF that this is not a memory access.
-       (print-mem-ref :compute value width stream dstate)
-       (when (eq (machine-ea-base value) :rip)
-         (setq addr (+ (dstate-next-addr dstate) (machine-ea-disp value)))))
+  (let* ((width (inst-operand-size dstate))
+         (addr
+          (etypecase value
+           (machine-ea
+            ;; Indicate to PRINT-MEM-REF that this is not a memory access.
+            (print-mem-ref :compute value width stream dstate)
+            (when (eq (machine-ea-base value) :rip)
+              (+ (dstate-next-addr dstate) (machine-ea-disp value))))
 
-      ;; LEA Rx,Ry is an illegal encoding, but we'll show it as-is.
-      (reg (print-reg-with-width value width stream dstate))
+           ((or string integer)
+            ;; A label for the EA should not print as itself, but as the decomposed
+            ;; addressing mode so that [ADDR] and [RIP+disp] are unmistakable.
+            ;; We can see an INTEGER here because LEA-COMPUTE-LABEL is always called
+            ;; on the operand to LEA, and it will compute an absolute address based
+            ;; off RIP when possible. If :use-labels NIL was specified, there is
+            ;; no hashtable of address to string, so we get the address.
+            ;; But ordinarily we get the string. Either way, the r/m arg reveals the
+            ;; EA calculation. DCHUNK-ZERO is a meaningless value - any would do -
+            ;; because the EA was computed in a prefilter.
+            ;; (the instruction format is known because LEA has exactly one format)
+            (print-mem-ref :compute (regrm-inst-r/m dchunk-zero dstate)
+                           width stream dstate)
+            value)
 
-      ((or string integer)
-       ;; A label for the EA should not print as itself, but as the decomposed
-       ;; addressing mode so that [ADDR] and [RIP+disp] are unmistakable.
-       ;; We can see an INTEGER here because LEA-COMPUTE-LABEL is always called
-       ;; on the operand to LEA, and it will compute an absolute address based
-       ;; off RIP when possible. If :use-labels NIL was specified, there is
-       ;; no hashtable of address to string, so we get the address.
-       ;; But ordinarily we get the string. Either way, the r/m arg reveals the
-       ;; EA calculation. DCHUNK-ZERO is a meaningless value - any would do -
-       ;; because the EA was computed in a prefilter.
-       (print-mem-ref :compute (regrm-inst-r/m dchunk-zero dstate)
-                      width stream dstate)
-       (setq addr value)
-       (when (stringp value) (setq fmt "= ~A"))))
-    (when addr
+           ;; LEA Rx,Ry is an illegal encoding, but we'll show it as-is.
+           ;; When we used integers instead of REG to represent registers, this case
+           ;; overlapped with the preceding. It's nice that it no longer does.
+           (reg
+            (print-reg-with-width value width stream dstate)
+            nil))))
+
+    (when (and addr stream) ; no end-of-line comments if storing into dstate
       (unless (maybe-note-assembler-routine addr nil dstate)
-        (note (lambda (s) (format s fmt addr)) dstate)))))
+        (note (lambda (s) (format s (if (stringp addr) "= ~A" "= #x~x") addr))
+              dstate)))))
 
 ;;;; interrupt instructions
 
 (defun break-control (chunk inst stream dstate)
-  (declare (ignore inst))
+  ;; Do not parse bytes following a trap instruction unless it belongs to lisp code.
+  ;; C++ compilers will emit ud2 for various reasons.
+  (when (sb-disassem::dstate-foreign-code-p dstate)
+    (return-from break-control))
   (flet ((nt (x) (if stream (note x dstate))))
-    (let ((trap #!-ud2-breakpoints (byte-imm-code chunk dstate)
-           #!+ud2-breakpoints (word-imm-code chunk dstate)))
+    (let ((trap (if (eq (sb-disassem::inst-print-name inst) 'ud2)
+                    (word-imm-code chunk dstate)
+                    (byte-imm-code chunk dstate))))
      (case trap
        (#.breakpoint-trap
         (nt "breakpoint trap"))
@@ -444,26 +573,21 @@
         (nt "single-step trap (before)"))
        (#.invalid-arg-count-trap
         (nt "Invalid argument count trap"))
-       (#.cerror-trap
-        (nt "cerror trap")
-        (handle-break-args #'snarf-error-junk trap stream dstate))
        (t
-        (handle-break-args #'snarf-error-junk trap stream dstate))))))
-
-(defun sb!c::convert-alloc-point-fixups (code locs)
-  ;; Find the instruction which jumps over the profiling code,
-  ;; and record the offset, and not the instruction that makes the call
-  ;; to enable the counter. The instructions preceding the call comprise
-  ;; a test, jmp, and long nop. Luckily a long nop encoding never
-  ;; has the byte #xEB in it, so just scan backwards looking for that.
-  (pack-code-fixup-locs
-   (mapcar (lambda (loc)
-             (loop (cond ((zerop (decf loc))
-                          (bug "Failed to find allocation point"))
-                         ((eql (sap-ref-8 (code-instructions code) loc) #xEB)
-                          (return loc)))))
-           locs)
-   nil))
+        (when (or (and (= trap cerror-trap) (progn (nt "cerror trap") t))
+                  (>= trap uninitialized-load-trap))
+          (handle-break-args
+           (lambda (sap offset trap-number length-only)
+             (if (= trap-number uninitialized-load-trap)
+                 (let ((reg (ash (sap-ref-8 sap offset) -2)))
+                   ;; decode a single byte arg, not an SC+OFFSET
+                   (values (error-number-or-lose 'uninitialized-memory-error)
+                           1     ; total number of bytes consumed after the trap
+                           (list (make-sc+offset unsigned-reg-sc-number reg))
+                           '(1)  ; display 1 byte for the register encoding
+                           nil)) ; no error number after the trap instruction
+                 (snarf-error-junk sap offset trap-number length-only)))
+           trap stream dstate)))))))
 
 ;;; Disassemble memory of CODE from START-ADDRESS for LENGTH bytes
 ;;; calling FUNCTION on each instruction that has a PC-relative operand.
@@ -478,6 +602,7 @@
          (jmp-inst (find-inst #xE9 inst-space))
          (cond-jmp-inst (find-inst #x800f inst-space))
          (lea-inst (find-inst #x8D inst-space))
+         (address (get-lisp-obj-address code))
          (text-start (sap-int (code-instructions code)))
          (text-end (+ text-start (%code-text-size code)))
          (sap (int-sap start-address)))
@@ -488,7 +613,7 @@
      (lambda (dchunk inst)
        (flet ((includep (target)
                 ;; Self-relative (to the code object) operands are ignored.
-                (and (or (< target text-start) (>= target text-end))
+                (and (or (< target address) (>= target text-end))
                      (funcall predicate target))))
          (cond ((or (eq inst jmp-inst) (eq inst call-inst))
                 (let ((operand (+ (near-jump-displacement dchunk dstate)
@@ -497,8 +622,8 @@
                     (funcall function (+ (dstate-cur-offs dstate) 1)
                              operand inst))))
                ((eq inst cond-jmp-inst)
-                ;; TAIL-CALL-SYMBOL is invoked with a conditional jump
-                ;; (but not CALL-SYMBOL because only JMP can be conditional)
+                ;; jmp CALL-SYMBOL is invoked with a conditional jump
+                ;; (but not call CALL-SYMBOL because only JMP can be conditional)
                 (let ((operand (+ (near-cond-jump-displacement dchunk dstate)
                                   (dstate-next-addr dstate))))
                   (when (includep operand)
@@ -524,7 +649,7 @@
 ;;; signatures, each of which is cons of a vector of instruction bytes with some
 ;;; replaced by 0, plus an opaque set of integers that need to be compared for
 ;;; equality to test whether the blobs of code are functionally equivalent.
-(defun sb!vm::compute-code-signature (code dstate &aux result)
+(defun sb-vm::compute-code-signature (code dstate &aux result)
   (dotimes (i (code-n-entries code) result)
     (let* ((f (%code-entry-point code i))
            (len (%simple-fun-text-len f i))
@@ -550,16 +675,16 @@
       (push (cons buffer (coerce operand-values 'simple-vector)) result))))
 
 ;;; Perform ICF on instructions of CODE
-(defun sb!vm::machine-code-icf (code mapper replacements print)
+(defun sb-vm::machine-code-icf (code mapper replacements print)
   (declare (ignorable code mapper replacements print))
-  #!+immobile-space
+  #+immobile-space
   (flet ((scan (sap length dstate segment)
            (scan-relative-operands
             code (sap-int sap) length dstate segment
             (lambda (offset operand inst)
               (declare (ignorable inst))
               (let ((lispobj (when (immobile-space-addr-p operand)
-                               (sb!vm::find-called-object operand))))
+                               (sb-vm::find-called-object operand))))
                 (when (functionp lispobj)
                   (let ((replacement (funcall mapper lispobj)))
                     (unless (eq replacement lispobj)
@@ -570,8 +695,8 @@
                              (old-rel32 (signed-sap-ref-32 sap offset))
                              (new-rel32 (the (signed-byte 32) (+ old-rel32 disp))))
                         (setf (signed-sap-ref-32 sap offset) new-rel32))))))))))
-    (if (eq code sb!fasl:*assembler-routines*)
-        (multiple-value-bind (start end) (sb!fasl::calc-asm-routine-bounds)
+    (if (eq code sb-fasl:*assembler-routines*)
+        (multiple-value-bind (start end) (sb-fasl::calc-asm-routine-bounds)
           (scan (sap+ (code-instructions code) start)
                 (- end start)
                 (make-dstate)
@@ -582,7 +707,8 @@
                     thereis (let ((obj (code-header-ref code i)))
                               (typecase obj
                                 (fdefn (awhen (fdefn-fun obj)
-                                         (gethash (fun-code-header it) replacements)))
+                                         (gethash (fun-code-header (%fun-fun it))
+                                                  replacements)))
                                 (simple-fun
                                  (gethash (fun-code-header obj) replacements)))))
           (let ((dstate (make-dstate))
