@@ -13,6 +13,80 @@
 
 ;;;; register specs
 
+;;; Observe that [R12 + displacement] needs an extra byte of encoding in
+;;; certain addressing modes, as does [R13 + index]
+;;;     49895B08         MOV [R11+8], RBX
+;;;     49895C2408       MOV [R12+8], RBX
+;;;     49895D08         MOV [R13+8], RBX
+;;;     49891C03         MOV [R11+RAX], RBX
+;;;     49891C04         MOV [R12+RAX], RBX
+;;;     49895C0500       MOV [R13+RAX], RBX
+;;; This is because 12 and 13 are equal-mod-8 to 4 and 5, the indices of RSP
+;;; and RBP respectively, which have different behaviors in the ModRegR/M byte.
+;;; So due to lack of perfectly orthogonal encodings, it would be slightly
+;;; beneficial to hint to the register allocator that it should pick registers
+;;; 12 and 13 only when all other registers are unavailable.
+;;; And you might think that it should "work" simply to list the usable offsets
+;;; in the SC in the order in which they should be chosen when the register
+;;; allocator needs to pick the next available register, but unfortunately
+;;; it does not work to do that, because it uses bitmaps from which to pick,
+;;; and not an ordered list.
+
+;;; A register that's never used by the code generator, and can therefore
+;;; be used as an assembly temporary in cases where a VOP :TEMPORARY can't
+;;; be used.
+;;; This is only enabled for certain build configurations that need
+;;; a scratch register in various random places.
+#-ubsan (eval-when (:compile-toplevel :load-toplevel :execute)
+          (defconstant global-temp-reg nil))
+#+ubsan (progn (define-symbol-macro temp-reg-tn r11-tn)
+               (defconstant global-temp-reg 11))
+
+#+gs-seg
+(progn
+  ;; There is no permanent THREAD-TN. There are a few problems in trying
+  ;; cleverly to make PSEUDO-ATOMIC agnostic of whether you're using a global
+  ;; THREAD-TN or a local one (a global symbol macro vs. a let binding)
+  ;; 1) ECL seems to think that it is an error to LET bind a global symbol macro.
+  ;;    > (define-symbol-macro thread-tn 'wat)
+  ;;    > (defun f (&optional (thread-tn thread-tn)) (format t "Hi: ~a~%" thread-tn))
+  ;;    > (compile'f)
+  ;;    ;;; Error:
+  ;;    ;;;   * The constant THREAD-TN is being bound.
+  ;; 2) It actually doesn't completely work completely anyway,
+  ;;    because pseudo-atomic needs to know not to use a segment override,
+  ;;    and TNs don't indicate a memory segment.
+  ;; While the second problem could be solved, the first can't,
+  ;; and it's not clear to me whether ECL is wrong to say it's an error,
+  ;; or SBCL is remiss in choosing not to style-warn (which would annoy me
+  ;; for that reason).
+  ;; So we wire the thread structure to r15 within some vops when convenient.
+  ;; Wiring to r15 is not strictly necessary, but I want to see if sb-aprof
+  ;; can be made to work slightly easier regardless of #+gs-seg.
+  ;; And with 15 being the highest register number, it is the least likely
+  ;; to be picked by the register allocator for anything else.
+  ;; Additionally I'd like to have an optimization pass that avoids reloading
+  ;; r15 when consecutive allocation sequences occur in straight-line code.
+  (define-symbol-macro thread-segment-reg :gs)
+  (define-symbol-macro thread-reg nil))
+#-gs-seg
+(progn
+  (define-symbol-macro thread-segment-reg :cs)
+  ;; r13 is preferable to r12 because 12 is an alias of 4 in ModRegRM, which
+  ;; implies use of a SIB byte with no index register for fixed displacement.
+  (define-symbol-macro thread-reg 13)
+  (define-symbol-macro thread-tn r13-tn))
+
+;;; Kludge needed for decoding internal error args - we assume that the
+;;; shadow memory is pointed to by this register (RAX).
+(defconstant msan-temp-reg-number 0)
+
+;;; The encoding anomaly for r12 makes it a perfect choice for the card table base.
+;;; It will never be used with a constant displacement.
+(define-symbol-macro card-table-reg 12)
+(define-symbol-macro gc-card-table-reg-tn r12-tn)
+(define-symbol-macro card-index-mask (make-fixup nil :gc-barrier))
+
 (macrolet ((defreg (name offset size)
              (declare (ignore size))
              `(eval-when (:compile-toplevel :load-toplevel :execute)
@@ -45,8 +119,9 @@
                              array))))
                 (defglobal ,offsets-list
                     (remove-if (lambda (x)
-                                 (member x `(,r11-offset ; temp reg
-                                             ,r13-offset ; thread base
+                                 (member x `(,global-temp-reg ; if there is one
+                                             ,thread-reg      ; if using a GPR
+                                             ,card-table-reg
                                              ,rsp-offset
                                              ,rbp-offset)))
                                (loop for i below 16 collect i))))))
@@ -377,11 +452,6 @@
   (and (tn-p thing)
        (eq (sb-name (sc-sb (tn-sc thing))) 'stack)))
 
-;; A register that's never used by the code generator, and can therefore
-;; be used as an assembly temporary in cases where a VOP :TEMPORARY can't
-;; be used.
-(define-symbol-macro temp-reg-tn r11-tn)
-
 ;;; TNs for registers used to pass arguments
 ;;; This can't be a DEFCONSTANT-EQX, for a similar reason to above, but worse.
 ;;; Among the problems, RECEIVE-UNKNOWN-VALUES uses (FIRST *REGISTER-ARG-TNS*)
@@ -393,10 +463,6 @@
             (symbol-value (symbolicate register-arg-name "-TN")))
           *register-arg-names*))
 
-;; r13 is preferable to r12 because 12 is an alias of 4 in ModRegRM, which
-;; implies use of a SIB byte with no index register for fixed displacement.
-(define-symbol-macro thread-base-tn r13-tn)
-
 ;;; If value can be represented as an immediate constant, then return
 ;;; the appropriate SC number, otherwise return NIL.
 (defun immediate-constant-sc (value)
@@ -419,8 +485,9 @@
                ;; is used to determine whether it's immediate.
                #+(and (not sb-xc-host) immobile-space (not immobile-symbols))
                (or (logbitp +initial-core-symbol-bit+ (get-header-data value))
-                   (and (sb-c::core-object-p sb-c::*compile-object*)
-                        (immobile-space-obj-p value)))
+                   (locally (declare (notinline sb-c::producing-fasl-file))
+                     (and (not (sb-c::producing-fasl-file))
+                          (immobile-space-obj-p value))))
 
                (static-symbol-p value))
        immediate-sc-number))
