@@ -124,7 +124,7 @@
 
 ;;; Return VALUE if it is not the unbound marker, otherwise executes ELSE:
 (defmacro non-empty-or (value else)
-  (with-unique-names (n-value)
+  (let ((n-value '#:val))
     `(let ((,n-value ,value))
        (if (unbound-marker-p ,n-value)
            ,else
@@ -157,9 +157,10 @@
 ;;; Perhaps we can dynamically optimize the mixing by picking bits that work
 ;;; best for the set of keys in the cache.
 (declaim (inline compute-cache-index))
+(export 'compute-cache-index) ; for a test
 (defun compute-cache-index (cache layouts)
   (macrolet ((fetch-hash (x)
-               `(let ((hash (layout-clos-hash ,x)))
+               `(let ((hash (wrapper-clos-hash ,x)))
                   (when (zerop hash) (return-from compute-cache-index nil))
                   hash)))
     (let ((index (fetch-hash (car layouts))))
@@ -196,12 +197,19 @@
 ;;; In other words, produces inlined code for COMPUTE-CACHE-INDEX when
 ;;; number of keys and presence of values in the cache is known
 ;;; beforehand.
+(defglobal *hash-vars*
+  (make-array 6 :initial-contents
+              (loop for i from 0 below 6 collect (make-symbol (format nil "H~D" i)))))
 (defun emit-cache-lookup (cache-var layout-vars miss-tag value-var)
   (declare (muffle-conditions code-deletion-note))
-  (with-unique-names (probe n-vector n-depth n-mask
+  (multiple-value-bind (probe n-vector n-depth n-mask
                       MATCH-WRAPPERS EXIT-WITH-HIT)
+      (values '#:probe '#:vect '#:depth '#:mask '#:try '#:hit)
     (let* ((num-keys (length layout-vars))
-           (hash-vars (make-gensym-list num-keys))
+           (hash-vars (loop for i from 0 below num-keys
+                            collect (if (< i #.(length *hash-vars*))
+                                        (aref *hash-vars* i)
+                                        (make-symbol (format nil "H~D" i)))))
            (pointer
             ;; We don't need POINTER if the cache has 1 key and no value,
             ;; or if FOLD-INDEX-ADDRESSING is supported, in which case adding
@@ -210,7 +218,7 @@
             (when (or (> num-keys 1) value-var) (make-symbol "PTR")))
            (line-size (power-of-two-ceiling (+ num-keys (if value-var 1 0)))))
       ;; Why not use PROG* ? are we expressly trying to avoid a new block?
-      `(let* (,@(mapcar (lambda (x y) `(,x (layout-clos-hash ,y))) hash-vars layout-vars)
+      `(let* (,@(mapcar (lambda (x y) `(,x (wrapper-clos-hash ,y))) hash-vars layout-vars)
               (,n-mask (cache-mask ,cache-var))
               (,probe (if (zerop ,(cache-mixer-expression 'logand hash-vars t))
                           (go ,miss-tag)
@@ -277,7 +285,7 @@
       (case key-count
        (1
         (let* ((layout (if (%instancep key) key (car key)))
-               (hash (layout-clos-hash layout))
+               (hash (wrapper-clos-hash layout))
                (index (logand hash mask)))
           (unless (= hash 0)
             (probe-loop (eq (svref vector (truly-the index index)) layout)))))
@@ -419,14 +427,14 @@
       ;; Check if the line is in use, and check validity of the keys.
       (let ((key1 (svref vector index)))
         (when (cache-key-p key1)
-          (if (zerop (layout-clos-hash key1))
+          (if (zerop (wrapper-clos-hash key1))
               ;; First key invalid.
               (return-from cache-has-invalid-entries-p t)
               ;; Line is in use and the first key is valid: check the rest.
               (loop for offset from 1 below key-count
                     do (let ((thing (svref vector (+ index offset))))
                          (when (or (not (cache-key-p thing))
-                                   (zerop (layout-clos-hash thing)))
+                                   (zerop (wrapper-clos-hash thing)))
                            ;; Incomplete line or invalid layout.
                            (return-from cache-has-invalid-entries-p t)))))))
       ;; Line empty of valid, onwards.
@@ -487,7 +495,7 @@
   cache)
 
 ;;; Copying a cache without expanding it is very much like mapping it:
-;;; we need to be carefull because there may be updates while we are
+;;; we need to be careful because there may be updates while we are
 ;;; copying it, and we don't want to copy incomplete entries or invalid
 ;;; ones.
 (defun copy-cache (cache)
@@ -569,7 +577,7 @@
          (total-lines (/ size line-size))
          (total-n-keys 0) ; a "key" is a tuple of layouts
          (n-dirty 0) ; lines that have an unbound marker but are not wholly empty
-         (n-obsolete 0) ; lines that need to be evicted due to 0 in a layout-clos-hash
+         (n-obsolete 0) ; lines that need to be evicted due to 0 in a wrapper-clos-hash
          (histogram
            (when compute-histogram
              (make-array (1+ (cache-limit cache)) :initial-element 0))))
@@ -582,7 +590,7 @@
                         (n-misses 0))
                    (cond ((find-if-not #'cache-key-p layouts)
                           (incf n-dirty))
-                         ((find 0 layouts :key #'layout-clos-hash)
+                         ((find 0 layouts :key #'wrapper-clos-hash)
                           (incf n-obsolete))
                          (t
                           (incf total-n-keys)
