@@ -118,8 +118,56 @@
                                               method-alist
                                               wrappers))))
 
+;;; methods-tracing TODO:
+;;;
+;;; 2. tracing method calls for non-fast-method-function calls
+;;;    - [DONE] the calls themselves
+;;;    - calls to the METHOD-FUNCTION of methods with fast functions
+;;;      (e.g. from something implementing CALL-NEXT-METHOD; handle this with
+;;;      some more smarts in %METHOD-FUNCTION objects?)
+;;;    - calls to the METHOD-FUNCTION of methods without fast functions
+;;;      (TRACE :METHODS T /could/ modify the METHOD-FUNCTION slot)
+;;; 4. tracing particular methods
+;;;    - need an interface.
+;;;      * (trace (method foo :around (t)))? [ how to trace the method and not
+;;;        the generic function as a whole?]
+;;;      * (trace :methods '((:around (t))) foo)? [probably not, interacts
+;;;        poorly with TRACE arg handling]
+;;; 5. supporting non-munged arguments as an option
+
+(defun method-trace-name (gf method)
+  ;; KLUDGE: we abuse NIL as second argument to mean that this is a
+  ;; combined method (i.e. something resulting from MAKE-METHOD in a
+  ;; method combination, rather than CALL-METHOD on a method object).
+  (if method
+      `(method ,(generic-function-name gf)
+               ,@(method-qualifiers method)
+               ,(unparse-specializers gf (method-specializers method)))
+      `(combined-method ,(generic-function-name gf))))
+
+(defun maybe-trace-method (gf method fun fmf-p)
+  (let ((m-name (when (plusp (hash-table-count sb-debug::*traced-funs*))
+                  ;; KLUDGE: testing if *TRACE-FUNS* has anything anything to
+                  ;; avoid calling METHOD-TRACE-NAME during PCL bootstrapping
+                  ;; when the generic-function type is not yet defined.)
+                  (method-trace-name gf method))))
+    (when m-name
+      (sb-debug::retrace-local-funs m-name))
+    (let ((info (when m-name
+                  (or (gethash m-name sb-debug::*traced-funs*)
+                      (let ((gf-info (gethash (or (generic-function-name gf) gf)
+                                              sb-debug::*traced-funs*)))
+                        (when (and gf-info (sb-debug::trace-info-methods gf-info))
+                          (let ((copy (copy-structure gf-info)))
+                            (setf (sb-debug::trace-info-what copy) m-name)
+                            copy)))))))
+      (if info
+          (lambda (&rest args)
+            (apply #'sb-debug::trace-method-call info fun fmf-p args))
+          fun))))
+
 (defun make-emf-from-method
-    (method cm-args &optional gf fmf-p method-alist wrappers)
+    (gf method cm-args fmf-p &optional method-alist wrappers)
   ;; Avoid style-warning about compiler-macro being unavailable.
   (declare (notinline make-instance))
   (multiple-value-bind (mf real-mf-p fmf pv)
@@ -132,12 +180,13 @@
                       fmf-p method-alist wrappers))
                (arg-info (method-plist-value method :arg-info))
                (default (cons nil nil))
-               (value (method-plist-value method :constant-value default)))
+               (value (method-plist-value method :constant-value default))
+               (fun (maybe-trace-method gf method fmf t)))
           (if (eq value default)
-              (make-fast-method-call :function fmf :pv pv
-                                     :next-method-call next :arg-info arg-info)
+              (make-fast-method-call
+               :function fun :pv pv :next-method-call next :arg-info arg-info)
               (make-constant-fast-method-call
-               :function fmf :pv pv :next-method-call next
+               :function fun :pv pv :next-method-call next
                :arg-info arg-info :value value)))
         (if real-mf-p
             (flet ((frob-cm-arg (arg)
@@ -158,7 +207,7 @@
                                   (let* ((fmf (fast-method-call-function emf))
                                          (fun (method-function-from-fast-method-call emf))
                                          (mf (%make-method-function fmf)))
-                                    (set-funcallable-instance-function mf fun)
+                                    (setf (%funcallable-instance-fun mf) fun)
                                     (make-instance 'standard-method
                                                    :specializers nil ; XXX
                                                    :qualifiers nil
@@ -181,11 +230,12 @@
                      ;; user-defined class don't work at all.  -- CSR,
                      ;; 2006-08-05
                      (args (cons (mapcar #'frob-cm-arg (car cm-args))
-                                 (cdr cm-args))))
+                                 (cdr cm-args)))
+                     (fun (maybe-trace-method gf method mf nil)))
                 (if (eq value default)
-                    (make-method-call :function mf :call-method-args args)
-                    (make-constant-method-call :function mf :value value
-                                               :call-method-args args))))
+                    (make-method-call :function fun :call-method-args args)
+                    (make-constant-method-call
+                     :function fun :value value :call-method-args args))))
             mf))))
 
 (defun make-effective-method-function-simple1
@@ -194,7 +244,7 @@
     (if (if (listp method)
             (eq (car method) :early-method)
             (method-p method))
-        (make-emf-from-method method cm-args gf fmf-p method-alist wrappers)
+        (make-emf-from-method gf method cm-args fmf-p method-alist wrappers)
         (if (and (consp method) (eq (car method) 'make-method))
             (make-effective-method-function gf
                                             (cadr method)
@@ -359,16 +409,16 @@
            (effective-method-lambda (expand-effective-method-function
                                      generic-function effective-method)))
       (multiple-value-bind (cfunction constants)
-          (get-fun1 effective-method-lambda
-                    (lambda (form)
-                      (memf-test-converter form generic-function
-                                           method-alist-p wrappers-p))
-                    (lambda (form)
-                      (memf-code-converter form generic-function
-                                           metatypes applyp
-                                           method-alist-p wrappers-p))
-                    (lambda (form)
-                      (memf-constant-converter form generic-function)))
+          (get-fun effective-method-lambda
+                   (lambda (form)
+                     (memf-test-converter form generic-function
+                                          method-alist-p wrappers-p))
+                   (lambda (form)
+                     (memf-code-converter form generic-function
+                                          metatypes applyp
+                                          method-alist-p wrappers-p))
+                   (lambda (form)
+                     (memf-constant-converter form generic-function)))
         (lambda (method-alist wrappers)
           (flet ((compute-constant (constant)
                    (if (consp constant)
@@ -385,7 +435,7 @@
             (let ((fun (apply cfunction
                               (mapcar #'compute-constant constants))))
               (set-fun-name fun `(combined-method ,name))
-              (make-fast-method-call :function fun
+              (make-fast-method-call :function (maybe-trace-method generic-function nil fun t)
                                      :arg-info arg-info))))))))
 
 (defmacro call-method-list (&rest calls)
