@@ -246,18 +246,6 @@ os_invalidate(os_vm_address_t addr, os_vm_size_t len)
     if (munmap(addr, len) == -1)
         perror("munmap");
 }
-
-void
-os_protect(os_vm_address_t address, os_vm_size_t length, os_vm_prot_t prot)
-{
-    if (mprotect(address, length, prot) == -1) {
-
-        perror("mprotect");
-#ifdef LISP_FEATURE_DARWIN_JIT
-        lose("%p %lu", address, length);
-#endif
-    }
-}
 
 /*
  * any OS-dependent special low-level handling for signals
@@ -276,8 +264,6 @@ memory_fault_handler(int signal, siginfo_t *siginfo, os_context_t *context)
     void *fault_addr = arch_get_bad_addr(signal, siginfo, context);
 
 #if defined(LISP_FEATURE_RESTORE_TLS_SEGMENT_REGISTER_FROM_CONTEXT)
-    FSHOW_SIGNAL((stderr, "/ TLS: restoring fs: %p in memory_fault_handler\n",
-                  *CONTEXT_ADDR_FROM_STEM(fs)));
     os_restore_tls_segment_register(context);
 #endif
 
@@ -366,20 +352,6 @@ The system may fail to start.\n",
     }
     max_allocation_size = (os_vm_size_t)((rl.rlim_cur / 2) &
       ~(32 * 1024 * 1024));
-
-#ifdef LISP_FEATURE_X86
-    {
-        size_t len;
-        int sse;
-
-        len = sizeof(sse);
-        if (sysctlbyname("machdep.sse", &sse, &len,
-                         NULL, 0) == 0 && sse != 0) {
-            /* Use the SSE detector */
-            fast_bzero_pointer = fast_bzero_detect;
-        }
-     }
-#endif /* LISP_FEATURE_X86 */
 }
 
 /* Various routines in NetBSD's C library are compatibility wrappers
@@ -447,26 +419,6 @@ static void freebsd_init()
     else
         sig_memory_fault = SIGSEGV;
 #endif
-
-    /* Quote from sbcl-devel (NIIMI Satoshi): "Some OSes, like FreeBSD
-     * 4.x with GENERIC kernel, does not enable SSE support even on
-     * SSE capable CPUs". Detect this situation and skip the
-     * fast_bzero sse/base selection logic that's normally done in
-     * x86-assem.S.
-     */
-#ifdef LISP_FEATURE_X86
-    {
-        size_t len;
-        int instruction_sse;
-
-        len = sizeof(instruction_sse);
-        if (sysctlbyname("hw.instruction_sse", &instruction_sse, &len,
-                         NULL, 0) == 0 && instruction_sse != 0) {
-            /* Use the SSE detector */
-            fast_bzero_pointer = fast_bzero_detect;
-        }
-    }
-#endif /* LISP_FEATURE_X86 */
 }
 
 #ifdef LISP_FEATURE_SB_FUTEX
@@ -502,17 +454,6 @@ futex_wake(int *lock_word, int n)
 #ifdef __DragonFly__
 static void dragonfly_init()
 {
-#ifdef LISP_FEATURE_X86
-    size_t len;
-    int instruction_sse;
-
-    len = sizeof(instruction_sse);
-    if (sysctlbyname("hw.instruction_sse", &instruction_sse, &len,
-                     NULL, 0) == 0 && instruction_sse != 0) {
-        /* Use the SSE detector */
-        fast_bzero_pointer = fast_bzero_detect;
-    }
-#endif /* LISP_FEATURE_X86 */
 }
 
 
@@ -628,9 +569,6 @@ openbsd_init()
     mib[1] = CPU_OSFXSR;
     size = sizeof (openbsd_use_fxsave);
     sysctl(mib, 2, &openbsd_use_fxsave, &size, NULL, 0);
-    if (openbsd_use_fxsave)
-        /* Use the SSE detector */
-        fast_bzero_pointer = fast_bzero_detect;
 #endif
 
     /* OpenBSD, like NetBSD, counts mmap()ed space against the
