@@ -102,9 +102,6 @@ static inline unsigned int code_n_named_calls(struct code* code) {
 #endif
 }
 
-// How many elements in 'code->constants[]' are taken by each simple-fun
-#define CODE_SLOTS_PER_SIMPLE_FUN 4
-
 // Iterate over the native pointers to each function in 'code_var'
 // offsets are stored as the number of bytes into the instructions
 // portion of the code object at which the simple-fun object resides.
@@ -170,7 +167,7 @@ base_pointer(lispobj ptr)
     return embedded_obj_p(widetag) ? fun_code_header(obj) : obj;
 }
 
-#if defined LISP_FEATURE_X86 || defined LISP_FEATURE_X86_64
+#if defined LISP_FEATURE_X86 || defined LISP_FEATURE_X86_64 || defined LISP_FEATURE_ARM64
 # define fun_self_from_baseptr(simple_fun) (lispobj)simple_fun->insts
 # define fun_self_from_taggedptr(funptr) \
     funptr - FUN_POINTER_LOWTAG + 2*N_WORD_BYTES
