@@ -52,6 +52,8 @@
 #define BIGNUM_MARK_BIT MARK_BIT
 #endif
 
+extern lispobj lisp_init_function, gc_object_watcher;
+
 static inline uword_t boxedobj_widetag_to_markbit(int widetag) {
     return (widetag == FDEFN_WIDETAG || widetag == WEAK_POINTER_WIDETAG)
          ? FDEFN_MARK_BIT : MARK_BIT;
@@ -92,7 +94,7 @@ static void* get_free_page() {
         lose("Needed more space to GC");
     page_table[free_page].type = PAGE_TYPE_UNBOXED;
     char* mem = page_address(free_page);
-    zero_dirty_pages(free_page, free_page, 0);
+    zeroize_pages_if_needed(free_page, free_page, 0);
     return mem;
 }
 
@@ -212,6 +214,7 @@ void __mark_obj(lispobj pointer)
         lispobj header = *base;
         int widetag = header_widetag(header);
         if (widetag == BIGNUM_WIDETAG) {
+            if (points_to_asm_code_p(pointer)) return;
             *base |= BIGNUM_MARK_BIT;
             return; // don't enqueue - no pointers
         } else {
@@ -438,7 +441,7 @@ void execute_full_mark_phase()
     while (where < end) {
         lispobj obj = compute_lispobj(where);
         gc_enqueue(obj);
-        where += listp(obj) ? 2 : sizetab[widetag_of(where)](where);
+        where += listp(obj) ? 2 : headerobj_size(where);
     }
 #ifdef LISP_FEATURE_METASPACE
     where = (lispobj*)METASPACE_START;
@@ -446,10 +449,13 @@ void execute_full_mark_phase()
     while (where < end) {
         lispobj obj = compute_lispobj(where);
         gc_enqueue(obj);
-        where += listp(obj) ? 2 : sizetab[widetag_of(where)](where);
+        where += listp(obj) ? 2 : headerobj_size(where);
     }
 #endif
     gc_mark_obj(lisp_package_vector);
+    gc_mark_obj(lisp_init_function);
+    gc_mark_obj(gc_object_watcher);
+    gc_mark_obj(alloc_profile_data);
     do {
         lispobj ptr = gc_dequeue();
         gc_dcheck(ptr != 0);
@@ -562,57 +568,53 @@ static void sweep_fixedobj_pages(long *zeroed)
         lispobj header = *obj;
         uword_t markbit = boxedobj_widetag_to_markbit(header_widetag(header));
         if (header & markbit) *obj = header ^ markbit;
-        obj += sizetab[widetag_of(obj)](obj);
+        obj += headerobj_size(obj);
     }
 }
 #endif
 
-static uword_t sweep(lispobj* where, lispobj* end, uword_t arg)
+/* Overwrite exactly 1 object wit non-pointer words of some sort.
+ * This eliminates tenured garbage in pseudo-static-generation,
+ * and does NOT strive to to write as few words as possible,
+ * unlike deposit_filler() which tries to be efficient */
+static void clobber_headered_object(lispobj* addr, sword_t nwords)
+{
+    page_index_t page = find_page_index(addr);
+    if (page < 0) { // code space
+        struct code* code  = (struct code*)addr;
+        if (!filler_obj_p((lispobj*)code)) {
+            code->boxed_size = 0;
+            code->header = (nwords << CODE_HEADER_SIZE_SHIFT)
+                           | CODE_HEADER_WIDETAG;
+            memset(addr+2, 0, (nwords - 2) * N_WORD_BYTES);
+        }
+    } else if (page_table[page].type == PAGE_TYPE_CODE) {
+        // Code pages don't want (0 . 0) fillers, otherwise heap checking
+        // gets an error: "object @ 0x..... is non-code on code page"
+        addr[0] = (nwords - 1) << N_WIDETAG_BITS | FILLER_WIDETAG;
+        addr[1] = 0;
+    } else {
+        memset(addr, 0, nwords * N_WORD_BYTES);
+    }
+}
+
+static uword_t sweep(lispobj* where, lispobj* end,
+                     __attribute__((unused)) uword_t arg)
 {
-    long *zeroed = (long*)arg; // one count per generation
     sword_t nwords;
 
-    // TODO: consecutive dead objects on same page should be merged.
     for ( ; where < end ; where += nwords ) {
         lispobj word = *where;
+        int livep = 0;
         if (is_header(word)) {
-            nwords = sizetab[header_widetag(word)](where);
+            nwords = headerobj_size2(where, word);
             lispobj markbit = general_widetag_to_markbit(header_widetag(word));
-            if (word & markbit)
-                *where = word ^ markbit;
-            else {
-                // Turn the object into either a (0 . 0) cons
-                // or an unboxed filler depending on size.
-                if (nwords <= 2) // could be SAP, SIMPLE-ARRAY-NIL, 1-word bignum, etc
-                    goto cons;
-                struct code* code  = (struct code*)where;
-                // Keep in sync with the definition of filler_obj_p()
-                if (!filler_obj_p((lispobj*)code)) {
-                    page_index_t page = find_page_index(where);
-                    int gen = page >= 0 ? page_table[page].gen
-                      : immobile_obj_gen_bits(where);
-                    NOTE_GARBAGE(gen, where, nwords, zeroed, {
-                        code->boxed_size = 0;
-                        code->header = (nwords << CODE_HEADER_SIZE_SHIFT)
-                                     | CODE_HEADER_WIDETAG;
-                        /* This memset() is needed for strictly boxed pages,
-                         * but probably not for other page types. No big deal.
-                         * fullcgc isn't really used much */
-                        memset(where+2, 0, (nwords - 2) * N_WORD_BYTES);
-                    })
-                }
-            }
+            if (word & markbit) livep = 1, *where = word ^ markbit;
+            if (!livep) clobber_headered_object(where, nwords);
         } else {
             nwords = 2;
-            if (!cons_markedp((lispobj)where)) {
-                if (where[0] | where[1]) {
-               cons:
-                    gc_dcheck(!immobile_space_p((lispobj)where));
-                    NOTE_GARBAGE(page_table[find_page_index(where)].gen,
-                                 where, 2, zeroed,
-                                 where[0] = where[1] = 0);
-                }
-            }
+            livep = cons_markedp((lispobj)where);
+            if (!livep) where[0] = where[1] = (uword_t)-1;
         }
     }
     return 0;
@@ -677,8 +679,8 @@ void execute_full_sweep_phase()
     if (sweeplog)
         fflush(sweeplog);
 
-    free_page = next_free_page;
-    while (free_page < page_table_pages) {
-        page_table[free_page++].type = FREE_PAGE_FLAG;
-    }
+    for (free_page = next_free_page; free_page < page_table_pages; ++free_page) {
+        set_page_need_to_zero(free_page, 1);
+        page_table[free_page].type = FREE_PAGE_FLAG;
+      }
 }
