@@ -15,23 +15,31 @@
 #ifndef _GC_PRIVATE_H_
 #define _GC_PRIVATE_H_
 
+#include "genesis/instance.h"
 #include "genesis/weak-pointer.h"
 #include "immobile-space.h"
 #include "code.h"
 
 #ifdef LISP_FEATURE_GENCGC
-#include "gencgc-alloc-region.h"
-static inline void *
-gc_general_alloc(sword_t nbytes, int page_type_flag)
+void *collector_alloc_fallback(struct alloc_region*,sword_t,int);
+static inline void* __attribute__((unused))
+gc_general_alloc(struct alloc_region* region, sword_t nbytes, int page_type)
 {
-    void *gc_alloc_with_region(struct alloc_region*,sword_t,int);
-    if (1 <= page_type_flag && page_type_flag <= 3)
-        return gc_alloc_with_region(&gc_alloc_region[page_type_flag-1],
-                                    nbytes, page_type_flag);
-    lose("bad page type flag: %d", page_type_flag);
+    void *new_obj = region->free_pointer;
+    void *new_free_pointer = (char*)new_obj + nbytes;
+    // Large objects will never fit in a region, so we automatically dtrt
+    if (new_free_pointer <= region->end_addr) {
+        region->free_pointer = new_free_pointer;
+        return new_obj;
+    }
+    return collector_alloc_fallback(region, nbytes, page_type);
 }
+lispobj copy_potential_large_object(lispobj object, sword_t nwords,
+                                   struct alloc_region*, int page_type);
 #else
-extern void *gc_general_alloc(sword_t nbytes,int page_type_flag);
+void *gc_general_alloc(void*,sword_t,int);
+lispobj copy_potential_large_object(lispobj object, sword_t nwords,
+                                   void*, int page_type);
 #endif
 
 #define CHECK_COPY_PRECONDITIONS(object, nwords) \
@@ -43,34 +51,45 @@ extern void *gc_general_alloc(sword_t nbytes,int page_type_flag);
     gc_dcheck(lowtag_of(copy) == lowtag); \
     gc_dcheck(!from_space_p(copy));
 
-#define note_transported_object(old, new) /* do nothing */
+#define GC_LOGGING 0
 
+/* For debugging purposes, you can make this macro as complicated as you like,
+ * such as checking various other aspects of the object in 'old' */
+#if GC_LOGGING
+#define NOTE_TRANSPORTING(old, new, nwords) really_note_transporting(old,new,nwords)
+void really_note_transporting(lispobj old,void*new,sword_t nwords);
+#elif defined COLLECT_GC_STATS && COLLECT_GC_STATS
+#define NOTE_TRANSPORTING(old, new, nwords) gc_copied_nwords += nwords
+#else
+#define NOTE_TRANSPORTING(old, new, nwords) /* do nothing */
+#endif
+
+extern uword_t gc_copied_nwords;
 static inline lispobj
-gc_general_copy_object(lispobj object, size_t nwords, int page_type_flag)
+gc_copy_object(lispobj object, size_t nwords, void* region, int page_type)
 {
     CHECK_COPY_PRECONDITIONS(object, nwords);
 
     /* Allocate space. */
-    lispobj *new = gc_general_alloc(nwords*N_WORD_BYTES, page_type_flag);
+    lispobj *new = gc_general_alloc(region, nwords*N_WORD_BYTES, page_type);
+    NOTE_TRANSPORTING(object, new,  nwords);
 
     /* Copy the object. */
     memcpy(new,native_pointer(object),nwords*N_WORD_BYTES);
 
-    note_transported_object(object, new);
-
     return make_lispobj(new, lowtag_of(object));
 }
 
 // Like above but copy potentially fewer words than are allocated.
 // ('old_nwords' can be, but does not have to be, smaller than 'nwords')
 static inline lispobj
-gc_copy_object_resizing(lispobj object, long nwords, int page_type_flag,
+gc_copy_object_resizing(lispobj object, long nwords, void* region, int page_type,
                         int old_nwords)
 {
     CHECK_COPY_PRECONDITIONS(object, nwords);
-    lispobj *new = gc_general_alloc(nwords*N_WORD_BYTES, page_type_flag);
+    lispobj *new = gc_general_alloc(region, nwords*N_WORD_BYTES, page_type);
+    NOTE_TRANSPORTING(object, new, old_nwords);
     memcpy(new, native_pointer(object), old_nwords*N_WORD_BYTES);
-    note_transported_object(object, new);
     return make_lispobj(new, lowtag_of(object));
 }
 
@@ -101,8 +120,6 @@ extern void gc_common_init();
 extern boolean test_weak_triggers(int (*)(lispobj), void (*)(lispobj));
 
 lispobj  copy_unboxed_object(lispobj object, sword_t nwords);
-lispobj  copy_object(lispobj object, sword_t nwords);
-lispobj  copy_large_object(lispobj object, sword_t nwords, int page_type_flag);
 
 lispobj *search_read_only_space(void *pointer);
 lispobj *search_static_space(void *pointer);
@@ -133,11 +150,11 @@ static inline int header_rememberedp(lispobj header) {
   return (header & (OBJ_WRITTEN_FLAG << 24)) != 0;
 }
 
-#ifndef LISP_FEATURE_IMMOBILE_SPACE
+static inline boolean filler_obj_p(lispobj* obj) {
+    return widetag_of(obj) == CODE_HEADER_WIDETAG && obj[1] == 0;
+}
 
-static inline boolean filler_obj_p(lispobj __attribute__((unused)) *obj) { return 0; }
-
-#else
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
 
 extern void enliven_immobile_obj(lispobj*,int);
 
@@ -202,39 +219,47 @@ static inline void assign_generation(lispobj* obj, generation_index_t gen)
 #error "Need to define immobile_obj_gen_bits() for big-endian"
 #endif /* little-endian */
 
-static inline boolean filler_obj_p(lispobj* obj) {
-    return widetag_of(obj) == CODE_HEADER_WIDETAG && obj[1] == 0;
-}
-
 #endif /* immobile space */
 
-#define WEAK_POINTER_CHAIN_END (void*)(intptr_t)-1
-#define WEAK_POINTER_NWORDS ALIGN_UP(WEAK_POINTER_SIZE,2)
-
-static inline boolean weak_pointer_breakable_p(struct weak_pointer *wp)
-{
-    lispobj pointee = wp->value;
-    // A broken weak-pointer's value slot has unbound-marker
-    // which does not satisfy is_lisp_pointer().
-    return is_lisp_pointer(pointee) && (from_space_p(pointee)
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
-         || (immobile_space_p(pointee) &&
-             immobile_obj_gen_bits(base_pointer(pointee)) == from_space)
-#endif
-            );
+#ifdef LISP_FEATURE_64_BIT
+#define WEAK_POINTER_CHAIN_END (void*)(intptr_t)1
+#define in_weak_pointer_list(wp) ((wp->header>>16)!=0)
+static inline struct weak_pointer *get_weak_pointer_next(struct weak_pointer *wp) {
+    // 6 bytes to encode 'next' is way more than enough
+    uword_t offset = wp->header >> 16;
+    if (offset <= 1) return (void*)offset;
+    return (void*)((lispobj*)DYNAMIC_SPACE_START + offset);
 }
+static inline void reset_weak_pointer_next(struct weak_pointer *wp) {
+    wp->header = wp->header & 0xffff;
+}
+static inline lispobj encode_weakptr_next(void* x) {
+    if ((uword_t)x <= 1) return (uword_t)x;
+    sword_t wordindex = (lispobj*)x - (lispobj*)DYNAMIC_SPACE_START;
+    gc_assert(wordindex != 0); // wp can't be the first object in dynamic space
+    return wordindex;
+}
+#else
+#define WEAK_POINTER_CHAIN_END (void*)(intptr_t)-1
+#define in_weak_pointer_list(wp) (wp->next)
+#define get_weak_pointer_next(wp) wp->next
+#define reset_weak_pointer_next(wp) wp->next = 0
+#endif
 
 static inline void add_to_weak_pointer_chain(struct weak_pointer *wp) {
+    // Better already be fixed in position or we're in trouble
+    gc_assert(!from_space_p(make_lispobj(wp,OTHER_POINTER_LOWTAG)));
     /* Link 'wp' into weak_pointer_chain using its 'next' field.
      * We ensure that 'next' is always NULL when the weak pointer isn't
      * in the chain, and not NULL otherwise. The end of the chain
      * is denoted by WEAK_POINTER_CHAIN_END which is distinct from NULL.
      * The test of whether the weak pointer has been placed in the chain
-     * is performed in 'scav_weak_pointer' for gencgc.
-     * In cheneygc, chaining is performed in 'trans_weak_pointer'
-     * which works just as well, since an object is transported
-     * at most once per GC cycle */
+     * is performed in 'scav_weak_pointer' */
+#ifdef LISP_FEATURE_64_BIT
+    wp->header = (encode_weakptr_next(weak_pointer_chain)<<16) | (wp->header & 0xffff);
+#else
     wp->next = weak_pointer_chain;
+#endif
     weak_pointer_chain = wp;
 }
 
@@ -245,15 +270,14 @@ static inline struct bitmap get_layout_bitmap(struct layout* layout)
     struct bitmap bitmap;
     const int layout_id_vector_fixed_capacity = 7;
 #ifdef LISP_FEATURE_64_BIT
-    sword_t depthoid = layout->flags;
-    // Depthoid is stored in the upper 4 bytes of the header, as a fixnum.
+    sword_t depthoid = layout->sw_flags;
+    // Depthoid is stored in the upper 4 bytes of 'flags', as a fixnum.
     depthoid >>= (32 + N_FIXNUM_TAG_BITS);
     int extra_id_words =
       (depthoid > layout_id_vector_fixed_capacity) ?
       ALIGN_UP(depthoid - layout_id_vector_fixed_capacity, 2) / 2 : 0;
 #else
-    sword_t depthoid = layout->depthoid;
-    depthoid >>= N_FIXNUM_TAG_BITS;
+    sword_t depthoid = fixnum_value(layout->depthoid);
     int extra_id_words = (depthoid > layout_id_vector_fixed_capacity) ?
       depthoid - layout_id_vector_fixed_capacity : 0;
 #endif
@@ -285,80 +309,116 @@ static inline boolean bitmap_logbitp(unsigned int index, struct bitmap bitmap)
 
 #if defined(LISP_FEATURE_GENCGC)
 
-/* Define a macro to avoid a detour through the write fault handler.
- *
- * It's usually more efficient to do these extra tests than to receive
- * a signal. And it leaves the page protected, which is a bonus.
- * The downside is that multiple operations on the same page ought to
- * be batched, so that there is at most one unprotect/reprotect per page
- * rather than per write operation per page.
- *
- * This also should fix -fsanitize=thread which makes handling of SIGSEGV
- * during GC difficult. Not impossible, but definitely broken.
- * It has to do with the way the sanitizer intercepts calls
- * to sigaction() - it mucks with your sa_mask :-(.
- *
- * This macro take an aribtrary expression as the 'operation' rather than
- * an address and value to assign, for two reasons:
- * 1. there may be more than one store operation that has to be
- *    within the scope of the lifted write barrier,
- *    so a single lvalue and rvalue is maybe inadequate.
- * 2. it might need to use a sync_fetch_and_<frob>() gcc intrinsic,
- *    so it's not necessarily just going to be an '=' operator
- *
- * KLUDGE: assume that faults do not occur in immobile space.
- * for the most part. (This is pretty obviously not true,
- * but seems only to be a problem in fullcgc)
- */
-
-#define NON_FAULTING_STORE(operation, addr) { \
-  page_index_t page_index = find_page_index(addr); \
-  if (page_index < 0 || !page_table[page_index].write_protected) { operation; } \
-  else { unprotect_page_index(page_index); \
-         operation; \
-         protect_page(page_address(page_index), page_index); }}
-
-/* This is used bu the fault handler, and potentially during GC */
-static inline void unprotect_page_index(page_index_t page_index)
-{
-    os_protect(page_address(page_index), GENCGC_CARD_BYTES, OS_VM_PROT_ALL);
-    unsigned char *pflagbits = (unsigned char*)&page_table[page_index].gen - 1;
-    __sync_fetch_and_or(pflagbits, WP_CLEARED_FLAG);
-    __sync_fetch_and_and(pflagbits, ~WRITE_PROTECTED_FLAG);
-}
-
-static inline void protect_page(void* page_addr, page_index_t page_index)
-{
-    os_protect((void *)page_addr,
-               GENCGC_CARD_BYTES,
-               OS_VM_PROT_READ|OS_VM_PROT_EXECUTE);
-
-    /* Note: we never touch the write_protected_cleared bit when protecting
-     * a page. Consider two random threads that reach their SIGSEGV handlers
-     * concurrently, each checking why it got a write fault. One thread wins
-     * the race to remove the memory protection, and marks our shadow bit.
-     * wp_cleared is set so that the other thread can conclude that the fault
-     * was reasonable.
-     * If GC unprotects and reprotects a page, it's probably OK to reset the
-     * cleared bit 0 if it was 0 before. (Because the fault handler blocks
-     * SIG_STOP_FOR_GC which is usually SIGUSR2, handling the wp fault is
-     * atomic with respect to invocation of GC)
-     * But nothing is really gained by resetting the cleared flag.
-     * It is explicitly zeroed on pages marked as free though.
-     */
-    page_table[page_index].write_protected = 1;
-}
+extern unsigned char* gc_card_mark;
 
+#ifdef LISP_FEATURE_DARWIN_JIT
+#define OS_VM_PROT_JIT_READ OS_VM_PROT_READ
+#define OS_VM_PROT_JIT_ALL OS_VM_PROT_READ | OS_VM_PROT_WRITE
 #else
-
-#define NON_FAULTING_STORE(operation, addr) operation
-
+#define OS_VM_PROT_JIT_READ OS_VM_PROT_READ | OS_VM_PROT_EXECUTE
+#define OS_VM_PROT_JIT_ALL OS_VM_PROT_ALL
 #endif
 
-#if defined(LISP_FEATURE_X86_64) || defined(LISP_FEATURE_X86)
-# define CODE_PAGES_USE_SOFT_PROTECTION 1
+// "assign" as the operation name is a little clearer than "set"
+// which tends to be synonymous with setting a bit to 1.
+#define assign_page_card_marks(page, val) \
+  memset(gc_card_mark+page_to_card_index(page), val, CARDS_PER_PAGE)
+
+#ifdef LISP_FEATURE_SOFT_CARD_MARKS
+
+#define NON_FAULTING_STORE(operation, addr) { operation; }
+// The low bit of 0 implies "marked". So CARD_MARKED and STICKY_MARK
+// are both considered marked. All bits of UNMARKED are 1s, so that
+// one word full of mark bytes reads as -1. See the autogenerated
+// functions in "genesis/cardmarks.h" for the use-case.
+#define CARD_MARKED 0
+#define STICKY_MARK 2
+#define CARD_UNMARKED 0xff
+#define MARK_BYTE_MASK 0xff
+
 #else
-# define CODE_PAGES_USE_SOFT_PROTECTION 0
+
+// With physical page protection, bit index 0 is the MARKED bit (inverted WP bit)
+// and bit index 1 is the WP_CLEARED (write_protect_cleared) bit.
+// The fault handler always sets both bits.
+// The only other bit pair value would be illegal.
+#define CARD_UNMARKED         0 /* write-protected = 1 */
+#define CARD_MARKED           1 /* write-protected = 0 */
+#define WP_CLEARED_AND_MARKED 3 /* write-protected = 0, wp-cleared = 1 */
+// CODE-HEADER-SET can store the low byte from NULL-TN into the mark array.
+// This sets the two low bits on, but also spuriously sets other bits,
+// which we can ignore when reading the byte.
+#define MARK_BYTE_MASK 3
+
+#define PAGE_WRITEPROTECTED_P(n) (~gc_card_mark[page_to_card_index(n)] & CARD_MARKED)
+#define SET_PAGE_PROTECTED(n,val) gc_card_mark[page_to_card_index(n)] = \
+      (val ? CARD_UNMARKED : CARD_MARKED)
+
+#define cardseq_any_marked(card_index) (gc_card_mark[card_index] & CARD_MARKED)
+#define cardseq_all_marked_nonsticky(card_index) cardseq_any_marked(card_index)
+#define page_cards_all_marked_nonsticky(page_index) \
+  cardseq_all_marked_nonsticky(page_to_card_index(page_index))
+
+/* NON_FAULTING_STORE is only for fixnums and GC metadata where we need
+ * the ability to write-through the store barrier.
+ * The uses are limited to updating the weak vector chain, weak hash-table
+ * chain, and vector rehash flags. Those don't affect a page's marked state */
+#define PAGE_BASE(addr) ((char*)ALIGN_DOWN((uword_t)(addr),GENCGC_PAGE_BYTES))
+#define NON_FAULTING_STORE(operation, addr) { \
+  page_index_t page_index = find_page_index(addr); \
+  if (page_index < 0 || !PAGE_WRITEPROTECTED_P(page_index)) { operation; } \
+  else { os_protect(PAGE_BASE(addr), GENCGC_PAGE_BYTES, OS_VM_PROT_JIT_ALL); \
+         operation; \
+         os_protect(PAGE_BASE(addr), GENCGC_PAGE_BYTES, OS_VM_PROT_JIT_READ); } }
+
+/* This is used by the fault handler, and potentially during GC
+ * if we need to remember that a pointer store occurred.
+ * The fault handler should supply WP_CLEARED_AND_MARKED as the mark,
+ * but the collector should use CARD_MARKED */
+static inline void unprotect_page(void* addr, unsigned char mark)
+{
+    // No atomic op needed for a 1-byte store.
+    gc_card_mark[addr_to_card_index(addr)] = mark;
+    os_protect(PAGE_BASE(addr), GENCGC_PAGE_BYTES, OS_VM_PROT_JIT_ALL);
+}
+#endif
+
+// Two helpers to avoid invoking the memory fault signal handler.
+// For clarity, distinguish between words which *actually* need to frob
+// physical (MMU-based) protection versus those which don't,
+// but are forced to call mprotect() because it's the only choice.
+// Unlike with NON_FAULTING_STORE, in this case we actually do want to record that
+// the ensuing store toggles the WP bit without invoking the fault handler.
+static inline void notice_pointer_store(void* addr) {
+#ifdef LISP_FEATURE_SOFT_CARD_MARKS
+    int card = addr_to_card_index(addr);
+    // STICKY is stronger than MARKED. Only change if UNMARKED.
+    if (gc_card_mark[card] == CARD_UNMARKED) gc_card_mark[card] = CARD_MARKED;
+#else
+    page_index_t index = find_page_index(addr);
+    gc_assert(index >= 0);
+    if (PAGE_WRITEPROTECTED_P(index)) unprotect_page(addr, CARD_MARKED);
+#endif
+}
+static inline void ensure_non_ptr_word_writable(__attribute__((unused)) void* addr)
+{
+  // there's nothing to "ensure" if using software card marks
+#ifndef LISP_FEATURE_SOFT_CARD_MARKS
+    notice_pointer_store(addr);
+#endif
+}
+
+// #+soft-card-mark: this expresion is true of both CARD_MARKED and STICKY_MARK
+// #-soft-card-mark: this expresion is true of both CARD_MARKED and WP_CLEARED_AND_MARKED
+#define card_dirtyp(index) (gc_card_mark[index] & MARK_BYTE_MASK) != CARD_UNMARKED
+
+#else
+
+/* cheneygc */
+#define notice_pointer_store(dummy)
+#define ensure_non_ptr_word_writable(dummy)
+#define NON_FAULTING_STORE(operation, addr) operation
+
 #endif
 
 #define KV_PAIRS_HIGH_WATER_MARK(kvv) fixnum_value(kvv[0])
@@ -384,6 +444,21 @@ static inline int instance_length(lispobj header)
     return (((unsigned int)header >> INSTANCE_LENGTH_SHIFT) & 0x3FFF) + extra;
 }
 
+// One bit differentiates FUNCALLABLE_INSTANCE_WIDETAG from INSTANCE_WIDETAG.
+// This is index of that bit.
+#ifdef LISP_FEATURE_64_BIT
+#  define FUNINSTANCE_SELECTOR_BIT_NUMBER 3
+#else
+#  define FUNINSTANCE_SELECTOR_BIT_NUMBER 2
+#endif
+static inline boolean instanceoid_widetag_p(unsigned char widetag) {
+    return (widetag | (1<<FUNINSTANCE_SELECTOR_BIT_NUMBER)) == FUNCALLABLE_INSTANCE_WIDETAG;
+}
+static inline int instanceoid_length(lispobj header) {
+    return (header & (1<<FUNINSTANCE_SELECTOR_BIT_NUMBER))
+        ? (int)(HeaderValue(header) & SHORT_HEADER_MAX_WORDS) : instance_length(header);
+}
+
 /// instance_layout() and layout_of() macros takes a lispobj* and are lvalues
 #ifdef LISP_FEATURE_COMPACT_INSTANCE_HEADER
 
@@ -403,31 +478,95 @@ static inline int instance_length(lispobj header)
 // first 4 words of funcallable instance are: header, trampoline, layout, fin-fun
 # define funinstance_layout(native_ptr) ((lispobj*)native_ptr)[2]
 # define layout_of(native_ptr) \
-  ((lispobj*)native_ptr)[1+((widetag_of(native_ptr)>>LAYOUT_SELECTOR_BIT)&1)]
+  ((lispobj*)native_ptr)[1+((widetag_of(native_ptr)>>FUNINSTANCE_SELECTOR_BIT_NUMBER)&1)]
 
 #endif
 
 static inline int layout_depth2_id(struct layout* layout) {
-    int32_t* vector = (int32_t*)&layout->id_word0;
+    int32_t* vector = (int32_t*)&layout->uw_id_word0;
     return vector[0];
 }
-// Keep in sync with hardwired IDs in src/compiler/generic/genesis.lisp
+// Keep in sync with hardwired IDs in CHOOSE-LAYOUT-ID
+// in src/compiler/generic/layout-ids.lisp
+#define WRAPPER_LAYOUT_ID 2
 #define LAYOUT_LAYOUT_ID 3
 #define LFLIST_NODE_LAYOUT_ID 4
+#define BROTHERTREE_UNARY_NODE_LAYOUT_ID 5
 
 /// Return true if 'thing' is a layout.
 /// This predicate is careful, as is it used to verify heap invariants.
 static inline boolean layoutp(lispobj thing)
 {
-    lispobj base_ptr = thing - INSTANCE_POINTER_LOWTAG;
     lispobj layout;
-    if ((base_ptr & LOWTAG_MASK) || !(layout = layout_of((lispobj*)base_ptr)))
-        return 0;
+    if (lowtag_of(thing) != INSTANCE_POINTER_LOWTAG) return 0;
+    if ((layout = instance_layout(INSTANCE(thing))) == 0) return 0;
     return layout_depth2_id(LAYOUT(layout)) == LAYOUT_LAYOUT_ID;
 }
+#ifdef LISP_FEATURE_METASPACE
+static inline boolean wrapperp(lispobj thing)
+{
+    lispobj layout;
+    if (lowtag_of(thing) != INSTANCE_POINTER_LOWTAG) return 0;
+    if ((layout = instance_layout(INSTANCE(thing))) == 0) return 0;
+    return layout_depth2_id(LAYOUT(layout)) == WRAPPER_LAYOUT_ID;
+}
+static inline int wrapper_id(lispobj wrapper)
+{
+    struct layout* layout = LAYOUT(WRAPPER(wrapper)->friend);
+    return layout_depth2_id(layout);
+}
+#endif
 /// Return true if 'thing' is the layout of any subtype of sb-lockless::list-node.
 static inline boolean lockfree_list_node_layout_p(struct layout* layout) {
     return layout_depth2_id(layout) == LFLIST_NODE_LAYOUT_ID;
 }
 
+#ifdef LISP_FEATURE_METASPACE
+#define METASPACE_START (READ_ONLY_SPACE_START+32768) /* KLUDGE */
+// Keep in sync with the macro definitions in src/compiler/generic/early-vm.lisp
+struct slab_header {
+    short sizeclass;
+    short capacity;
+    short chunksize;
+    short count;
+    void* freelist;
+    struct slab_header *next;
+    struct slab_header *prev;
+};
+#endif
+
+/* Check whether 'pointee' was forwarded. If it has been, update the contents
+ * of 'cell' to point to it. Otherwise, set 'cell' to 'broken'.
+ * Note that this macro has no braces around the body because one of the uses
+ * of it needs to stick on another 'else' or two */
+#define TEST_WEAK_CELL(cell, pointee, broken) \
+    lispobj *native = native_pointer(pointee); \
+    if (from_space_p(pointee)) \
+        cell = forwarding_pointer_p(native) ? forwarding_pointer_value(native) : broken; \
+    else if (immobile_space_p(pointee)) { \
+        if (immobile_obj_gen_bits(base_pointer(pointee)) == from_space) cell = broken; \
+    }
+
+#ifdef MAX_CONSES_PER_PAGE
+static const int CONS_PAGE_USABLE_BYTES = MAX_CONSES_PER_PAGE*CONS_SIZE*N_WORD_BYTES;
+#endif
+
+#include "genesis/cons.h"
+/* Return true if 'addr' has a lowtag and widetag that correspond,
+ * given that the words at 'addr' are within range for an allocated page.
+ * 'addr' could be a pointer to random data, and this check is merely
+ * a heuristic. False positives are possible. */
+static inline boolean plausible_tag_p(lispobj addr)
+{
+    if (listp(addr))
+        return is_cons_half(CONS(addr)->car)
+            && is_cons_half(CONS(addr)->cdr)
+            // -1 can be left by the */signed=>integer vop
+            // and is also useful as filler on cons pages.
+            && CONS(addr)->car != (uword_t)-1;
+    unsigned char widetag = widetag_of(native_pointer(addr));
+    return other_immediate_lowtag_p(widetag)
+        && lowtag_of(addr) == LOWTAG_FOR_WIDETAG(widetag);
+}
+
 #endif /* _GC_PRIVATE_H_ */
