@@ -10,20 +10,14 @@
 /* Abstract out the data for an allocation region allowing a single
  * routine to be used for allocation and closing. */
 /* Caution: if you change this, you may have to change compiler/generic/objdef
- * (for the THREAD object), all the backends' allocators, and room.lisp */
+ * (for the THREAD object), all the backends' allocators, and room.lisp.
+ * But as long as the first two words are left alone,
+ * it's generally OK to add or remove other words.
+ */
 struct alloc_region {
-
     /* These two are needed for quick allocation. */
     void  *free_pointer;
     void  *end_addr; /* pointer to the byte after the last usable byte */
-
-    /* These are needed when closing the region. */
-    /* 'last_page' is identical to 'find_page_index((char*)end_addr - 1)'
-     * whenever the region is in an open state. The value is preserved on
-     * closing so that allocation can potentially resume where it left off,
-     * though that's not quite how things are implemented at present.
-     */
-    page_index_t  last_page;
     void  *start_addr;
 };
 
