@@ -26,11 +26,13 @@
 #include "code.h"
 #include "gc-internal.h"
 #include "gc-private.h"
+#include "genesis/gc-tables.h"
 #include <stdarg.h>
 #include "thread.h"              /* genesis/primitive-objects.h needs this */
 #include <errno.h>
 #include <stdlib.h>
 #include <inttypes.h>
+#include <setjmp.h>
 
 /* FSHOW and odxprint provide debugging output for low-level information
  * (signal handling, exceptions, safepoints) which is hard to debug by
@@ -38,14 +40,14 @@
  *
  * If enabled at all, environment variables control whether calls of the
  * form odxprint(name, ...) are enabled at run-time, e.g. using
- * SBCL_DYNDEBUG="fshow fshow_signal safepoints".
+ * SBCL_DYNDEBUG="fshow safepoints".
  *
- * In the case of FSHOW and FSHOW_SIGNAL, old-style code from runtime.h
+ * In the case of FSHOW, old-style code from runtime.h
  * can also be used to enable or disable these more aggressively.
  */
 
 struct dyndebug_config dyndebug_config = {
-    QSHOW == 2, QSHOW_SIGNALS == 2
+    QSHOW == 2,
 };
 
 void
@@ -66,7 +68,6 @@ dyndebug_init()
     int *ptrs[DYNDEBUG_NFLAGS];
 
     dyndebug_init1(fshow,          "FSHOW");
-    dyndebug_init1(fshow_signal,   "FSHOW_SIGNAL");
     dyndebug_init1(gencgc_verbose, "GENCGC_VERBOSE");
     dyndebug_init1(safepoints,     "SAFEPOINTS");
     dyndebug_init1(seh,            "SEH");
@@ -237,6 +238,7 @@ static void indent(int in)
         fputs(spaces + 64 - in, stdout);
 }
 
+static jmp_buf ldb_print_nlx;
 static boolean continue_p(boolean newline)
 {
     char buffer[256];
@@ -256,7 +258,7 @@ static boolean continue_p(boolean newline)
 
             if (fgets(buffer, sizeof(buffer), stdin)) {
                 if (buffer[0] == 'n' || buffer[0] == 'N')
-                    throw_to_monitor();
+                    longjmp(ldb_print_nlx, 1);
                 else
                     cur_lines = 0;
             } else {
@@ -392,6 +394,22 @@ static void brief_list(lispobj obj)
     }
 }
 
+void print_list_car_ptrs(lispobj obj, FILE* f)
+{
+    char sep = '(';
+    int len = 0;
+    if (obj == NIL) { fprintf(f, "NIL"); return; }
+    do {
+        if (++len > 20) { fprintf(f, "...)"); return; }
+        fprintf(f, "%c%p", sep, (void*)CONS(obj)->car);
+        obj = CONS(obj)->cdr;
+        sep = ' ';
+    } while (listp(obj) && obj != NIL);
+    if (obj != NIL) fprintf(f, " . %p", (void*)obj);
+    putc(')', f);
+}
+
+
 static void print_list(lispobj obj)
 {
     if (obj == NIL) {
@@ -407,7 +425,7 @@ char * simple_base_stringize(struct vector * string)
 {
   if (widetag_of(&string->header) == SIMPLE_BASE_STRING_WIDETAG)
       return (char*)string->data;
-  int length = string->length;
+  int length = vector_len(string);
   char * newstring = malloc(length+1);
   uint32_t * data = (uint32_t*)string->data;
   int i;
@@ -419,7 +437,7 @@ char * simple_base_stringize(struct vector * string)
 
 static void brief_struct(lispobj obj)
 {
-    struct instance *instance = (struct instance *)native_pointer(obj);
+    struct instance *instance = INSTANCE(obj);
     extern struct vector * instance_classoid_name(lispobj*);
     struct vector * classoid_name;
     classoid_name = instance_classoid_name((lispobj*)instance);
@@ -441,8 +459,14 @@ static boolean tagged_slot_p(struct layout *layout, int slot_index)
 {
     // Since we're doing this scan, we could return the name
     // and exact raw type.
+#ifdef LISP_FEATURE_METASPACE
+    struct wrapper *wrapper = (void*)(layout->friend-INSTANCE_POINTER_LOWTAG);
+    if (instancep(wrapper->_info)) {
+        struct defstruct_description* dd = (void*)(wrapper->_info-INSTANCE_POINTER_LOWTAG);
+#else
     if (instancep(layout->_info)) {
         struct defstruct_description* dd = (void*)(layout->_info-INSTANCE_POINTER_LOWTAG);
+#endif
         lispobj slots = dd->slots;
         for ( ; slots != NIL ; slots = CONS(slots)->cdr ) {
             struct defstruct_slot_description* dsd =
@@ -456,7 +480,7 @@ static boolean tagged_slot_p(struct layout *layout, int slot_index)
 
 static void print_struct(lispobj obj)
 {
-    struct instance *instance = (struct instance *)native_pointer(obj);
+    struct instance *instance = INSTANCE(obj);
     short int i;
     char buffer[16];
     lispobj layout = instance_layout(native_pointer(obj));
@@ -475,7 +499,7 @@ static void print_struct(lispobj obj)
 void show_lstring(struct vector * string, int quotes, FILE *s)
 {
   int ucs4_p = 0;
-  int i, len = fixnum_value(string->length);
+  int i, len = vector_len(string);
 
 #ifdef SIMPLE_CHARACTER_STRING_WIDETAG
   if (widetag_of(&string->header) == SIMPLE_CHARACTER_STRING_WIDETAG) {
@@ -507,7 +531,6 @@ void show_lstring(struct vector * string, int quotes, FILE *s)
 
 static void brief_fun_or_otherptr(lispobj obj)
 {
-    extern void safely_show_lstring(struct vector*, int, FILE*);
     lispobj *ptr, header;
     int type;
     struct symbol *symbol;
@@ -518,9 +541,10 @@ static void brief_fun_or_otherptr(lispobj obj)
     switch (type) {
         case SYMBOL_WIDETAG:
             symbol = (struct symbol *)ptr;
-            if (symbol->package == NIL)
+            lispobj package = symbol_package(symbol);
+            if (package == NIL)
                 printf("#:");
-            show_lstring(VECTOR(symbol->name), 0, stdout);
+            show_lstring(symbol_name(symbol), 0, stdout);
             break;
 
         case SIMPLE_BASE_STRING_WIDETAG:
@@ -539,7 +563,7 @@ static void brief_fun_or_otherptr(lispobj obj)
                 if (lowtag_of(name) == OTHER_POINTER_LOWTAG
                     && widetag_of(native_pointer(name)) == SYMBOL_WIDETAG) {
                   printf(" for ");
-                  struct vector* str = symbol_name(native_pointer(name));
+                  struct vector* str = symbol_name(SYMBOL(name));
                   safely_show_lstring(str, 0, stdout);
                 }
             }
@@ -551,10 +575,14 @@ static void print_slots(char **slots, int count, lispobj *ptr)
 {
     while (count-- > 0) {
         if (*slots) {
-            // kludge for half-lispword sized slot
-            print_obj(*slots,
-                      (N_WORD_BYTES == 8 && !strcmp(*slots, "boxed_size: "))
-                      ? *ptr & 0xFFFFFFFF : *ptr);
+            // kludge for encoded slots
+            lispobj word = *ptr;
+            char* slot_name = *slots;
+            if (N_WORD_BYTES == 8 && !strcmp(slot_name, "boxed_size: ")) word = word & 0xFFFFFFFF;
+#ifdef LISP_FEATURE_COMPACT_SYMBOL
+            else if (!strcmp(slot_name, "name: ")) word = decode_symbol_name(word);
+#endif
+            print_obj(slot_name, word);
             slots++;
         } else {
             print_obj("???: ", *ptr);
@@ -563,23 +591,9 @@ static void print_slots(char **slots, int count, lispobj *ptr)
     }
 }
 
-lispobj symbol_function(lispobj* symbol)
+lispobj symbol_function(struct symbol* symbol)
 {
-    lispobj info = ((struct symbol*)symbol)->info;
-    if (listp(info))
-        info = CONS(info)->cdr;
-    if (lowtag_of(info) == OTHER_POINTER_LOWTAG) {
-        struct vector* v = VECTOR(info);
-        int len = fixnum_value(v->length);
-        if (len != 0) {
-            lispobj elt = v->data[0];  // Just like INFO-VECTOR-FDEFN
-            if (fixnump(elt) && (fixnum_value(elt) & 07777) >= 07701) {
-                lispobj fdefn = v->data[len-1];
-                if (lowtag_of(fdefn) == OTHER_POINTER_LOWTAG)
-                    return FDEFN(fdefn)->fun;
-            }
-        }
-    }
+    if (symbol->fdefn) return FDEFN(symbol->fdefn)->fun;
     return NIL;
 }
 
@@ -609,6 +623,7 @@ static void print_fun_or_otherptr(lispobj obj)
 
     switch (type) {
     case BIGNUM_WIDETAG:
+        count &= 0x7fffff;
         ptr += count;
         NEWLINE_OR_RETURN;
         printf("0x");
@@ -634,15 +649,20 @@ static void print_fun_or_otherptr(lispobj obj)
         // Only 1 byte of a symbol header conveys its size.
         // The other bytes may be freely used by the backend.
         print_slots(symbol_slots, count & 0xFF, ptr);
-        if (symbol_function(ptr-1) != NIL)
-            print_obj("fun: ", symbol_function(ptr-1));
+        struct symbol* sym = (void*)(ptr - 1);
+        if (symbol_function(sym) != NIL) print_obj("fun: ", symbol_function(sym));
 #ifdef LISP_FEATURE_SB_THREAD
-        int tlsindex = tls_index_of((struct symbol*)(ptr-1));
-        struct thread*th = arch_os_get_current_thread();
+        int tlsindex = tls_index_of(sym);
+        struct thread*th = get_sb_vm_thread();
         if (th != 0 && tlsindex != 0) {
             lispobj v = *(lispobj*)(tlsindex + (char*)th);
             print_obj("tlsval: ", v);
         }
+#endif
+#ifdef LISP_FEATURE_COMPACT_SYMBOL
+        // print_obj doesn't understand raw words, so make it a fixnum
+        int pkgid = symbol_package_id(sym) << N_FIXNUM_TAG_BITS;
+        print_obj("package_id: ", pkgid);
 #endif
         break;
 
@@ -710,7 +730,7 @@ static void print_fun_or_otherptr(lispobj obj)
     case SIMPLE_VECTOR_WIDETAG:
         NEWLINE_OR_RETURN;
         {
-        long length = fixnum_value(*ptr);
+        long length = vector_len(VECTOR(obj));
         printf("length = %ld", length);
         ptr++;
         index = 0;
@@ -721,15 +741,21 @@ static void print_fun_or_otherptr(lispobj obj)
         }
         break;
 
-    // FIXME: This case looks unreachable. print_struct() does it
-    case INSTANCE_WIDETAG:
+    case SIMPLE_BIT_VECTOR_WIDETAG:
         NEWLINE_OR_RETURN;
-        count = instance_length(header);
-        printf("length = %ld", (long) count);
-        index = 0;
-        while (count-- > 0) {
-            sprintf(buffer, "%d: ", index++);
-            print_obj(buffer, *ptr++);
+        {
+        long length = vector_len(VECTOR(obj));
+        printf("length = %ld : ", length);
+        int bits_to_print = (length < N_WORD_BITS) ? length : N_WORD_BITS;
+        uword_t word = ptr[1];
+        int i;
+        for(i=0; i<bits_to_print; ++i) {
+            putchar((word & 1) ? '1' : '0');
+            if ((i%8)==7) putchar('_');
+            word >>= 1;
+        }
+        if(bits_to_print < length) printf("...");
+        printf("\n");
         }
         break;
 
@@ -789,11 +815,11 @@ static void print_fun_or_otherptr(lispobj obj)
         print_obj("entry: ", fdefn_callee_lispobj((struct fdefn*)(ptr-1)));
         break;
 
-    // Make certain vectors printable from C for when all hell breaks lose
+    // Make some vectors printable from C, for when all hell breaks lose
     case SIMPLE_ARRAY_UNSIGNED_BYTE_32_WIDETAG:
         NEWLINE_OR_RETURN;
         {
-        long length = fixnum_value(*ptr);
+        long length = vector_len(VECTOR(obj));
         uint32_t * data = (uint32_t*)(ptr + 1);
         long i;
         printf("#(");
@@ -806,9 +832,8 @@ static void print_fun_or_otherptr(lispobj obj)
         break;
     default:
         NEWLINE_OR_RETURN;
-        if (type >= SIMPLE_ARRAY_UNSIGNED_BYTE_2_WIDETAG &&
-            type <= SIMPLE_BIT_VECTOR_WIDETAG)
-            printf("length = %ld", (long)fixnum_value(*ptr));
+        if (specialized_vector_widetag_p(type))
+            printf("length = %"OBJ_FMTd, vector_len(VECTOR(obj)));
         else
             printf("Unknown header object?");
         break;
@@ -885,7 +910,8 @@ void print(lispobj obj)
     max_depth = 5;
     max_lines = 20;
 
-    print_obj("", obj);
+    if (!setjmp(ldb_print_nlx))
+        print_obj("", obj);
 
     putchar('\n');
 }
@@ -908,27 +934,36 @@ void brief_print(lispobj obj)
 
 #include "forwarding-ptr.h"
 #include "genesis/classoid.h"
-struct vector * symbol_name(lispobj * sym)
+struct vector * symbol_name(struct symbol* sym)
 {
-  if (forwarding_pointer_p(sym))
-    sym = native_pointer(forwarding_pointer_value(sym));
-  if (lowtag_of(((struct symbol*)sym)->name) != OTHER_POINTER_LOWTAG)
-      return NULL;
-  return VECTOR(follow_maybe_fp(((struct symbol*)sym)->name));
+  if (forwarding_pointer_p((lispobj*)sym))
+    sym = (void*)native_pointer(forwarding_pointer_value((lispobj*)sym));
+  lispobj name = sym->name;
+  if (lowtag_of(name) != OTHER_POINTER_LOWTAG) return NULL;
+  lispobj string = decode_symbol_name(name);
+  return VECTOR(follow_maybe_fp(string));
 }
 struct vector * classoid_name(lispobj * classoid)
 {
   if (forwarding_pointer_p(classoid))
       classoid = native_pointer(forwarding_pointer_value(classoid));
+  // Classoids are named by symbols even though a CLASS name is arbitrary (theoretically)
   lispobj sym = ((struct classoid*)classoid)->name;
-  return lowtag_of(sym) != OTHER_POINTER_LOWTAG ? NULL
-    : symbol_name(native_pointer(sym));
+  return lowtag_of(sym) != OTHER_POINTER_LOWTAG ? NULL : symbol_name(SYMBOL(sym));
 }
 struct vector * layout_classoid_name(lispobj * layout)
 {
+#ifdef LISP_FEATURE_METASPACE
+  // layout can't be forwarded, but wrapper could be
+  lispobj* wrapper = native_pointer(((struct layout*)layout)->friend);
+  if (forwarding_pointer_p(wrapper))
+      wrapper = native_pointer(forwarding_pointer_value(wrapper));
+  lispobj classoid = ((struct wrapper*)wrapper)->classoid;
+#else
   if (forwarding_pointer_p(layout))
       layout = native_pointer(forwarding_pointer_value(layout));
   lispobj classoid = ((struct layout*)layout)->classoid;
+#endif
   return instancep(classoid) ? classoid_name(native_pointer(classoid)) : NULL;
 }
 struct vector * instance_classoid_name(lispobj * instance)
