@@ -22,7 +22,7 @@
 #include "interrupt.h"
 #include "interr.h"
 #include "breakpoint.h"
-#include "getallocptr.h"
+#include "pseudo-atomic.h"
 
 os_vm_address_t
 arch_get_bad_addr(int signam, siginfo_t *siginfo, os_context_t *context)
@@ -113,11 +113,27 @@ arch_handle_single_step_trap(os_context_t *context, int trap)
     arch_skip_instruction(context);
 }
 
-void
+int riscv_user_emulation;
+
+static void
 sigtrap_handler(int signal, siginfo_t *info, os_context_t *context)
 {
     uint32_t trap_instruction = *(uint32_t *)OS_CONTEXT_PC(context);
 
+    static int sigaction_workaround;
+    if (riscv_user_emulation && !sigaction_workaround) {
+        sigset_t curmask;
+        thread_sigmask(SIG_BLOCK, 0, &curmask);
+        if (*(unsigned long int*)&curmask == 0) {
+            char msg[] = "WARNING: broken sigaction() workaround enabled\n";
+            write(2, msg, sizeof msg-1);
+            sigaction_workaround = 1;
+        } else {
+            sigaction_workaround = -1;
+        }
+    }
+    if (sigaction_workaround == 1) thread_sigmask(SIG_BLOCK, &blockable_sigset, 0);
+
     if (trap_instruction != 0x100073) {
         lose("Unrecognized trap instruction %08x in sigtrap_handler()",
              trap_instruction);
@@ -138,13 +154,62 @@ arch_install_interrupt_handlers(void)
     ll_install_handler(SIGTRAP, sigtrap_handler);
 }
 
-/* Linkage table */
+/* Linkage tables
+ *
+ * Linkage entry size is 8 or 20, because we need 2 instructions for the 32-bit case and we need 3 instructions and an 8 byte address in the 64-bit case.
+ */
+
+#define LINKAGE_TEMP_REG reg_NL3
+
 void arch_write_linkage_table_entry(int index, void *target_addr, int datap)
 {
     // allocate successive entries downward
     char *reloc_addr =
         (char*)LINKAGE_TABLE_SPACE_END - (index + 1) * LINKAGE_TABLE_ENTRY_SIZE;
-    *(uword_t*)reloc_addr = (uword_t)target_addr;
+    if (datap) {
+      *(unsigned long *)reloc_addr = (unsigned long)target_addr;
+      return;
+    }
+    int* inst_ptr;
+    unsigned inst;
+
+    inst_ptr = (int*) reloc_addr;
+
+#ifndef LISP_FEATURE_64_BIT
+    /*
+      lui   reg, %hi(address)
+      jr    reg, %lo(address)
+    */
+    unsigned int addr = (unsigned int)target_addr;
+    unsigned int hi = ((addr + 0x800) >> 12);
+    int lo = addr - (hi << 12);
+
+    inst = 0x37 | LINKAGE_TEMP_REG << 7 | hi << 12;
+    *inst_ptr++ = inst;
+
+    inst = 0x67 | LINKAGE_TEMP_REG << 15 | lo << 20;
+    *inst_ptr++ = inst;
+#else
+    /*
+      auipc reg, 0
+      load reg, 12(reg)
+      jr  reg
+      address
+    */
+
+    inst = 0x17 | LINKAGE_TEMP_REG << 7;
+    *inst_ptr++ = inst;
+
+    inst = 0x3 | LINKAGE_TEMP_REG << 7 | WORD_SHIFT << 12 | LINKAGE_TEMP_REG << 15 | 12 << 20;
+    *inst_ptr++ = inst;
+
+    inst = 0x67 | LINKAGE_TEMP_REG << 15;
+    *inst_ptr++ = inst;
+
+    *(unsigned long *)inst_ptr++ = (unsigned long)target_addr;
+#endif
+
+    os_flush_icache((os_vm_address_t) reloc_addr, (char*) inst_ptr - reloc_addr);
 }
 
 lispobj call_into_lisp(lispobj fun, lispobj *args, int nargs) {
