@@ -35,7 +35,6 @@
 #include "gc-internal.h"
 #include "thread.h"
 #include "arch.h"
-#include "getallocptr.h"
 #include "genesis/static-symbols.h"
 #include "genesis/symbol.h"
 #include "genesis/vector.h"
@@ -254,13 +253,18 @@ void unwind_binding_stack()
     write_TLS(CURRENT_UNWIND_PROTECT_BLOCK, 0, th);
     unsigned int hint = 0;
     char symbol_name[] = "*SAVE-LISP-CLOBBERED-GLOBALS*";
-    lispobj* sym = find_symbol(symbol_name, sb_kernel_package(), &hint);
+    lispobj* sym = find_symbol(symbol_name,
+                               VECTOR(lisp_package_vector)->data[PACKAGE_ID_KERNEL],
+                               &hint);
     lispobj value;
     int i;
     if (!sym || !simple_vector_p(value = ((struct symbol*)sym)->value))
         fprintf(stderr, "warning: bad value in %s\n", symbol_name);
     else for(i=vector_len(VECTOR(value))-1; i>=0; --i)
         SYMBOL(VECTOR(value)->data[i])->value = UNBOUND_MARKER_WIDETAG;
+    // these are akin to weak-pointers
+    lisp_package_vector = 0;
+    alloc_profile_data = 0;
     if (verbose) printf("done]\n");
 }
 
@@ -330,8 +334,8 @@ save_to_filehandle(FILE *file, char *filename, lispobj init_function,
 #endif
     output_space(file,
                  DYNAMIC_CORE_SPACE_ID,
-                 current_dynamic_space,
-                 (lispobj *)get_alloc_pointer(),
+                 (void*)DYNAMIC_SPACE_START,
+                 (void*)dynamic_space_highwatermark(),
                  core_start_pos,
                  core_compression_level);
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
@@ -368,7 +372,8 @@ save_to_filehandle(FILE *file, char *filename, lispobj init_function,
         memset(data + aligned_size - N_WORD_BYTES, 0, N_WORD_BYTES);
         gc_store_corefile_ptes((struct corefile_pte*)data);
         write_lispobj(PAGE_TABLE_CORE_ENTRY_TYPE_CODE, file);
-        write_lispobj(5, file); // 5 = # of words in this core header entry
+        write_lispobj(6, file); // number of words in this core header entry
+        write_lispobj(gc_card_table_nbits, file);
         write_lispobj(next_free_page, file);
         write_lispobj(aligned_size, file);
         sword_t offset = write_bytes(file, data, aligned_size, core_start_pos,
